@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*********************************************************************
  * Copyright 2014,2015,2016,2017 Cumulus Networks, Inc.  All rights reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
- *
  * bfd.h: implements the BFD protocol.
  */
 
@@ -45,6 +32,13 @@ DECLARE_MGROUP(BFDD);
 DECLARE_MTYPE(BFDD_CONTROL);
 DECLARE_MTYPE(BFDD_NOTIFICATION);
 
+#define BFDD_SOCK_NAME "%s/bfdd.sock", frr_runstatedir
+
+/* bfd Authentication Type. */
+#define BFD_AUTH_NULL 0
+#define BFD_AUTH_SIMPLE 1
+#define BFD_AUTH_CRYPTOGRAPHIC 2
+
 struct bfd_timers {
 	uint32_t desired_min_tx;
 	uint32_t required_min_rx;
@@ -73,6 +67,15 @@ struct bfd_pkt {
 	struct bfd_timers timers;
 };
 
+/*
+ * Format of authentification.
+ */
+struct bfd_auth {
+	uint8_t type;
+	uint8_t length;
+};
+
+
 /*
  * Format of Echo packet.
  */
@@ -92,7 +95,7 @@ struct bfd_echo_pkt {
 
 
 /* Macros for manipulating control packets */
-#define BFD_VERMASK 0x03
+#define BFD_VERMASK 0x07
 #define BFD_DIAGMASK 0x1F
 #define BFD_GETVER(diag) ((diag >> 5) & BFD_VERMASK)
 #define BFD_SETVER(diag, val) ((diag) |= (val & BFD_VERMASK) << 5)
@@ -192,7 +195,7 @@ struct bfd_key {
 	uint16_t mhop;
 	struct in6_addr peer;
 	struct in6_addr local;
-	char ifname[INTERFACE_NAMSIZ];
+	char ifname[IFNAMSIZ];
 	char vrfname[VRF_NAMSIZ];
 } __attribute__((packed));
 
@@ -278,12 +281,12 @@ struct bfd_session {
 	struct bfd_config_timers timers;
 	struct bfd_timers cur_timers;
 	uint64_t detect_TO;
-	struct thread *echo_recvtimer_ev;
-	struct thread *recvtimer_ev;
+	struct event *echo_recvtimer_ev;
+	struct event *recvtimer_ev;
 	uint64_t xmt_TO;
 	uint64_t echo_xmt_TO;
-	struct thread *xmttimer_ev;
-	struct thread *echo_xmttimer_ev;
+	struct event *xmttimer_ev;
+	struct event *echo_xmttimer_ev;
 	uint64_t echo_detect_TO;
 
 	/* software object state */
@@ -414,8 +417,8 @@ struct bfd_control_socket {
 	TAILQ_ENTRY(bfd_control_socket) bcs_entry;
 
 	int bcs_sd;
-	struct thread *bcs_ev;
-	struct thread *bcs_outev;
+	struct event *bcs_ev;
+	struct event *bcs_outev;
 	struct bcqueue bcs_bcqueue;
 
 	/* Notification data */
@@ -435,7 +438,7 @@ int control_init(const char *path);
 void control_shutdown(void);
 int control_notify(struct bfd_session *bs, uint8_t notify_state);
 int control_notify_config(const char *op, struct bfd_session *bs);
-void control_accept(struct thread *t);
+void control_accept(struct event *t);
 
 
 /*
@@ -452,7 +455,7 @@ struct bfd_vrf_global {
 	int bg_echov6;
 	struct vrf *vrf;
 
-	struct thread *bg_ev[6];
+	struct event *bg_ev[6];
 };
 
 /* Forward declaration of data plane context struct. */
@@ -461,7 +464,7 @@ TAILQ_HEAD(dplane_queue, bfd_dplane_ctx);
 
 struct bfd_global {
 	int bg_csock;
-	struct thread *bg_csockev;
+	struct event *bg_csockev;
 	struct bcslist bg_bcslist;
 
 	struct pllist bg_pllist;
@@ -479,7 +482,7 @@ struct bfd_global {
 	/* Distributed BFD items. */
 	bool bg_use_dplane;
 	int bg_dplane_sock;
-	struct thread *bg_dplane_sockev;
+	struct event *bg_dplane_sockev;
 	struct dplane_queue bg_dplaneq;
 
 	/* Debug options. */
@@ -566,7 +569,7 @@ void ptm_bfd_snd(struct bfd_session *bfd, int fbit);
 void ptm_bfd_echo_snd(struct bfd_session *bfd);
 void ptm_bfd_echo_fp_snd(struct bfd_session *bfd);
 
-void bfd_recv_cb(struct thread *t);
+void bfd_recv_cb(struct event *t);
 
 
 /*
@@ -574,7 +577,7 @@ void bfd_recv_cb(struct thread *t);
  *
  * Contains the code related with event loop.
  */
-typedef void (*bfd_ev_cb)(struct thread *t);
+typedef void (*bfd_ev_cb)(struct event *t);
 
 void bfd_recvtimer_update(struct bfd_session *bs);
 void bfd_echo_recvtimer_update(struct bfd_session *bs);
@@ -699,12 +702,12 @@ void bfd_key_iterate(hash_iter_func hif, void *arg);
 unsigned long bfd_get_session_count(void);
 
 /* Export callback functions for `event.c`. */
-extern struct thread_master *master;
+extern struct event_loop *master;
 
-void bfd_recvtimer_cb(struct thread *t);
-void bfd_echo_recvtimer_cb(struct thread *t);
-void bfd_xmt_cb(struct thread *t);
-void bfd_echo_xmt_cb(struct thread *t);
+void bfd_recvtimer_cb(struct event *t);
+void bfd_echo_recvtimer_cb(struct event *t);
+void bfd_xmt_cb(struct event *t);
+void bfd_echo_xmt_cb(struct event *t);
 
 extern struct in6_addr zero_addr;
 
@@ -781,6 +784,7 @@ void bfdd_cli_init(void);
  */
 void bfdd_zclient_init(struct zebra_privs_t *bfdd_priv);
 void bfdd_zclient_stop(void);
+void bfdd_zclient_terminate(void);
 void bfdd_zclient_unregister(vrf_id_t vrf_id);
 void bfdd_zclient_register(vrf_id_t vrf_id);
 void bfdd_sessions_enable_vrf(struct vrf *vrf);
