@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* BGP open message handling
  * Copyright (C) 1998, 1999 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -23,7 +8,7 @@
 #include "linklist.h"
 #include "prefix.h"
 #include "stream.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "log.h"
 #include "command.h"
 #include "memory.h"
@@ -42,24 +27,23 @@
 #include "bgpd/bgp_vty.h"
 #include "bgpd/bgp_memory.h"
 
-static const struct message capcode_str[] = {
-	{CAPABILITY_CODE_MP, "MultiProtocol Extensions"},
-	{CAPABILITY_CODE_REFRESH, "Route Refresh"},
-	{CAPABILITY_CODE_ORF, "Cooperative Route Filtering"},
-	{CAPABILITY_CODE_RESTART, "Graceful Restart"},
-	{CAPABILITY_CODE_AS4, "4-octet AS number"},
-	{CAPABILITY_CODE_ADDPATH, "AddPath"},
-	{CAPABILITY_CODE_DYNAMIC, "Dynamic"},
-	{CAPABILITY_CODE_ENHE, "Extended Next Hop Encoding"},
-	{CAPABILITY_CODE_DYNAMIC_OLD, "Dynamic (Old)"},
-	{CAPABILITY_CODE_REFRESH_OLD, "Route Refresh (Old)"},
-	{CAPABILITY_CODE_ORF_OLD, "ORF (Old)"},
-	{CAPABILITY_CODE_FQDN, "FQDN"},
-	{CAPABILITY_CODE_ENHANCED_RR, "Enhanced Route Refresh"},
-	{CAPABILITY_CODE_EXT_MESSAGE, "BGP Extended Message"},
-	{CAPABILITY_CODE_LLGR, "Long-lived BGP Graceful Restart"},
-	{CAPABILITY_CODE_ROLE, "Role"},
-	{0}};
+const struct message capcode_str[] = {
+	{ CAPABILITY_CODE_MP, "MultiProtocol Extensions" },
+	{ CAPABILITY_CODE_REFRESH, "Route Refresh" },
+	{ CAPABILITY_CODE_ORF, "Cooperative Route Filtering" },
+	{ CAPABILITY_CODE_RESTART, "Graceful Restart" },
+	{ CAPABILITY_CODE_AS4, "4-octet AS number" },
+	{ CAPABILITY_CODE_ADDPATH, "AddPath" },
+	{ CAPABILITY_CODE_DYNAMIC, "Dynamic" },
+	{ CAPABILITY_CODE_ENHE, "Extended Next Hop Encoding" },
+	{ CAPABILITY_CODE_FQDN, "FQDN" },
+	{ CAPABILITY_CODE_ENHANCED_RR, "Enhanced Route Refresh" },
+	{ CAPABILITY_CODE_EXT_MESSAGE, "BGP Extended Message" },
+	{ CAPABILITY_CODE_LLGR, "Long-lived BGP Graceful Restart" },
+	{ CAPABILITY_CODE_ROLE, "Role" },
+	{ CAPABILITY_CODE_SOFT_VERSION, "Software Version" },
+	{ 0 }
+};
 
 /* Minimum sizes for length field of each cap (so not inc. the header) */
 static const size_t cap_minsizes[] = {
@@ -70,15 +54,13 @@ static const size_t cap_minsizes[] = {
 		[CAPABILITY_CODE_AS4] = CAPABILITY_CODE_AS4_LEN,
 		[CAPABILITY_CODE_ADDPATH] = CAPABILITY_CODE_ADDPATH_LEN,
 		[CAPABILITY_CODE_DYNAMIC] = CAPABILITY_CODE_DYNAMIC_LEN,
-		[CAPABILITY_CODE_DYNAMIC_OLD] = CAPABILITY_CODE_DYNAMIC_LEN,
 		[CAPABILITY_CODE_ENHE] = CAPABILITY_CODE_ENHE_LEN,
-		[CAPABILITY_CODE_REFRESH_OLD] = CAPABILITY_CODE_REFRESH_LEN,
-		[CAPABILITY_CODE_ORF_OLD] = CAPABILITY_CODE_ORF_LEN,
 		[CAPABILITY_CODE_FQDN] = CAPABILITY_CODE_MIN_FQDN_LEN,
 		[CAPABILITY_CODE_ENHANCED_RR] = CAPABILITY_CODE_ENHANCED_LEN,
 		[CAPABILITY_CODE_EXT_MESSAGE] = CAPABILITY_CODE_EXT_MESSAGE_LEN,
 		[CAPABILITY_CODE_LLGR] = CAPABILITY_CODE_LLGR_LEN,
 		[CAPABILITY_CODE_ROLE] = CAPABILITY_CODE_ROLE_LEN,
+		[CAPABILITY_CODE_SOFT_VERSION] = CAPABILITY_CODE_SOFT_VERSION_LEN,
 };
 
 /* value the capability must be a multiple of.
@@ -94,15 +76,13 @@ static const size_t cap_modsizes[] = {
 		[CAPABILITY_CODE_AS4] = 4,
 		[CAPABILITY_CODE_ADDPATH] = 4,
 		[CAPABILITY_CODE_DYNAMIC] = 1,
-		[CAPABILITY_CODE_DYNAMIC_OLD] = 1,
 		[CAPABILITY_CODE_ENHE] = 6,
-		[CAPABILITY_CODE_REFRESH_OLD] = 1,
-		[CAPABILITY_CODE_ORF_OLD] = 1,
 		[CAPABILITY_CODE_FQDN] = 1,
 		[CAPABILITY_CODE_ENHANCED_RR] = 1,
 		[CAPABILITY_CODE_EXT_MESSAGE] = 1,
 		[CAPABILITY_CODE_LLGR] = 1,
 		[CAPABILITY_CODE_ROLE] = 1,
+		[CAPABILITY_CODE_SOFT_VERSION] = 1,
 };
 
 /* BGP-4 Multiprotocol Extentions lead us to the complex world. We can
@@ -167,7 +147,8 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, bool use_json,
 						"capabilityErrorMultiProtocolAfi",
 						"L2VPN");
 					break;
-				default:
+				case AFI_UNSPEC:
+				case AFI_MAX:
 					json_object_int_add(
 						json_cap,
 						"capabilityErrorMultiProtocolAfiUnknown",
@@ -217,7 +198,8 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, bool use_json,
 						"capabilityErrorMultiProtocolSafi",
 						"flowspec");
 					break;
-				default:
+				case SAFI_UNSPEC:
+				case SAFI_MAX:
 					json_object_int_add(
 						json_cap,
 						"capabilityErrorMultiProtocolSafiUnknown",
@@ -237,7 +219,8 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, bool use_json,
 				case AFI_L2VPN:
 					vty_out(vty, "AFI L2VPN, ");
 					break;
-				default:
+				case AFI_UNSPEC:
+				case AFI_MAX:
 					vty_out(vty, "AFI Unknown %d, ",
 						ntohs(mpc.afi));
 					break;
@@ -264,7 +247,8 @@ void bgp_capability_vty_out(struct vty *vty, struct peer *peer, bool use_json,
 				case SAFI_EVPN:
 					vty_out(vty, "SAFI EVPN");
 					break;
-				default:
+				case SAFI_UNSPEC:
+				case SAFI_MAX:
 					vty_out(vty, "SAFI Unknown %d ",
 						mpc.safi);
 					break;
@@ -356,15 +340,14 @@ static void bgp_capability_orf_not_support(struct peer *peer, iana_afi_t afi,
 			peer->host, afi, safi, type, mode);
 }
 
-static const struct message orf_type_str[] = {
-	{ORF_TYPE_PREFIX, "Prefixlist"},
-	{ORF_TYPE_PREFIX_OLD, "Prefixlist (old)"},
-	{0}};
+const struct message orf_type_str[] = { { ORF_TYPE_RESERVED, "Reserved" },
+					{ ORF_TYPE_PREFIX, "Prefixlist" },
+					{ 0 } };
 
-static const struct message orf_mode_str[] = {{ORF_MODE_RECEIVE, "Receive"},
-					      {ORF_MODE_SEND, "Send"},
-					      {ORF_MODE_BOTH, "Both"},
-					      {0}};
+const struct message orf_mode_str[] = { { ORF_MODE_RECEIVE, "Receive" },
+					{ ORF_MODE_SEND, "Send" },
+					{ ORF_MODE_BOTH, "Both" },
+					{ 0 } };
 
 static int bgp_capability_orf_entry(struct peer *peer,
 				    struct capability_header *hdr)
@@ -408,7 +391,7 @@ static int bgp_capability_orf_entry(struct peer *peer,
 		zlog_info(
 			"%s ORF Capability entry length error, Cap length %u, num %u",
 			peer->host, hdr->length, num);
-		bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+		bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 				BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 		return -1;
 	}
@@ -433,17 +416,13 @@ static int bgp_capability_orf_entry(struct peer *peer,
 		switch (hdr->code) {
 		case CAPABILITY_CODE_ORF:
 			switch (type) {
-			case ORF_TYPE_PREFIX:
+			case ORF_TYPE_RESERVED:
+				if (bgp_debug_neighbor_events(peer))
+					zlog_debug(
+						"%s Addr-family %d/%d has reserved ORF type, ignoring",
+						peer->host, afi, safi);
 				break;
-			default:
-				bgp_capability_orf_not_support(
-					peer, pkt_afi, pkt_safi, type, mode);
-				continue;
-			}
-			break;
-		case CAPABILITY_CODE_ORF_OLD:
-			switch (type) {
-			case ORF_TYPE_PREFIX_OLD:
+			case ORF_TYPE_PREFIX:
 				break;
 			default:
 				bgp_capability_orf_not_support(
@@ -477,9 +456,6 @@ static int bgp_capability_orf_entry(struct peer *peer,
 		if (hdr->code == CAPABILITY_CODE_ORF) {
 			sm_cap = PEER_CAP_ORF_PREFIX_SM_RCV;
 			rm_cap = PEER_CAP_ORF_PREFIX_RM_RCV;
-		} else if (hdr->code == CAPABILITY_CODE_ORF_OLD) {
-			sm_cap = PEER_CAP_ORF_PREFIX_SM_OLD_RCV;
-			rm_cap = PEER_CAP_ORF_PREFIX_RM_OLD_RCV;
 		} else {
 			bgp_capability_orf_not_support(peer, pkt_afi, pkt_safi,
 						       type, mode);
@@ -599,18 +575,6 @@ static int bgp_capability_restart(struct peer *peer,
 static int bgp_capability_llgr(struct peer *peer,
 			       struct capability_header *caphdr)
 {
-/*
- * +--------------------------------------------------+
- * | Address Family Identifier (16 bits)              |
- * +--------------------------------------------------+
- * | Subsequent Address Family Identifier (8 bits)    |
- * +--------------------------------------------------+
- * | Flags for Address Family (8 bits)                |
- * +--------------------------------------------------+
- * | Long-lived Stale Time (24 bits)                  |
- * +--------------------------------------------------+
- */
-#define BGP_CAP_LLGR_MIN_PACKET_LEN 7
 	struct stream *s = BGP_INPUT(peer);
 	size_t end = stream_get_getp(s) + caphdr->length;
 
@@ -658,17 +622,17 @@ static int bgp_capability_llgr(struct peer *peer,
 /* Unlike other capability parsing routines, this one returns 0 on error */
 static as_t bgp_capability_as4(struct peer *peer, struct capability_header *hdr)
 {
-	SET_FLAG(peer->cap, PEER_CAP_AS4_RCV);
-
 	if (hdr->length != CAPABILITY_CODE_AS4_LEN) {
 		flog_err(EC_BGP_PKT_OPEN,
 			 "%s AS4 capability has incorrect data length %d",
 			 peer->host, hdr->length);
-		return 0;
+		return -1;
 	}
 
 	as_t as4 = stream_getl(BGP_INPUT(peer));
 
+	SET_FLAG(peer->cap, PEER_CAP_AS4_RCV);
+
 	if (BGP_DEBUG(as4, AS4))
 		zlog_debug(
 			"%s [AS4] about to set cap PEER_CAP_AS4_RCV, got as4 %u",
@@ -698,10 +662,8 @@ static int bgp_capability_addpath(struct peer *peer,
 	struct stream *s = BGP_INPUT(peer);
 	size_t end = stream_get_getp(s) + hdr->length;
 
-	SET_FLAG(peer->cap, PEER_CAP_ADDPATH_RCV);
-
 	/* Verify length is a multiple of 4 */
-	if (hdr->length % 4) {
+	if (hdr->length % CAPABILITY_CODE_ADDPATH_LEN) {
 		flog_warn(
 			EC_BGP_CAPABILITY_INVALID_LENGTH,
 			"Add Path: Received invalid length %d, non-multiple of 4",
@@ -709,23 +671,38 @@ static int bgp_capability_addpath(struct peer *peer,
 		return -1;
 	}
 
-	while (stream_get_getp(s) + 4 <= end) {
+	SET_FLAG(peer->cap, PEER_CAP_ADDPATH_RCV);
+
+	while (stream_get_getp(s) + CAPABILITY_CODE_ADDPATH_LEN <= end) {
 		afi_t afi;
 		safi_t safi;
 		iana_afi_t pkt_afi = stream_getw(s);
 		iana_safi_t pkt_safi = stream_getc(s);
 		uint8_t send_receive = stream_getc(s);
 
+		/* If any other value (other than 1-3) is received, then
+		 * the capability SHOULD be treated as not understood
+		 * and ignored.
+		 */
+		if (!send_receive || send_receive > 3) {
+			flog_warn(EC_BGP_CAPABILITY_INVALID_DATA,
+				  "Add Path: Received invalid send/receive value %u in Add Path capability",
+				  send_receive);
+			continue;
+		}
+
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug(
-				"%s OPEN has %s capability for afi/safi: %s/%s%s%s",
-				peer->host,
-				lookup_msg(capcode_str, hdr->code, NULL),
-				iana_afi2str(pkt_afi), iana_safi2str(pkt_safi),
-				(send_receive & BGP_ADDPATH_RX) ? ", receive"
-								: "",
-				(send_receive & BGP_ADDPATH_TX) ? ", transmit"
-								: "");
+			zlog_debug("%s OPEN has %s capability for afi/safi: %s/%s%s%s",
+				   peer->host,
+				   lookup_msg(capcode_str, hdr->code, NULL),
+				   iana_afi2str(pkt_afi),
+				   iana_safi2str(pkt_safi),
+				   CHECK_FLAG(send_receive, BGP_ADDPATH_RX)
+					   ? ", receive"
+					   : "",
+				   CHECK_FLAG(send_receive, BGP_ADDPATH_TX)
+					   ? ", transmit"
+					   : "");
 
 		/* Convert AFI, SAFI to internal values, check. */
 		if (bgp_map_afi_safi_iana2int(pkt_afi, pkt_safi, &afi, &safi)) {
@@ -744,13 +721,19 @@ static int bgp_capability_addpath(struct peer *peer,
 			continue;
 		}
 
-		if (send_receive & BGP_ADDPATH_RX)
+		if (CHECK_FLAG(send_receive, BGP_ADDPATH_RX))
 			SET_FLAG(peer->af_cap[afi][safi],
 				 PEER_CAP_ADDPATH_AF_RX_RCV);
+		else
+			UNSET_FLAG(peer->af_cap[afi][safi],
+				   PEER_CAP_ADDPATH_AF_RX_RCV);
 
-		if (send_receive & BGP_ADDPATH_TX)
+		if (CHECK_FLAG(send_receive, BGP_ADDPATH_TX))
 			SET_FLAG(peer->af_cap[afi][safi],
 				 PEER_CAP_ADDPATH_AF_TX_RCV);
+		else
+			UNSET_FLAG(peer->af_cap[afi][safi],
+				   PEER_CAP_ADDPATH_AF_TX_RCV);
 	}
 
 	return 0;
@@ -835,8 +818,6 @@ static int bgp_capability_hostname(struct peer *peer,
 	size_t end = stream_get_getp(s) + hdr->length;
 	uint8_t len;
 
-	SET_FLAG(peer->cap, PEER_CAP_HOSTNAME_RCV);
-
 	len = stream_getc(s);
 	if (stream_get_getp(s) + len > end) {
 		flog_warn(
@@ -894,6 +875,8 @@ static int bgp_capability_hostname(struct peer *peer,
 		peer->domainname = XSTRDUP(MTYPE_BGP_PEER_HOST, str);
 	}
 
+	SET_FLAG(peer->cap, PEER_CAP_HOSTNAME_RCV);
+
 	if (bgp_debug_neighbor_events(peer)) {
 		zlog_debug("%s received hostname %s, domainname %s", peer->host,
 			   peer->hostname, peer->domainname);
@@ -904,18 +887,65 @@ static int bgp_capability_hostname(struct peer *peer,
 
 static int bgp_capability_role(struct peer *peer, struct capability_header *hdr)
 {
-	SET_FLAG(peer->cap, PEER_CAP_ROLE_RCV);
 	if (hdr->length != CAPABILITY_CODE_ROLE_LEN) {
 		flog_warn(EC_BGP_CAPABILITY_INVALID_LENGTH,
 			  "Role: Received invalid length %d", hdr->length);
 		return -1;
 	}
+
 	uint8_t role = stream_getc(BGP_INPUT(peer));
 
+	SET_FLAG(peer->cap, PEER_CAP_ROLE_RCV);
+
 	peer->remote_role = role;
 	return 0;
 }
 
+static int bgp_capability_software_version(struct peer *peer,
+					   struct capability_header *hdr)
+{
+	struct stream *s = BGP_INPUT(peer);
+	char str[BGP_MAX_SOFT_VERSION + 1];
+	size_t end = stream_get_getp(s) + hdr->length;
+	uint8_t len;
+
+	len = stream_getc(s);
+	if (stream_get_getp(s) + len > end) {
+		flog_warn(
+			EC_BGP_CAPABILITY_INVALID_DATA,
+			"%s: Received malformed Software Version capability from peer %s",
+			__func__, peer->host);
+		return -1;
+	}
+
+	SET_FLAG(peer->cap, PEER_CAP_SOFT_VERSION_RCV);
+
+	if (len > BGP_MAX_SOFT_VERSION) {
+		flog_warn(EC_BGP_CAPABILITY_INVALID_LENGTH,
+			  "%s: Received Software Version, but the length is too big, truncating, from peer %s",
+			  __func__, peer->host);
+		stream_get(str, s, BGP_MAX_SOFT_VERSION);
+		stream_forward_getp(s, len - BGP_MAX_SOFT_VERSION);
+		len = BGP_MAX_SOFT_VERSION;
+	} else if (len) {
+		stream_get(str, s, len);
+	}
+
+	if (len) {
+		str[len] = '\0';
+
+		XFREE(MTYPE_BGP_SOFT_VERSION, peer->soft_version);
+
+		peer->soft_version = XSTRDUP(MTYPE_BGP_SOFT_VERSION, str);
+
+		if (bgp_debug_neighbor_events(peer))
+			zlog_debug("%s received Software Version: %s",
+				   peer->host, peer->soft_version);
+	}
+
+	return 0;
+}
+
 /**
  * Parse given capability.
  * XXX: This is reading into a stream, but not using stream API
@@ -943,7 +973,7 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 		if (stream_get_getp(s) + 2 > end) {
 			zlog_info("%s Capability length error (< header)",
 				  peer->host);
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 			return -1;
 		}
@@ -956,7 +986,7 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 		if (start + caphdr.length > end) {
 			zlog_info("%s Capability length error (< length)",
 				  peer->host);
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 			return -1;
 		}
@@ -971,19 +1001,17 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 		switch (caphdr.code) {
 		case CAPABILITY_CODE_MP:
 		case CAPABILITY_CODE_REFRESH:
-		case CAPABILITY_CODE_REFRESH_OLD:
 		case CAPABILITY_CODE_ORF:
-		case CAPABILITY_CODE_ORF_OLD:
 		case CAPABILITY_CODE_RESTART:
 		case CAPABILITY_CODE_AS4:
 		case CAPABILITY_CODE_ADDPATH:
 		case CAPABILITY_CODE_DYNAMIC:
-		case CAPABILITY_CODE_DYNAMIC_OLD:
 		case CAPABILITY_CODE_ENHE:
 		case CAPABILITY_CODE_FQDN:
 		case CAPABILITY_CODE_ENHANCED_RR:
 		case CAPABILITY_CODE_EXT_MESSAGE:
 		case CAPABILITY_CODE_ROLE:
+		case CAPABILITY_CODE_SOFT_VERSION:
 			/* Check length. */
 			if (caphdr.length < cap_minsizes[caphdr.code]) {
 				zlog_info(
@@ -993,7 +1021,8 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 						   NULL),
 					caphdr.length,
 					(unsigned)cap_minsizes[caphdr.code]);
-				bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+				bgp_notify_send(peer->connection,
+						BGP_NOTIFY_OPEN_ERR,
 						BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 				return -1;
 			}
@@ -1006,10 +1035,12 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 						   NULL),
 					caphdr.length,
 					(unsigned)cap_modsizes[caphdr.code]);
-				bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+				bgp_notify_send(peer->connection,
+						BGP_NOTIFY_OPEN_ERR,
 						BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 				return -1;
 			}
+			break;
 		/* we deliberately ignore unknown codes, see below */
 		default:
 			break;
@@ -1035,18 +1066,14 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 			}
 		} break;
 		case CAPABILITY_CODE_ENHANCED_RR:
-		case CAPABILITY_CODE_REFRESH:
-		case CAPABILITY_CODE_REFRESH_OLD: {
+		case CAPABILITY_CODE_REFRESH: {
 			/* BGP refresh capability */
 			if (caphdr.code == CAPABILITY_CODE_ENHANCED_RR)
 				SET_FLAG(peer->cap, PEER_CAP_ENHANCED_RR_RCV);
-			else if (caphdr.code == CAPABILITY_CODE_REFRESH_OLD)
-				SET_FLAG(peer->cap, PEER_CAP_REFRESH_OLD_RCV);
 			else
-				SET_FLAG(peer->cap, PEER_CAP_REFRESH_NEW_RCV);
+				SET_FLAG(peer->cap, PEER_CAP_REFRESH_RCV);
 		} break;
 		case CAPABILITY_CODE_ORF:
-		case CAPABILITY_CODE_ORF_OLD:
 			ret = bgp_capability_orf_entry(peer, &caphdr);
 			break;
 		case CAPABILITY_CODE_RESTART:
@@ -1056,7 +1083,6 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 			ret = bgp_capability_llgr(peer, &caphdr);
 			break;
 		case CAPABILITY_CODE_DYNAMIC:
-		case CAPABILITY_CODE_DYNAMIC_OLD:
 			SET_FLAG(peer->cap, PEER_CAP_DYNAMIC_RCV);
 			break;
 		case CAPABILITY_CODE_AS4:
@@ -1084,6 +1110,9 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 		case CAPABILITY_CODE_ROLE:
 			ret = bgp_capability_role(peer, &caphdr);
 			break;
+		case CAPABILITY_CODE_SOFT_VERSION:
+			ret = bgp_capability_software_version(peer, &caphdr);
+			break;
 		default:
 			if (caphdr.code > 128) {
 				/* We don't send Notification for unknown vendor
@@ -1104,7 +1133,7 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 		}
 
 		if (ret < 0) {
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 			return -1;
 		}
@@ -1128,13 +1157,6 @@ static int bgp_capability_parse(struct peer *peer, size_t length,
 	return 0;
 }
 
-static int bgp_auth_parse(struct peer *peer, size_t length)
-{
-	bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
-			BGP_NOTIFY_OPEN_AUTH_FAILURE);
-	return -1;
-}
-
 static bool strict_capability_same(struct peer *peer)
 {
 	int i, j;
@@ -1159,7 +1181,7 @@ static bool bgp_role_violation(struct peer *peer)
 	      (local_role == ROLE_RS_SERVER && remote_role == ROLE_RS_CLIENT) ||
 	      (local_role == ROLE_RS_CLIENT &&
 	       remote_role == ROLE_RS_SERVER))) {
-		bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+		bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 				BGP_NOTIFY_OPEN_ROLE_MISMATCH);
 		return true;
 	}
@@ -1167,7 +1189,7 @@ static bool bgp_role_violation(struct peer *peer)
 	    CHECK_FLAG(peer->flags, PEER_FLAG_ROLE_STRICT_MODE)) {
 		const char *err_msg =
 			"Strict mode. Please set the role on your side.";
-		bgp_notify_send_with_data(peer, BGP_NOTIFY_OPEN_ERR,
+		bgp_notify_send_with_data(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					  BGP_NOTIFY_OPEN_ROLE_MISMATCH,
 					  (uint8_t *)err_msg, strlen(err_msg));
 		return true;
@@ -1295,7 +1317,7 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 		 */
 		if (STREAM_READABLE(s) < 1) {
 			zlog_info("%s Option length error", peer->host);
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 			return -1;
 		}
@@ -1308,7 +1330,8 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 		if (BGP_OPEN_EXT_OPT_PARAMS_CAPABLE(peer)) {
 			if (STREAM_READABLE(s) < 2) {
 				zlog_info("%s Option length error", peer->host);
-				bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+				bgp_notify_send(peer->connection,
+						BGP_NOTIFY_OPEN_ERR,
 						BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 				return -1;
 			}
@@ -1317,7 +1340,8 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 		} else {
 			if (STREAM_READABLE(s) < 1) {
 				zlog_info("%s Option length error", peer->host);
-				bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+				bgp_notify_send(peer->connection,
+						BGP_NOTIFY_OPEN_ERR,
 						BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 				return -1;
 			}
@@ -1329,7 +1353,7 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 		if (STREAM_READABLE(s) < opt_length) {
 			zlog_info("%s Option length error (%d)", peer->host,
 				  opt_length);
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_MALFORMED_ATTR);
 			return -1;
 		}
@@ -1338,23 +1362,17 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 			zlog_debug(
 				"%s rcvd OPEN w/ optional parameter type %u (%s) len %u",
 				peer->host, opt_type,
-				opt_type == BGP_OPEN_OPT_AUTH
-					? "Authentication"
-					: opt_type == BGP_OPEN_OPT_CAP
-						  ? "Capability"
-						  : "Unknown",
+				opt_type == BGP_OPEN_OPT_CAP ? "Capability"
+							     : "Unknown",
 				opt_length);
 
 		switch (opt_type) {
-		case BGP_OPEN_OPT_AUTH:
-			ret = bgp_auth_parse(peer, opt_length);
-			break;
 		case BGP_OPEN_OPT_CAP:
 			ret = bgp_capability_parse(peer, opt_length,
 						   mp_capability, &error);
 			break;
 		default:
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_UNSUP_PARAM);
 			ret = -1;
 			break;
@@ -1373,7 +1391,8 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 	if (CHECK_FLAG(peer->flags, PEER_FLAG_STRICT_CAP_MATCH)) {
 		/* If Unsupported Capability exists. */
 		if (error != error_data) {
-			bgp_notify_send_with_data(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send_with_data(peer->connection,
+						  BGP_NOTIFY_OPEN_ERR,
 						  BGP_NOTIFY_OPEN_UNSUP_CAPBL,
 						  error_data,
 						  error - error_data);
@@ -1383,7 +1402,7 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 		/* Check local capability does not negotiated with remote
 		   peer. */
 		if (!strict_capability_same(peer)) {
-			bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+			bgp_notify_send(peer->connection, BGP_NOTIFY_OPEN_ERR,
 					BGP_NOTIFY_OPEN_UNSUP_CAPBL);
 			return -1;
 		}
@@ -1422,12 +1441,14 @@ int bgp_open_option_parse(struct peer *peer, uint16_t length,
 				 peer->host);
 
 			if (error != error_data)
-				bgp_notify_send_with_data(
-					peer, BGP_NOTIFY_OPEN_ERR,
-					BGP_NOTIFY_OPEN_UNSUP_CAPBL, error_data,
-					error - error_data);
+				bgp_notify_send_with_data(peer->connection,
+							  BGP_NOTIFY_OPEN_ERR,
+							  BGP_NOTIFY_OPEN_UNSUP_CAPBL,
+							  error_data,
+							  error - error_data);
 			else
-				bgp_notify_send(peer, BGP_NOTIFY_OPEN_ERR,
+				bgp_notify_send(peer->connection,
+						BGP_NOTIFY_OPEN_ERR,
 						BGP_NOTIFY_OPEN_UNSUP_CAPBL);
 			return -1;
 		}
@@ -1467,9 +1488,7 @@ static void bgp_open_capability_orf(struct stream *s, struct peer *peer,
 	/* Address Prefix ORF */
 	if (CHECK_FLAG(peer->af_flags[afi][safi], PEER_FLAG_ORF_PREFIX_SM)
 	    || CHECK_FLAG(peer->af_flags[afi][safi], PEER_FLAG_ORF_PREFIX_RM)) {
-		stream_putc(s, (code == CAPABILITY_CODE_ORF
-					? ORF_TYPE_PREFIX
-					: ORF_TYPE_PREFIX_OLD));
+		stream_putc(s, ORF_TYPE_PREFIX);
 
 		if (CHECK_FLAG(peer->af_flags[afi][safi],
 			       PEER_FLAG_ORF_PREFIX_SM)
@@ -1657,7 +1676,7 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 	iana_safi_t pkt_safi = IANA_SAFI_UNICAST;
 	as_t local_as;
 	uint8_t afi_safi_count = 0;
-	int adv_addpath_tx = 0;
+	bool adv_addpath_tx = false;
 
 	/* Non-Ext OP Len. */
 	cp = stream_get_endp(s);
@@ -1702,11 +1721,11 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 			 * supporting RFC-5549 for
 			 * Link-Local peering only
 			 */
-			if (CHECK_FLAG(peer->flags, PEER_FLAG_CAPABILITY_ENHE)
-			    && peer->su.sa.sa_family == AF_INET6
-			    && afi == AFI_IP
-			    && (safi == SAFI_UNICAST || safi == SAFI_MPLS_VPN
-				|| safi == SAFI_LABELED_UNICAST)) {
+			if (CHECK_FLAG(peer->flags, PEER_FLAG_CAPABILITY_ENHE) &&
+			    peer->connection->su.sa.sa_family == AF_INET6 &&
+			    afi == AFI_IP &&
+			    (safi == SAFI_UNICAST || safi == SAFI_MPLS_VPN ||
+			     safi == SAFI_LABELED_UNICAST)) {
 				/* RFC 5549 Extended Next Hop Encoding
 				 */
 				SET_FLAG(peer->cap, PEER_CAP_ENHE_ADV);
@@ -1738,11 +1757,6 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 	/* Route refresh. */
 	SET_FLAG(peer->cap, PEER_CAP_REFRESH_ADV);
 	stream_putc(s, BGP_OPEN_OPT_CAP);
-	ext_opt_params ? stream_putw(s, CAPABILITY_CODE_REFRESH_LEN + 2)
-		       : stream_putc(s, CAPABILITY_CODE_REFRESH_LEN + 2);
-	stream_putc(s, CAPABILITY_CODE_REFRESH_OLD);
-	stream_putc(s, CAPABILITY_CODE_REFRESH_LEN);
-	stream_putc(s, BGP_OPEN_OPT_CAP);
 	ext_opt_params ? stream_putw(s, CAPABILITY_CODE_REFRESH_LEN + 2)
 		       : stream_putc(s, CAPABILITY_CODE_REFRESH_LEN + 2);
 	stream_putc(s, CAPABILITY_CODE_REFRESH);
@@ -1796,7 +1810,17 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 			 * will use it is
 			 * configured */
 			if (peer->addpath_type[afi][safi] != BGP_ADDPATH_NONE)
-				adv_addpath_tx = 1;
+				adv_addpath_tx = true;
+
+			/* If we have enabled labeled unicast, we MUST check
+			 * against unicast SAFI because addpath IDs are
+			 * allocated under unicast SAFI, the same as the RIB
+			 * is managed in unicast SAFI.
+			 */
+			if (safi == SAFI_LABELED_UNICAST)
+				if (peer->addpath_type[afi][SAFI_UNICAST] !=
+				    BGP_ADDPATH_NONE)
+					adv_addpath_tx = true;
 		}
 	}
 
@@ -1837,6 +1861,10 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 				SET_FLAG(flags, BGP_ADDPATH_TX);
 				SET_FLAG(peer->af_cap[afi][safi],
 					 PEER_CAP_ADDPATH_AF_TX_ADV);
+				if (safi == SAFI_LABELED_UNICAST)
+					SET_FLAG(
+						peer->af_cap[afi][SAFI_UNICAST],
+						PEER_CAP_ADDPATH_AF_TX_ADV);
 			} else {
 				UNSET_FLAG(peer->af_cap[afi][safi],
 					   PEER_CAP_ADDPATH_AF_TX_ADV);
@@ -1852,9 +1880,6 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 			       PEER_FLAG_ORF_PREFIX_SM)
 		    || CHECK_FLAG(peer->af_flags[afi][safi],
 				  PEER_FLAG_ORF_PREFIX_RM)) {
-			bgp_open_capability_orf(s, peer, afi, safi,
-						CAPABILITY_CODE_ORF_OLD,
-						ext_opt_params);
 			bgp_open_capability_orf(s, peer, afi, safi,
 						CAPABILITY_CODE_ORF,
 						ext_opt_params);
@@ -1865,12 +1890,6 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 	if (CHECK_FLAG(peer->flags, PEER_FLAG_DYNAMIC_CAPABILITY)) {
 		SET_FLAG(peer->cap, PEER_CAP_DYNAMIC_ADV);
 		stream_putc(s, BGP_OPEN_OPT_CAP);
-		ext_opt_params
-			? stream_putw(s, CAPABILITY_CODE_DYNAMIC_LEN + 2)
-			: stream_putc(s, CAPABILITY_CODE_DYNAMIC_LEN + 2);
-		stream_putc(s, CAPABILITY_CODE_DYNAMIC_OLD);
-		stream_putc(s, CAPABILITY_CODE_DYNAMIC_LEN);
-		stream_putc(s, BGP_OPEN_OPT_CAP);
 		ext_opt_params
 			? stream_putw(s, CAPABILITY_CODE_DYNAMIC_LEN + 2)
 			: stream_putc(s, CAPABILITY_CODE_DYNAMIC_LEN + 2);
@@ -1878,8 +1897,9 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 		stream_putc(s, CAPABILITY_CODE_DYNAMIC_LEN);
 	}
 
-	/* Hostname capability */
-	if (cmd_hostname_get()) {
+	/* FQDN capability */
+	if (CHECK_FLAG(peer->flags, PEER_FLAG_CAPABILITY_FQDN)
+	    && cmd_hostname_get()) {
 		SET_FLAG(peer->cap, PEER_CAP_HOSTNAME_ADV);
 		stream_putc(s, BGP_OPEN_OPT_CAP);
 		rcapp = stream_get_endp(s); /* Ptr to length placeholder */
@@ -1922,6 +1942,50 @@ uint16_t bgp_open_capability(struct stream *s, struct peer *peer,
 	bgp_peer_send_gr_capability(s, peer, ext_opt_params);
 	bgp_peer_send_llgr_capability(s, peer, ext_opt_params);
 
+	/* Software Version capability
+	 * An implementation is REQUIRED Extended Optional Parameters
+	 * Length for BGP OPEN Message support as defined in [RFC9072].
+	 * The inclusion of the Software Version Capability is OPTIONAL.
+	 * If an implementation supports the inclusion of the capability,
+	 * the implementation MUST include a configuration switch to enable
+	 * or disable its use, and that switch MUST be off by default.
+	 */
+	if (peergroup_flag_check(peer, PEER_FLAG_CAPABILITY_SOFT_VERSION) ||
+	    peer->sort == BGP_PEER_IBGP || peer->sub_sort == BGP_PEER_EBGP_OAD) {
+		SET_FLAG(peer->cap, PEER_CAP_SOFT_VERSION_ADV);
+		stream_putc(s, BGP_OPEN_OPT_CAP);
+		rcapp = stream_get_endp(s);
+		ext_opt_params ? stream_putw(s, 0)
+			       : stream_putc(s, 0); /* Capability Length */
+		stream_putc(s, CAPABILITY_CODE_SOFT_VERSION);
+		capp = stream_get_endp(s);
+		stream_putc(s, 0); /* dummy placeholder len */
+
+		/* The Capability Length SHOULD be no greater than 64.
+		 * This is the limit to allow other capabilities as much
+		 * space as they require.
+		 */
+		len = strlen(cmd_software_version_get());
+		if (len > BGP_MAX_SOFT_VERSION)
+			len = BGP_MAX_SOFT_VERSION;
+
+		stream_putc(s, len);
+		stream_put(s, cmd_software_version_get(), len);
+
+		/* Software Version capability Len. */
+		len = stream_get_endp(s) - rcapp - 1;
+		ext_opt_params ? stream_putw_at(s, rcapp, len - 1)
+			       : stream_putc_at(s, rcapp, len);
+
+		/* Total Capability Len. */
+		len = stream_get_endp(s) - capp - 1;
+		stream_putc_at(s, capp, len);
+
+		if (bgp_debug_neighbor_events(peer))
+			zlog_debug("%s Sending Software Version cap, value: %s",
+				   peer->host, cmd_software_version_get());
+	}
+
 	/* Total Opt Parm Len. */
 	len = stream_get_endp(s) - cp - 1;
 
