@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_lsp.c
  *                             LSP processing
@@ -6,26 +7,12 @@
  *                           Tampere University of Technology
  *                           Institute of Communications Engineering
  * Copyright (C) 2013-2015   Christian Franke <chris@opensourcerouting.org>
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include "linklist.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "vty.h"
 #include "stream.h"
 #include "memory.h"
@@ -59,15 +46,18 @@
 #include "isisd/fabricd.h"
 #include "isisd/isis_tx_queue.h"
 #include "isisd/isis_nb.h"
+#include "isisd/isis_flex_algo.h"
 
 DEFINE_MTYPE_STATIC(ISISD, ISIS_LSP, "ISIS LSP");
 
-static void lsp_refresh(struct thread *thread);
-static void lsp_l1_refresh_pseudo(struct thread *thread);
-static void lsp_l2_refresh_pseudo(struct thread *thread);
+static void lsp_refresh(struct event *thread);
+static void lsp_l1_refresh_pseudo(struct event *thread);
+static void lsp_l2_refresh_pseudo(struct event *thread);
 
 static void lsp_destroy(struct isis_lsp *lsp);
 
+static bool device_startup;
+
 int lsp_id_cmp(uint8_t *id1, uint8_t *id2)
 {
 	return memcmp(id1, id2, ISIS_SYS_ID_LEN + 2);
@@ -204,10 +194,9 @@ int lsp_compare(char *areatag, struct isis_lsp *lsp, uint32_t seqno,
 		|| (lsp->hdr.rem_lifetime != 0 && rem_lifetime != 0))) {
 		if (IS_DEBUG_SNP_PACKETS) {
 			zlog_debug(
-				"ISIS-Snp (%s): Compare LSP %s seq 0x%08x, cksum 0x%04hx, lifetime %hus",
-				areatag, rawlspid_print(lsp->hdr.lsp_id),
-				lsp->hdr.seqno, lsp->hdr.checksum,
-				lsp->hdr.rem_lifetime);
+				"ISIS-Snp (%s): Compare LSP %pLS seq 0x%08x, cksum 0x%04hx, lifetime %hus",
+				areatag, lsp->hdr.lsp_id, lsp->hdr.seqno,
+				lsp->hdr.checksum, lsp->hdr.rem_lifetime);
 			zlog_debug(
 				"ISIS-Snp (%s):         is equal to ours seq 0x%08x, cksum 0x%04hx, lifetime %hus",
 				areatag, seqno, checksum, rem_lifetime);
@@ -234,9 +223,9 @@ int lsp_compare(char *areatag, struct isis_lsp *lsp, uint32_t seqno,
 			&& lsp->hdr.rem_lifetime)))) {
 		if (IS_DEBUG_SNP_PACKETS) {
 			zlog_debug(
-				"ISIS-Snp (%s): Compare LSP %s seq 0x%08x, cksum 0x%04hx, lifetime %hus",
-				areatag, rawlspid_print(lsp->hdr.lsp_id), seqno,
-				checksum, rem_lifetime);
+				"ISIS-Snp (%s): Compare LSP %pLS seq 0x%08x, cksum 0x%04hx, lifetime %hus",
+				areatag, lsp->hdr.lsp_id, seqno, checksum,
+				rem_lifetime);
 			zlog_debug(
 				"ISIS-Snp (%s):       is newer than ours seq 0x%08x, cksum 0x%04hx, lifetime %hus",
 				areatag, lsp->hdr.seqno, lsp->hdr.checksum,
@@ -245,9 +234,10 @@ int lsp_compare(char *areatag, struct isis_lsp *lsp, uint32_t seqno,
 		return LSP_NEWER;
 	}
 	if (IS_DEBUG_SNP_PACKETS) {
-		zlog_debug("ISIS-Snp (%s): Compare LSP %s seq 0x%08x, cksum 0x%04hx, lifetime %hus",
-			   areatag, rawlspid_print(lsp->hdr.lsp_id), seqno,
-			   checksum, rem_lifetime);
+		zlog_debug(
+			"ISIS-Snp (%s): Compare LSP %pLS seq 0x%08x, cksum 0x%04hx, lifetime %hus",
+			areatag, lsp->hdr.lsp_id, seqno, checksum,
+			rem_lifetime);
 		zlog_debug(
 			"ISIS-Snp (%s):       is older than ours seq 0x%08x, cksum 0x%04hx, lifetime %hus",
 			areatag, lsp->hdr.seqno, lsp->hdr.checksum,
@@ -437,6 +427,21 @@ bool isis_level2_adj_up(struct isis_area *area)
 	return false;
 }
 
+/*
+ * Unset the overload bit after the timer expires
+ */
+void set_overload_on_start_timer(struct event *thread)
+{
+	struct isis_area *area = EVENT_ARG(thread);
+	assert(area);
+
+	area->t_overload_on_startup_timer = NULL;
+
+	/* Check if set-overload-bit is not currently configured */
+	if (!area->overload_configured)
+		isis_area_overload_bit_set(area, false);
+}
+
 static void isis_reset_attach_bit(struct isis_adjacency *adj)
 {
 	struct isis_area *area = adj->circuit->area;
@@ -550,8 +555,8 @@ void lsp_update(struct isis_lsp *lsp, struct isis_lsp_hdr *hdr,
 	if (lsp->own_lsp) {
 		flog_err(
 			EC_LIB_DEVELOPMENT,
-			"ISIS-Upd (%s): BUG updating LSP %s still marked as own LSP",
-			area->area_tag, rawlspid_print(lsp->hdr.lsp_id));
+			"ISIS-Upd (%s): BUG updating LSP %pLS still marked as own LSP",
+			area->area_tag, lsp->hdr.lsp_id);
 		lsp_clear_data(lsp);
 		lsp->own_lsp = 0;
 	}
@@ -630,10 +635,8 @@ struct isis_lsp *lsp_new(struct isis_area *area, uint8_t *lsp_id,
 	put_lsp_hdr(lsp, NULL, false);
 
 	if (IS_DEBUG_EVENTS)
-		zlog_debug("New LSP with ID %s-%02x-%02x len %d seqnum %08x",
-			   sysid_print(lsp_id), LSP_PSEUDO_ID(lsp->hdr.lsp_id),
-			   LSP_FRAGMENT(lsp->hdr.lsp_id), lsp->hdr.pdu_len,
-			   lsp->hdr.seqno);
+		zlog_debug("New LSP with ID %pLS len %d seqnum %08x", lsp_id,
+			   lsp->hdr.pdu_len, lsp->hdr.seqno);
 
 	return lsp;
 }
@@ -700,7 +703,7 @@ void lspid_print(uint8_t *lsp_id, char *dest, size_t dest_len, char dynhost,
 	else if (!memcmp(isis->sysid, lsp_id, ISIS_SYS_ID_LEN) && dynhost)
 		snprintf(id, sizeof(id), "%.14s", cmd_hostname_get());
 	else
-		memcpy(id, sysid_print(lsp_id), 15);
+		snprintfrr(id, sizeof(id), "%pSY", lsp_id);
 
 	if (frag)
 		snprintf(dest, dest_len, "%s.%02x-%02x", id,
@@ -877,6 +880,65 @@ static uint16_t lsp_refresh_time(struct isis_lsp *lsp, uint16_t rem_lifetime)
 	return refresh_time;
 }
 
+static void lsp_build_internal_reach_ipv4(struct isis_lsp *lsp,
+					  struct isis_area *area,
+					  struct prefix_ipv4 *ipv4,
+					  uint32_t metric)
+{
+	struct sr_prefix_cfg *pcfgs[SR_ALGORITHM_COUNT] = {NULL};
+
+	if (area->oldmetric) {
+		lsp_debug(
+			"ISIS (%s): Adding old-style IP reachability for %pFX",
+			area->area_tag, ipv4);
+		isis_tlvs_add_oldstyle_ip_reach(lsp->tlvs, ipv4, metric);
+	}
+
+	if (area->newmetric) {
+		lsp_debug("ISIS (%s): Adding te-style IP reachability for %pFX",
+			  area->area_tag, ipv4);
+
+		if (area->srdb.enabled)
+			for (int i = 0; i < SR_ALGORITHM_COUNT; i++) {
+#ifndef FABRICD
+				if (flex_algo_id_valid(i) &&
+				    !isis_flex_algo_elected_supported(i, area))
+					continue;
+#endif /* ifndef FABRICD */
+				pcfgs[i] =
+					isis_sr_cfg_prefix_find(area, ipv4, i);
+			}
+
+		isis_tlvs_add_extended_ip_reach(lsp->tlvs, ipv4, metric, false,
+						pcfgs);
+	}
+}
+
+static void lsp_build_internal_reach_ipv6(struct isis_lsp *lsp,
+					  struct isis_area *area,
+					  struct prefix_ipv6 *ipv6,
+					  uint32_t metric)
+{
+	struct sr_prefix_cfg *pcfgs[SR_ALGORITHM_COUNT] = {NULL};
+
+	lsp_debug("ISIS (%s): Adding IPv6 reachability for %pFX",
+		  area->area_tag, ipv6);
+
+	if (area->srdb.enabled)
+		for (int i = 0; i < SR_ALGORITHM_COUNT; i++) {
+#ifndef FABRICD
+			if (flex_algo_id_valid(i) &&
+			    !isis_flex_algo_elected_supported(i, area))
+				continue;
+#endif /* ifndef FABRICD */
+			pcfgs[i] = isis_sr_cfg_prefix_find(area, ipv6, i);
+		}
+
+	isis_tlvs_add_ipv6_reach(lsp->tlvs, isis_area_ipv6_topology(area), ipv6,
+				 metric, false, pcfgs);
+}
+
+
 static void lsp_build_ext_reach_ipv4(struct isis_lsp *lsp,
 				     struct isis_area *area)
 {
@@ -902,13 +964,23 @@ static void lsp_build_ext_reach_ipv4(struct isis_lsp *lsp,
 			isis_tlvs_add_oldstyle_ip_reach(lsp->tlvs, ipv4,
 							metric);
 		if (area->newmetric) {
-			struct sr_prefix_cfg *pcfg = NULL;
+			struct sr_prefix_cfg *pcfgs[SR_ALGORITHM_COUNT] = {
+				NULL};
 
 			if (area->srdb.enabled)
-				pcfg = isis_sr_cfg_prefix_find(area, ipv4);
+				for (int i = 0; i < SR_ALGORITHM_COUNT; i++) {
+#ifndef FABRICD
+					if (flex_algo_id_valid(i) &&
+					    !isis_flex_algo_elected_supported(
+						    i, area))
+						continue;
+#endif /* ifndef FABRICD */
+					pcfgs[i] = isis_sr_cfg_prefix_find(
+						area, ipv4, i);
+				}
 
 			isis_tlvs_add_extended_ip_reach(lsp->tlvs, ipv4, metric,
-							true, pcfg);
+							true, pcfgs);
 		}
 	}
 }
@@ -936,14 +1008,24 @@ static void lsp_build_ext_reach_ipv6(struct isis_lsp *lsp,
 			metric = MAX_WIDE_PATH_METRIC;
 
 		if (!src_p || !src_p->prefixlen) {
-			struct sr_prefix_cfg *pcfg = NULL;
+			struct sr_prefix_cfg *pcfgs[SR_ALGORITHM_COUNT] = {
+				NULL};
 
 			if (area->srdb.enabled)
-				pcfg = isis_sr_cfg_prefix_find(area, p);
+				for (int i = 0; i < SR_ALGORITHM_COUNT; i++) {
+#ifndef FABRICD
+					if (flex_algo_id_valid(i) &&
+					    !isis_flex_algo_elected_supported(
+						    i, area))
+						continue;
+#endif /* ifndef FABRICD */
+					pcfgs[i] = isis_sr_cfg_prefix_find(
+						area, p, i);
+				}
 
 			isis_tlvs_add_ipv6_reach(lsp->tlvs,
 						 isis_area_ipv6_topology(area),
-						 p, metric, true, pcfg);
+						 p, metric, true, pcfgs);
 		} else if (isis_area_ipv6_dstsrc_enabled(area)) {
 			isis_tlvs_add_ipv6_dstsrc_reach(lsp->tlvs,
 							ISIS_MT_IPV6_DSTSRC,
@@ -1060,9 +1142,30 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 
 	/* Add Router Capability TLV. */
 	if (area->isis->router_id != 0) {
-		struct isis_router_cap cap = {};
+		struct isis_router_cap *rcap;
+#ifndef FABRICD
+		struct isis_router_cap_fad *rcap_fad;
+		struct listnode *node;
+		struct flex_algo *fa;
+#endif /* ifndef FABRICD */
 
-		cap.router_id.s_addr = area->isis->router_id;
+		rcap = isis_tlvs_init_router_capability(lsp->tlvs);
+
+		rcap->router_id.s_addr = area->isis->router_id;
+
+#ifndef FABRICD
+		/* Set flex-algo definitions */
+		for (ALL_LIST_ELEMENTS_RO(area->flex_algos->flex_algos, node,
+					  fa)) {
+			if (!fa->advertise_definition)
+				continue;
+			lsp_debug("ISIS (%s):   Flex-Algo Definition %u",
+				  area->area_tag, fa->algorithm);
+			isis_tlvs_set_router_capability_fad(lsp->tlvs, fa,
+							    fa->algorithm,
+							    area->isis->sysid);
+		}
+#endif /* ifndef FABRICD */
 
 		/* Add SR Sub-TLVs if SR is enabled. */
 		if (area->srdb.enabled) {
@@ -1072,30 +1175,97 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 			/* SRGB first */
 			range_size = srdb->config.srgb_upper_bound
 				     - srdb->config.srgb_lower_bound + 1;
-			cap.srgb.flags = ISIS_SUBTLV_SRGB_FLAG_I
-					 | ISIS_SUBTLV_SRGB_FLAG_V;
-			cap.srgb.range_size = range_size;
-			cap.srgb.lower_bound = srdb->config.srgb_lower_bound;
+			rcap->srgb.flags = ISIS_SUBTLV_SRGB_FLAG_I |
+					   ISIS_SUBTLV_SRGB_FLAG_V;
+			rcap->srgb.range_size = range_size;
+			rcap->srgb.lower_bound = srdb->config.srgb_lower_bound;
 			/* Then Algorithm */
-			cap.algo[0] = SR_ALGORITHM_SPF;
-			cap.algo[1] = SR_ALGORITHM_UNSET;
+			rcap->algo[0] = SR_ALGORITHM_SPF;
+			rcap->algo[1] = SR_ALGORITHM_UNSET;
+#ifndef FABRICD
+			for (ALL_LIST_ELEMENTS_RO(area->flex_algos->flex_algos,
+						  node, fa)) {
+				if (fa->advertise_definition)
+					rcap_fad = rcap->fads[fa->algorithm];
+				else
+					rcap_fad = NULL;
+
+				if (!isis_flex_algo_elected_supported_local_fad(
+					    fa->algorithm, area, &rcap_fad)) {
+					fa->state = false;
+					continue;
+				}
+				fa->state = true;
+				lsp_debug("ISIS (%s):   SR Algorithm %u",
+					  area->area_tag, fa->algorithm);
+				rcap->algo[fa->algorithm] = fa->algorithm;
+			}
+#endif /* ifndef FABRICD */
 			/* SRLB */
-			cap.srlb.flags = 0;
+			rcap->srlb.flags = 0;
 			range_size = srdb->config.srlb_upper_bound
 				     - srdb->config.srlb_lower_bound + 1;
-			cap.srlb.range_size = range_size;
-			cap.srlb.lower_bound = srdb->config.srlb_lower_bound;
+			rcap->srlb.range_size = range_size;
+			rcap->srlb.lower_bound = srdb->config.srlb_lower_bound;
 			/* And finally MSD */
-			cap.msd = srdb->config.msd;
-		} else {
-			/* Disable SR Algorithm */
-			cap.algo[0] = SR_ALGORITHM_UNSET;
-			cap.algo[1] = SR_ALGORITHM_UNSET;
+			rcap->msd = srdb->config.msd;
 		}
 
-		isis_tlvs_set_router_capability(lsp->tlvs, &cap);
-		lsp_debug("ISIS (%s): Adding Router Capabilities information",
+		/* Add SRv6 Sub-TLVs if SRv6 is enabled */
+		if (area->srv6db.config.enabled) {
+			struct isis_srv6_db *srv6db = &area->srv6db;
+
+			rcap->srv6_cap.is_srv6_capable = true;
+
+			/* SRv6 flags */
+			rcap->srv6_cap.flags = 0;
+
+			/* And finally MSDs */
+			rcap->srv6_msd.max_seg_left_msd =
+				srv6db->config.max_seg_left_msd;
+			rcap->srv6_msd.max_end_pop_msd =
+				srv6db->config.max_end_pop_msd;
+			rcap->srv6_msd.max_h_encaps_msd =
+				srv6db->config.max_h_encaps_msd;
+			rcap->srv6_msd.max_end_d_msd =
+				srv6db->config.max_end_d_msd;
+		} else {
+			rcap->srv6_cap.is_srv6_capable = false;
+		}
+	}
+
+	/* Add SRv6 Locator TLV. */
+	if (area->srv6db.config.enabled &&
+	    !list_isempty(area->srv6db.srv6_locator_chunks)) {
+		struct isis_srv6_locator locator = {};
+		struct srv6_locator_chunk *chunk;
+
+		/* TODO: support more than one locator */
+		chunk = (struct srv6_locator_chunk *)listgetdata(
+			listhead(area->srv6db.srv6_locator_chunks));
+
+		locator.metric = 0;
+		locator.prefix = chunk->prefix;
+		locator.flags = 0;
+		locator.algorithm = 0;
+
+		struct listnode *sid_node;
+		struct isis_srv6_sid *sid;
+		locator.srv6_sid = list_new();
+		for (ALL_LIST_ELEMENTS_RO(area->srv6db.srv6_sids, sid_node,
+					  sid)) {
+			listnode_add(locator.srv6_sid, sid);
+		}
+
+		isis_tlvs_add_srv6_locator(lsp->tlvs, 0, &locator);
+		lsp_debug("ISIS (%s): Adding SRv6 Locator information",
 			  area->area_tag);
+
+		list_delete(&locator.srv6_sid);
+
+		isis_tlvs_add_ipv6_reach(lsp->tlvs,
+					 isis_area_ipv6_topology(area),
+					 &chunk->prefix, 0, false, NULL);
 	}
 
 	/* IPv4 address and TE router ID TLVs.
@@ -1167,6 +1337,13 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 			continue;
 		}
 
+		if (area->advertise_passive_only && !circuit->is_passive) {
+			lsp_debug(
+				"ISIS (%s): Circuit is not passive, ignoring.",
+				area->area_tag);
+			continue;
+		}
+
 		uint32_t metric = area->oldmetric
 					  ? circuit->metric[level - 1]
 					  : circuit->te_metric[level - 1];
@@ -1178,31 +1355,9 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 			struct listnode *ipnode;
 			struct prefix_ipv4 *ipv4;
 			for (ALL_LIST_ELEMENTS_RO(circuit->ip_addrs, ipnode,
-						  ipv4)) {
-				if (area->oldmetric) {
-					lsp_debug(
-						"ISIS (%s): Adding old-style IP reachability for %pFX",
-						area->area_tag, ipv4);
-					isis_tlvs_add_oldstyle_ip_reach(
-						lsp->tlvs, ipv4, metric);
-				}
-
-				if (area->newmetric) {
-					struct sr_prefix_cfg *pcfg = NULL;
-
-					lsp_debug(
-						"ISIS (%s): Adding te-style IP reachability for %pFX",
-						area->area_tag, ipv4);
-
-					if (area->srdb.enabled)
-						pcfg = isis_sr_cfg_prefix_find(
-							area, ipv4);
-
-					isis_tlvs_add_extended_ip_reach(
-						lsp->tlvs, ipv4, metric, false,
-						pcfg);
-				}
-			}
+						  ipv4))
+				lsp_build_internal_reach_ipv4(lsp, area, ipv4,
+							      metric);
 		}
 
 		if (circuit->ipv6_router && circuit->ipv6_non_link->count > 0) {
@@ -1210,22 +1365,9 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 			struct prefix_ipv6 *ipv6;
 
 			for (ALL_LIST_ELEMENTS_RO(circuit->ipv6_non_link,
-						  ipnode, ipv6)) {
-				struct sr_prefix_cfg *pcfg = NULL;
-
-				lsp_debug(
-					"ISIS (%s): Adding IPv6 reachability for %pFX",
-					area->area_tag, ipv6);
-
-				if (area->srdb.enabled)
-					pcfg = isis_sr_cfg_prefix_find(area,
-								       ipv6);
-
-				isis_tlvs_add_ipv6_reach(
-					lsp->tlvs,
-					isis_area_ipv6_topology(area), ipv6,
-					metric, false, pcfg);
-			}
+						  ipnode, ipv6))
+				lsp_build_internal_reach_ipv6(lsp, area, ipv6,
+							      metric);
 		}
 
 		switch (circuit->circ_type) {
@@ -1239,10 +1381,8 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 				if (LSP_PSEUDO_ID(ne_id)) {
 					if (area->oldmetric) {
 						lsp_debug(
-							"ISIS (%s): Adding DIS %s.%02x as old-style neighbor",
-							area->area_tag,
-							sysid_print(ne_id),
-							LSP_PSEUDO_ID(ne_id));
+							"ISIS (%s): Adding DIS %pPN as old-style neighbor",
+							area->area_tag, ne_id);
 						isis_tlvs_add_oldstyle_reach(
 							lsp->tlvs, ne_id,
 							metric);
@@ -1268,9 +1408,8 @@ static void lsp_build(struct isis_lsp *lsp, struct isis_area *area)
 
 				if (area->oldmetric) {
 					lsp_debug(
-						"ISIS (%s): Adding old-style is reach for %s",
-						area->area_tag,
-						sysid_print(ne_id));
+						"ISIS (%s): Adding old-style is reach for %pSY",
+						area->area_tag, ne_id);
 					isis_tlvs_add_oldstyle_reach(
 						lsp->tlvs, ne_id, metric);
 				}
@@ -1355,14 +1494,31 @@ int lsp_generate(struct isis_area *area, int level)
 	uint32_t seq_num = 0;
 	uint8_t lspid[ISIS_SYS_ID_LEN + 2];
 	uint16_t rem_lifetime, refresh_time;
+	uint32_t overload_time;
 
 	if ((area == NULL) || (area->is_type & level) != level)
 		return ISIS_ERROR;
 
+	/* Check if config is still being processed */
+	if (event_is_scheduled(t_isis_cfg))
+		return ISIS_OK;
+
 	memset(&lspid, 0, ISIS_SYS_ID_LEN + 2);
 
 	memcpy(&lspid, area->isis->sysid, ISIS_SYS_ID_LEN);
 
+	/* Check if device should be overloaded on startup */
+	if (device_startup) {
+		overload_time = isis_restart_read_overload_time(area);
+		if (overload_time > 0) {
+			isis_area_overload_bit_set(area, true);
+			event_add_timer(master, set_overload_on_start_timer,
+					area, overload_time,
+					&area->t_overload_on_startup_timer);
+		}
+		device_startup = false;
+	}
+
 	/* only builds the lsp if the area shares the level */
 	oldlsp = lsp_search(&area->lspdb[level - 1], lspid);
 	if (oldlsp) {
@@ -1390,19 +1546,18 @@ int lsp_generate(struct isis_area *area, int level)
 
 	refresh_time = lsp_refresh_time(newlsp, rem_lifetime);
 
-	THREAD_OFF(area->t_lsp_refresh[level - 1]);
+	EVENT_OFF(area->t_lsp_refresh[level - 1]);
 	area->lsp_regenerate_pending[level - 1] = 0;
-	thread_add_timer(master, lsp_refresh,
-			 &area->lsp_refresh_arg[level - 1], refresh_time,
-			 &area->t_lsp_refresh[level - 1]);
+	event_add_timer(master, lsp_refresh, &area->lsp_refresh_arg[level - 1],
+			refresh_time, &area->t_lsp_refresh[level - 1]);
 
 	if (IS_DEBUG_UPDATE_PACKETS) {
-		zlog_debug("ISIS-Upd (%s): Building L%d LSP %s, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus refresh %hus",
-			   area->area_tag, level,
-			   rawlspid_print(newlsp->hdr.lsp_id),
-			   newlsp->hdr.pdu_len, newlsp->hdr.seqno,
-			   newlsp->hdr.checksum, newlsp->hdr.rem_lifetime,
-			   refresh_time);
+		zlog_debug(
+			"ISIS-Upd (%s): Building L%d LSP %pLS, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus refresh %hus",
+			area->area_tag, level, newlsp->hdr.lsp_id,
+			newlsp->hdr.pdu_len, newlsp->hdr.seqno,
+			newlsp->hdr.checksum, newlsp->hdr.rem_lifetime,
+			refresh_time);
 	}
 	sched_debug(
 		"ISIS (%s): Built L%d LSP. Set triggered regenerate to non-pending.",
@@ -1473,15 +1628,14 @@ static int lsp_regenerate(struct isis_area *area, int level)
 	lsp_seqno_update(lsp);
 
 	refresh_time = lsp_refresh_time(lsp, rem_lifetime);
-	thread_add_timer(master, lsp_refresh,
-			 &area->lsp_refresh_arg[level - 1], refresh_time,
-			 &area->t_lsp_refresh[level - 1]);
+	event_add_timer(master, lsp_refresh, &area->lsp_refresh_arg[level - 1],
+			refresh_time, &area->t_lsp_refresh[level - 1]);
 	area->lsp_regenerate_pending[level - 1] = 0;
 
 	if (IS_DEBUG_UPDATE_PACKETS) {
 		zlog_debug(
-			"ISIS-Upd (%s): Refreshed our L%d LSP %s, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus refresh %hus",
-			area->area_tag, level, rawlspid_print(lsp->hdr.lsp_id),
+			"ISIS-Upd (%s): Refreshed our L%d LSP %pLS, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus refresh %hus",
+			area->area_tag, level, lsp->hdr.lsp_id,
 			lsp->hdr.pdu_len, lsp->hdr.seqno, lsp->hdr.checksum,
 			lsp->hdr.rem_lifetime, refresh_time);
 	}
@@ -1495,9 +1649,9 @@ static int lsp_regenerate(struct isis_area *area, int level)
 /*
  * Something has changed or periodic refresh -> regenerate LSP
  */
-static void lsp_refresh(struct thread *thread)
+static void lsp_refresh(struct event *thread)
 {
-	struct lsp_refresh_arg *arg = THREAD_ARG(thread);
+	struct lsp_refresh_arg *arg = EVENT_ARG(thread);
 
 	assert(arg);
 
@@ -1577,7 +1731,7 @@ int _lsp_regenerate_schedule(struct isis_area *area, int level,
 			 * Note: in case of a BFD 'down' message the refresh is
 			 * scheduled once again just to be sure
 			 */
-			struct timeval remain = thread_timer_remain(
+			struct timeval remain = event_timer_remain(
 				area->t_lsp_refresh[lvl - 1]);
 			sched_debug(
 				"ISIS (%s): Regeneration is already pending, nothing todo. (Due in %lld.%03lld seconds)",
@@ -1601,7 +1755,7 @@ int _lsp_regenerate_schedule(struct isis_area *area, int level,
 			"ISIS (%s): Will schedule regen timer. Last run was: %lld, Now is: %lld",
 			area->area_tag, (long long)lsp->last_generated,
 			(long long)now);
-		THREAD_OFF(area->t_lsp_refresh[lvl - 1]);
+		EVENT_OFF(area->t_lsp_refresh[lvl - 1]);
 		diff = now - lsp->last_generated;
 		if (diff < area->lsp_gen_interval[lvl - 1]
 		    && !(area->bfd_signalled_down)) {
@@ -1640,10 +1794,9 @@ int _lsp_regenerate_schedule(struct isis_area *area, int level,
 		}
 
 		area->lsp_regenerate_pending[lvl - 1] = 1;
-		thread_add_timer_msec(master, lsp_refresh,
-				      &area->lsp_refresh_arg[lvl - 1],
-				      timeout,
-				      &area->t_lsp_refresh[lvl - 1]);
+		event_add_timer_msec(master, lsp_refresh,
+				     &area->lsp_refresh_arg[lvl - 1], timeout,
+				     &area->t_lsp_refresh[lvl - 1]);
 	}
 
 	if (all_pseudo) {
@@ -1673,9 +1826,9 @@ static void lsp_build_pseudo(struct isis_lsp *lsp, struct isis_circuit *circuit,
 	lsp_clear_data(lsp);
 	lsp->tlvs = isis_alloc_tlvs();
 	lsp_debug(
-		"ISIS (%s): Constructing pseudo LSP %s for interface %s level %d",
-		area->area_tag, rawlspid_print(lsp->hdr.lsp_id),
-		circuit->interface->name, level);
+		"ISIS (%s): Constructing pseudo LSP %pLS for interface %s level %d",
+		area->area_tag, lsp->hdr.lsp_id, circuit->interface->name,
+		level);
 
 	lsp->level = level;
 	/* RFC3787  section 4 SHOULD not set overload bit in pseudo LSPs */
@@ -1692,10 +1845,8 @@ static void lsp_build_pseudo(struct isis_lsp *lsp, struct isis_circuit *circuit,
 
 	if (circuit->area->oldmetric) {
 		isis_tlvs_add_oldstyle_reach(lsp->tlvs, ne_id, 0);
-		lsp_debug(
-			"ISIS (%s): Adding %s.%02x as old-style neighbor (self)",
-			area->area_tag, sysid_print(ne_id),
-			LSP_PSEUDO_ID(ne_id));
+		lsp_debug("ISIS (%s): Adding %pPN as old-style neighbor (self)",
+			  area->area_tag, ne_id);
 	}
 	if (circuit->area->newmetric) {
 		if (area_is_mt(circuit->area))
@@ -1703,10 +1854,8 @@ static void lsp_build_pseudo(struct isis_lsp *lsp, struct isis_circuit *circuit,
 		else
 			mtid = ISIS_MT_DISABLE;
 		isis_tlvs_add_extended_reach(lsp->tlvs, mtid, ne_id, 0, NULL);
-		lsp_debug(
-			"ISIS (%s): Adding %s.%02x as te-style neighbor (self)",
-			area->area_tag, sysid_print(ne_id),
-			LSP_PSEUDO_ID(ne_id));
+		lsp_debug("ISIS (%s): Adding %pPN as te-style neighbor (self)",
+			  area->area_tag, ne_id);
 	}
 
 	adj_list = list_new();
@@ -1715,8 +1864,8 @@ static void lsp_build_pseudo(struct isis_lsp *lsp, struct isis_circuit *circuit,
 	for (ALL_LIST_ELEMENTS_RO(adj_list, node, adj)) {
 		if (!(adj->level & level)) {
 			lsp_debug(
-				"ISIS (%s): Ignoring neighbor %s, level does not intersect",
-				area->area_tag, sysid_print(adj->sysid));
+				"ISIS (%s): Ignoring neighbor %pSY, level does not intersect",
+				area->area_tag, adj->sysid);
 			continue;
 		}
 
@@ -1728,8 +1877,8 @@ static void lsp_build_pseudo(struct isis_lsp *lsp, struct isis_circuit *circuit,
 		    && !(level == IS_LEVEL_2
 			 && adj->sys_type == ISIS_SYSTYPE_L2_IS)) {
 			lsp_debug(
-				"ISIS (%s): Ignoring neighbor %s, level does not match",
-				area->area_tag, sysid_print(adj->sysid));
+				"ISIS (%s): Ignoring neighbor %pSY, level does not match",
+				area->area_tag, adj->sysid);
 			continue;
 		}
 
@@ -1737,18 +1886,16 @@ static void lsp_build_pseudo(struct isis_lsp *lsp, struct isis_circuit *circuit,
 		if (circuit->area->oldmetric) {
 			isis_tlvs_add_oldstyle_reach(lsp->tlvs, ne_id, 0);
 			lsp_debug(
-				"ISIS (%s): Adding %s.%02x as old-style neighbor (peer)",
-				area->area_tag, sysid_print(ne_id),
-				LSP_PSEUDO_ID(ne_id));
+				"ISIS (%s): Adding %pPN as old-style neighbor (peer)",
+				area->area_tag, ne_id);
 		}
 		if (circuit->area->newmetric) {
 			isis_tlvs_add_extended_reach(lsp->tlvs,
 						     ISIS_MT_IPV4_UNICAST,
 						     ne_id, 0, NULL);
 			lsp_debug(
-				"ISIS (%s): Adding %s.%02x as te-style neighbor (peer)",
-				area->area_tag, sysid_print(ne_id),
-				LSP_PSEUDO_ID(ne_id));
+				"ISIS (%s): Adding %pPN as te-style neighbor (peer)",
+				area->area_tag, ne_id);
 		}
 	}
 	list_delete(&adj_list);
@@ -1794,23 +1941,22 @@ int lsp_generate_pseudo(struct isis_circuit *circuit, int level)
 	lsp_flood(lsp, NULL);
 
 	refresh_time = lsp_refresh_time(lsp, rem_lifetime);
-	THREAD_OFF(circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
+	EVENT_OFF(circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
 	circuit->lsp_regenerate_pending[level - 1] = 0;
 	if (level == IS_LEVEL_1)
-		thread_add_timer(
-			master, lsp_l1_refresh_pseudo, circuit, refresh_time,
-			&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
+		event_add_timer(master, lsp_l1_refresh_pseudo, circuit,
+				refresh_time,
+				&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
 	else if (level == IS_LEVEL_2)
-		thread_add_timer(
-			master, lsp_l2_refresh_pseudo, circuit, refresh_time,
-			&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
+		event_add_timer(master, lsp_l2_refresh_pseudo, circuit,
+				refresh_time,
+				&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
 
 	if (IS_DEBUG_UPDATE_PACKETS) {
 		zlog_debug(
-			"ISIS-Upd (%s): Built L%d Pseudo LSP %s, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus, refresh %hus",
-			circuit->area->area_tag, level,
-			rawlspid_print(lsp->hdr.lsp_id), lsp->hdr.pdu_len,
-			lsp->hdr.seqno, lsp->hdr.checksum,
+			"ISIS-Upd (%s): Built L%d Pseudo LSP %pLS, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus, refresh %hus",
+			circuit->area->area_tag, level, lsp->hdr.lsp_id,
+			lsp->hdr.pdu_len, lsp->hdr.seqno, lsp->hdr.checksum,
 			lsp->hdr.rem_lifetime, refresh_time);
 	}
 
@@ -1838,8 +1984,8 @@ static int lsp_regenerate_pseudo(struct isis_circuit *circuit, int level)
 
 	if (!lsp) {
 		flog_err(EC_LIB_DEVELOPMENT,
-			 "lsp_regenerate_pseudo: no l%d LSP %s found!", level,
-			 rawlspid_print(lsp_id));
+			 "lsp_regenerate_pseudo: no l%d LSP %pLS found!", level,
+			 lsp_id);
 		return ISIS_ERROR;
 	}
 
@@ -1852,20 +1998,19 @@ static int lsp_regenerate_pseudo(struct isis_circuit *circuit, int level)
 
 	refresh_time = lsp_refresh_time(lsp, rem_lifetime);
 	if (level == IS_LEVEL_1)
-		thread_add_timer(
-			master, lsp_l1_refresh_pseudo, circuit, refresh_time,
-			&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
+		event_add_timer(master, lsp_l1_refresh_pseudo, circuit,
+				refresh_time,
+				&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
 	else if (level == IS_LEVEL_2)
-		thread_add_timer(
-			master, lsp_l2_refresh_pseudo, circuit, refresh_time,
-			&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
+		event_add_timer(master, lsp_l2_refresh_pseudo, circuit,
+				refresh_time,
+				&circuit->u.bc.t_refresh_pseudo_lsp[level - 1]);
 
 	if (IS_DEBUG_UPDATE_PACKETS) {
 		zlog_debug(
-			"ISIS-Upd (%s): Refreshed L%d Pseudo LSP %s, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus, refresh %hus",
-			circuit->area->area_tag, level,
-			rawlspid_print(lsp->hdr.lsp_id), lsp->hdr.pdu_len,
-			lsp->hdr.seqno, lsp->hdr.checksum,
+			"ISIS-Upd (%s): Refreshed L%d Pseudo LSP %pLS, len %hu, seq 0x%08x, cksum 0x%04hx, lifetime %hus, refresh %hus",
+			circuit->area->area_tag, level, lsp->hdr.lsp_id,
+			lsp->hdr.pdu_len, lsp->hdr.seqno, lsp->hdr.checksum,
 			lsp->hdr.rem_lifetime, refresh_time);
 	}
 
@@ -1875,12 +2020,12 @@ static int lsp_regenerate_pseudo(struct isis_circuit *circuit, int level)
 /*
  * Something has changed or periodic refresh -> regenerate pseudo LSP
  */
-static void lsp_l1_refresh_pseudo(struct thread *thread)
+static void lsp_l1_refresh_pseudo(struct event *thread)
 {
 	struct isis_circuit *circuit;
 	uint8_t id[ISIS_SYS_ID_LEN + 2];
 
-	circuit = THREAD_ARG(thread);
+	circuit = EVENT_ARG(thread);
 
 	circuit->u.bc.t_refresh_pseudo_lsp[0] = NULL;
 	circuit->lsp_regenerate_pending[0] = 0;
@@ -1897,12 +2042,12 @@ static void lsp_l1_refresh_pseudo(struct thread *thread)
 	lsp_regenerate_pseudo(circuit, IS_LEVEL_1);
 }
 
-static void lsp_l2_refresh_pseudo(struct thread *thread)
+static void lsp_l2_refresh_pseudo(struct event *thread)
 {
 	struct isis_circuit *circuit;
 	uint8_t id[ISIS_SYS_ID_LEN + 2];
 
-	circuit = THREAD_ARG(thread);
+	circuit = EVENT_ARG(thread);
 
 	circuit->u.bc.t_refresh_pseudo_lsp[1] = NULL;
 	circuit->lsp_regenerate_pending[1] = 0;
@@ -1961,7 +2106,7 @@ int lsp_regenerate_schedule_pseudo(struct isis_circuit *circuit, int level)
 		}
 
 		if (circuit->lsp_regenerate_pending[lvl - 1]) {
-			struct timeval remain = thread_timer_remain(
+			struct timeval remain = event_timer_remain(
 				circuit->u.bc.t_refresh_pseudo_lsp[lvl - 1]);
 			sched_debug(
 				"ISIS (%s): Regenerate is already pending, nothing todo. (Due in %lld.%03lld seconds)",
@@ -1985,7 +2130,7 @@ int lsp_regenerate_schedule_pseudo(struct isis_circuit *circuit, int level)
 			"ISIS (%s): Will schedule PSN regen timer. Last run was: %lld, Now is: %lld",
 			area->area_tag, (long long)lsp->last_generated,
 			(long long)now);
-		THREAD_OFF(circuit->u.bc.t_refresh_pseudo_lsp[lvl - 1]);
+		EVENT_OFF(circuit->u.bc.t_refresh_pseudo_lsp[lvl - 1]);
 		diff = now - lsp->last_generated;
 		if (diff < circuit->area->lsp_gen_interval[lvl - 1]) {
 			timeout =
@@ -2004,11 +2149,11 @@ int lsp_regenerate_schedule_pseudo(struct isis_circuit *circuit, int level)
 		circuit->lsp_regenerate_pending[lvl - 1] = 1;
 
 		if (lvl == IS_LEVEL_1) {
-			thread_add_timer_msec(
+			event_add_timer_msec(
 				master, lsp_l1_refresh_pseudo, circuit, timeout,
 				&circuit->u.bc.t_refresh_pseudo_lsp[lvl - 1]);
 		} else if (lvl == IS_LEVEL_2) {
-			thread_add_timer_msec(
+			event_add_timer_msec(
 				master, lsp_l2_refresh_pseudo, circuit, timeout,
 				&circuit->u.bc.t_refresh_pseudo_lsp[lvl - 1]);
 		}
@@ -2021,7 +2166,7 @@ int lsp_regenerate_schedule_pseudo(struct isis_circuit *circuit, int level)
  * Walk through LSPs for an area
  *  - set remaining lifetime
  */
-void lsp_tick(struct thread *thread)
+void lsp_tick(struct event *thread)
 {
 	struct isis_area *area;
 	struct isis_lsp *lsp;
@@ -2029,10 +2174,10 @@ void lsp_tick(struct thread *thread)
 	uint16_t rem_lifetime;
 	bool fabricd_sync_incomplete = false;
 
-	area = THREAD_ARG(thread);
+	area = EVENT_ARG(thread);
 	assert(area);
 	area->t_tick = NULL;
-	thread_add_timer(master, lsp_tick, area, 1, &area->t_tick);
+	event_add_timer(master, lsp_tick, area, 1, &area->t_tick);
 
 	struct isis_circuit *fabricd_init_c = fabricd_initial_sync_circuit(area);
 
@@ -2076,10 +2221,9 @@ void lsp_tick(struct thread *thread)
 
 			if (lsp->age_out == 0) {
 				zlog_debug(
-					"ISIS-Upd (%s): L%u LSP %s seq 0x%08x aged out",
+					"ISIS-Upd (%s): L%u LSP %pLS seq 0x%08x aged out",
 					area->area_tag, lsp->level,
-					rawlspid_print(lsp->hdr.lsp_id),
-					lsp->hdr.seqno);
+					lsp->hdr.lsp_id, lsp->hdr.seqno);
 
 				/* if we're aging out fragment 0, lsp_destroy()
 				 * below will delete all other fragments too,
@@ -2182,11 +2326,10 @@ void _lsp_flood(struct isis_lsp *lsp, struct isis_circuit *circuit,
 		const char *func, const char *file, int line)
 {
 	if (IS_DEBUG_FLOODING) {
-		zlog_debug("Flooding LSP %s%s%s (From %s %s:%d)",
-			   rawlspid_print(lsp->hdr.lsp_id),
-			   circuit ? " except on " : "",
-			   circuit ? circuit->interface->name : "",
-			   func, file, line);
+		zlog_debug("Flooding LSP %pLS%s%s (From %s %s:%d)",
+			   lsp->hdr.lsp_id, circuit ? " except on " : "",
+			   circuit ? circuit->interface->name : "", func, file,
+			   line);
 	}
 
 	if (!fabricd)
@@ -2373,6 +2516,7 @@ int isis_lsp_iterate_is_reach(struct isis_lsp *lsp, uint16_t mtid,
 
 void lsp_init(void)
 {
+	device_startup = true;
 	hook_register(isis_adj_state_change_hook,
 		      lsp_handle_adj_state_change);
 }
