@@ -1,19 +1,8 @@
+// SPDX-License-Identifier: ISC
 /*	$OpenBSD$ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
- * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
- * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
- * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
- * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 #include <zebra.h>
@@ -46,13 +35,11 @@ vlog(int pri, const char *fmt, va_list ap)
 	switch (ldpd_process) {
 	case PROC_LDE_ENGINE:
 		vsnprintfrr(buf, sizeof(buf), fmt, ap);
-		lde_imsg_compose_parent_sync(IMSG_LOG, pri, buf,
-		    strlen(buf) + 1);
+		lde_imsg_compose_parent_sync(IMSG_LOG, pri, buf, strlen(buf) + 1);
 		break;
 	case PROC_LDP_ENGINE:
 		vsnprintfrr(buf, sizeof(buf), fmt, ap);
-		ldpe_imsg_compose_parent_sync(IMSG_LOG, pri, buf,
-		    strlen(buf) + 1);
+		ldpe_imsg_compose_parent_sync(IMSG_LOG, pri, buf, strlen(buf) + 1);
 		break;
 	case PROC_MAIN:
 		vzlog(pri, fmt, ap);
@@ -77,7 +64,11 @@ log_warn(const char *emsg, ...)
 			vlog(LOG_ERR, emsg, ap);
 			logit(LOG_ERR, "%s", strerror(errno));
 		} else {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+			/* format extended above */
 			vlog(LOG_ERR, nfmt, ap);
+#pragma GCC diagnostic pop
 			free(nfmt);
 		}
 		va_end(ap);
@@ -128,15 +119,13 @@ void
 fatal(const char *emsg)
 {
 	if (emsg == NULL)
-		logit(LOG_CRIT, "fatal in %s: %s", log_procname,
-		    strerror(errno));
+		logit(LOG_CRIT, "fatal in %s: %s", log_procname, strerror(errno));
 	else
 		if (errno)
 			logit(LOG_CRIT, "fatal in %s: %s: %s",
 			    log_procname, emsg, strerror(errno));
 		else
-			logit(LOG_CRIT, "fatal in %s: %s",
-			    log_procname, emsg);
+			logit(LOG_CRIT, "fatal in %s: %s", log_procname, emsg);
 
 	exit(1);
 }
