@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* key-chain for authentication.
  * Copyright (C) 2000 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published
- * by the Free Software Foundation; either version 2, or (at your
- * option) any later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "config.h"
@@ -52,6 +37,7 @@ static void keychain_free(struct keychain *keychain)
 static struct key *key_new(void)
 {
 	struct key *key = XCALLOC(MTYPE_KEY, sizeof(struct key));
+
 	QOBJ_REG(key, key);
 	return key;
 }
@@ -92,7 +78,7 @@ static int key_cmp_func(void *arg1, void *arg2)
 static void key_delete_func(struct key *key)
 {
 	if (key->string)
-		free(key->string);
+		XFREE(MTYPE_KEY, key->string);
 	key_free(key);
 }
 
@@ -1202,6 +1188,20 @@ static const struct cmd_variable_handler keychain_var_handlers[] = {
 	{.completions = NULL}
 };
 
+void keychain_terminate(void)
+{
+	struct keychain *keychain;
+
+	while (listcount(keychain_list)) {
+		keychain = listgetdata(listhead(keychain_list));
+
+		listnode_delete(keychain_list, keychain);
+		keychain_delete(keychain);
+	}
+
+	list_delete(&keychain_list);
+}
+
 void keychain_init(void)
 {
 	keychain_list = list_new();
