@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2018  NetDEF, Inc.
  *                     Renato Westphal
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -36,10 +23,10 @@ DEFINE_MTYPE_STATIC(LIB, CONFD, "ConfD module");
 
 static struct debug nb_dbg_client_confd = {0, "Northbound client: ConfD"};
 
-static struct thread_master *master;
+static struct event_loop *master;
 static struct sockaddr confd_addr;
 static int cdb_sub_sock, dp_ctl_sock, dp_worker_sock;
-static struct thread *t_cdb_sub, *t_dp_ctl, *t_dp_worker;
+static struct event *t_cdb_sub, *t_dp_ctl, *t_dp_worker;
 static struct confd_daemon_ctx *dctx;
 static struct confd_notification_ctx *live_ctx;
 static bool confd_connected;
@@ -234,7 +221,7 @@ frr_confd_cdb_diff_iter(confd_hkeypath_t *kp, enum cdb_iter_op cdb_op,
 		nb_op = NB_OP_DESTROY;
 		break;
 	case MOP_VALUE_SET:
-		if (nb_operation_is_valid(NB_OP_MODIFY, nb_node->snode))
+		if (nb_is_operation_allowed(nb_node, NB_OP_MODIFY))
 			nb_op = NB_OP_MODIFY;
 		else
 			/* Ignore list keys modifications. */
@@ -324,8 +311,9 @@ static void frr_confd_cdb_read_cb_prepare(int fd, int *subp, int reslen)
 	 */
 	transaction = NULL;
 	context.client = NB_CLIENT_CONFD;
-	ret = nb_candidate_commit_prepare(&context, candidate, NULL,
-					  &transaction, errmsg, sizeof(errmsg));
+	ret = nb_candidate_commit_prepare(context, candidate, NULL,
+					  &transaction, false, false, errmsg,
+					  sizeof(errmsg));
 	if (ret != NB_OK && ret != NB_ERR_NO_CHANGES) {
 		enum confd_errcode errcode;
 
@@ -413,15 +401,15 @@ static int frr_confd_cdb_read_cb_abort(int fd, int *subp, int reslen)
 	return 0;
 }
 
-static void frr_confd_cdb_read_cb(struct thread *thread)
+static void frr_confd_cdb_read_cb(struct event *thread)
 {
-	int fd = THREAD_FD(thread);
+	int fd = EVENT_FD(thread);
 	enum cdb_sub_notification cdb_ev;
 	int flags;
 	int *subp = NULL;
 	int reslen = 0;
 
-	thread_add_read(master, frr_confd_cdb_read_cb, NULL, fd, &t_cdb_sub);
+	event_add_read(master, frr_confd_cdb_read_cb, NULL, fd, &t_cdb_sub);
 
 	if (cdb_read_subscription_socket2(fd, &cdb_ev, &flags, &subp, &reslen)
 	    != CONFD_OK) {
@@ -586,8 +574,8 @@ static int frr_confd_init_cdb(void)
 	}
 	pthread_detach(cdb_trigger_thread);
 
-	thread_add_read(master, frr_confd_cdb_read_cb, NULL, cdb_sub_sock,
-			&t_cdb_sub);
+	event_add_read(master, frr_confd_cdb_read_cb, NULL, cdb_sub_sock,
+		       &t_cdb_sub);
 
 	return 0;
 
@@ -600,7 +588,7 @@ error:
 static void frr_confd_finish_cdb(void)
 {
 	if (cdb_sub_sock > 0) {
-		THREAD_OFF(t_cdb_sub);
+		EVENT_OFF(t_cdb_sub);
 		cdb_close(cdb_sub_sock);
 	}
 }
@@ -1185,22 +1173,23 @@ static int frr_confd_dp_read(struct confd_daemon_ctx *dctx, int fd)
 	return 0;
 }
 
-static void frr_confd_dp_ctl_read(struct thread *thread)
+static void frr_confd_dp_ctl_read(struct event *thread)
 {
-	struct confd_daemon_ctx *dctx = THREAD_ARG(thread);
-	int fd = THREAD_FD(thread);
+	struct confd_daemon_ctx *dctx = EVENT_ARG(thread);
+	int fd = EVENT_FD(thread);
 
-	thread_add_read(master, frr_confd_dp_ctl_read, dctx, fd, &t_dp_ctl);
+	event_add_read(master, frr_confd_dp_ctl_read, dctx, fd, &t_dp_ctl);
 
 	frr_confd_dp_read(dctx, fd);
 }
 
-static void frr_confd_dp_worker_read(struct thread *thread)
+static void frr_confd_dp_worker_read(struct event *thread)
 {
-	struct confd_daemon_ctx *dctx = THREAD_ARG(thread);
-	int fd = THREAD_FD(thread);
+	struct confd_daemon_ctx *dctx = EVENT_ARG(thread);
+	int fd = EVENT_FD(thread);
 
-	thread_add_read(master, frr_confd_dp_worker_read, dctx, fd, &t_dp_worker);
+	event_add_read(master, frr_confd_dp_worker_read, dctx, fd,
+		       &t_dp_worker);
 
 	frr_confd_dp_read(dctx, fd);
 }
@@ -1332,10 +1321,10 @@ static int frr_confd_init_dp(const char *program_name)
 		goto error;
 	}
 
-	thread_add_read(master, frr_confd_dp_ctl_read, dctx, dp_ctl_sock,
-			&t_dp_ctl);
-	thread_add_read(master, frr_confd_dp_worker_read, dctx, dp_worker_sock,
-			&t_dp_worker);
+	event_add_read(master, frr_confd_dp_ctl_read, dctx, dp_ctl_sock,
+		       &t_dp_ctl);
+	event_add_read(master, frr_confd_dp_worker_read, dctx, dp_worker_sock,
+		       &t_dp_worker);
 
 	return 0;
 
@@ -1348,11 +1337,11 @@ error:
 static void frr_confd_finish_dp(void)
 {
 	if (dp_worker_sock > 0) {
-		THREAD_OFF(t_dp_worker);
+		EVENT_OFF(t_dp_worker);
 		close(dp_worker_sock);
 	}
 	if (dp_ctl_sock > 0) {
-		THREAD_OFF(t_dp_ctl);
+		EVENT_OFF(t_dp_ctl);
 		close(dp_ctl_sock);
 	}
 	if (dctx != NULL)
@@ -1476,7 +1465,7 @@ static int frr_confd_finish(void)
 	return 0;
 }
 
-static int frr_confd_module_late_init(struct thread_master *tm)
+static int frr_confd_module_late_init(struct event_loop *tm)
 {
 	master = tm;
 
