@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Prefix related functions.
  * Copyright (C) 1997, 98, 99 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -138,6 +123,23 @@ afi_t family2afi(int family)
 	return 0;
 }
 
+const char *afi2str_lower(afi_t afi)
+{
+	switch (afi) {
+	case AFI_IP:
+		return "ipv4";
+	case AFI_IP6:
+		return "ipv6";
+	case AFI_L2VPN:
+		return "l2vpn";
+	case AFI_MAX:
+	case AFI_UNSPEC:
+		return "bad-value";
+	}
+
+	assert(!"Reached end of function we should never reach");
+}
+
 const char *afi2str(afi_t afi)
 {
 	switch (afi) {
@@ -148,11 +150,11 @@ const char *afi2str(afi_t afi)
 	case AFI_L2VPN:
 		return "l2vpn";
 	case AFI_MAX:
+	case AFI_UNSPEC:
 		return "bad-value";
-	default:
-		break;
 	}
-	return NULL;
+
+	assert(!"Reached end of function we should never reach");
 }
 
 const char *safi2str(safi_t safi)
@@ -172,9 +174,12 @@ const char *safi2str(safi_t safi)
 		return "labeled-unicast";
 	case SAFI_FLOWSPEC:
 		return "flowspec";
-	default:
+	case SAFI_UNSPEC:
+	case SAFI_MAX:
 		return "unknown";
 	}
+
+	assert(!"Reached end of function we should never reach");
 }
 
 /* If n includes p prefix then return 1 else return 0. */
@@ -256,7 +261,7 @@ int evpn_type5_prefix_match(const struct prefix *n, const struct prefix *p)
 		return 0;
 
 	prefixlen = evp->prefix.prefix_addr.ip_prefix_length;
-	np = &evp->prefix.prefix_addr.ip.ip.addr;
+	np = evp->prefix.prefix_addr.ip.ip.addrbytes;
 
 	/* If n's prefix is longer than p's one return 0. */
 	if (prefixlen > p->prefixlen)
@@ -350,6 +355,41 @@ void prefix_copy(union prefixptr udest, union prefixconstptr usrc)
 	}
 }
 
+bool evpn_addr_same(const struct evpn_addr *e1, const struct evpn_addr *e2)
+{
+	if (e1->route_type != e2->route_type)
+		return false;
+	if (e1->route_type == BGP_EVPN_AD_ROUTE)
+		return (!memcmp(&e1->ead_addr.esi.val,
+				&e2->ead_addr.esi.val, ESI_BYTES) &&
+			e1->ead_addr.eth_tag == e2->ead_addr.eth_tag &&
+			!ipaddr_cmp(&e1->ead_addr.ip, &e2->ead_addr.ip));
+	if (e1->route_type == BGP_EVPN_MAC_IP_ROUTE)
+		return (e1->macip_addr.eth_tag == e2->macip_addr.eth_tag &&
+			e1->macip_addr.ip_prefix_length
+				== e2->macip_addr.ip_prefix_length &&
+			!memcmp(&e1->macip_addr.mac,
+				&e2->macip_addr.mac, ETH_ALEN) &&
+			!ipaddr_cmp(&e1->macip_addr.ip, &e2->macip_addr.ip));
+	if (e1->route_type == BGP_EVPN_IMET_ROUTE)
+		return (e1->imet_addr.eth_tag == e2->imet_addr.eth_tag &&
+			e1->imet_addr.ip_prefix_length
+				== e2->imet_addr.ip_prefix_length &&
+			!ipaddr_cmp(&e1->imet_addr.ip, &e2->imet_addr.ip));
+	if (e1->route_type == BGP_EVPN_ES_ROUTE)
+		return (!memcmp(&e1->es_addr.esi.val,
+				&e2->es_addr.esi.val, ESI_BYTES) &&
+			e1->es_addr.ip_prefix_length
+				== e2->es_addr.ip_prefix_length &&
+			!ipaddr_cmp(&e1->es_addr.ip, &e2->es_addr.ip));
+	if (e1->route_type == BGP_EVPN_IP_PREFIX_ROUTE)
+		return (e1->prefix_addr.eth_tag == e2->prefix_addr.eth_tag &&
+			e1->prefix_addr.ip_prefix_length
+				== e2->prefix_addr.ip_prefix_length &&
+			!ipaddr_cmp(&e1->prefix_addr.ip, &e2->prefix_addr.ip));
+	return true;
+}
+
 /*
  * Return 1 if the address/netmask contained in the prefix structure
  * is the same, and else return 0.  For this routine, 'same' requires
@@ -382,8 +422,7 @@ int prefix_same(union prefixconstptr up1, union prefixconstptr up2)
 				    sizeof(struct ethaddr)))
 				return 1;
 		if (p1->family == AF_EVPN)
-			if (!memcmp(&p1->u.prefix_evpn, &p2->u.prefix_evpn,
-				    sizeof(struct evpn_addr)))
+			if (evpn_addr_same(&p1->u.prefix_evpn, &p2->u.prefix_evpn))
 				return 1;
 		if (p1->family == AF_FLOWSPEC) {
 			if (p1->u.prefix_flowspec.family !=
@@ -1394,7 +1433,7 @@ bool ipv4_unicast_valid(const struct in_addr *addr)
 	if (IPV4_CLASS_D(ip))
 		return false;
 
-	if (IPV4_CLASS_E(ip)) {
+	if (IPV4_NET0(ip) || IPV4_NET127(ip) || IPV4_CLASS_E(ip)) {
 		if (cmd_allow_reserved_ranges_get())
 			return true;
 		else
@@ -1404,6 +1443,65 @@ bool ipv4_unicast_valid(const struct in_addr *addr)
 	return true;
 }
 
+static int ipaddr2prefix(const struct ipaddr *ip, uint16_t prefixlen,
+			 struct prefix *p)
+{
+	switch (ip->ipa_type) {
+	case (IPADDR_V4):
+		p->family = AF_INET;
+		p->u.prefix4 = ip->ipaddr_v4;
+		p->prefixlen = prefixlen;
+		break;
+	case (IPADDR_V6):
+		p->family = AF_INET6;
+		p->u.prefix6 = ip->ipaddr_v6;
+		p->prefixlen = prefixlen;
+		break;
+	case (IPADDR_NONE):
+		p->family = AF_UNSPEC;
+		break;
+	}
+
+	return 0;
+}
+
+/*
+ * Convert type-2 and type-5 evpn route prefixes into the more
+ * general ipv4/ipv6 prefix types so we can match prefix lists
+ * and such.
+ */
+int evpn_prefix2prefix(const struct prefix *evpn, struct prefix *to)
+{
+	const struct evpn_addr *addr;
+
+	if (evpn->family != AF_EVPN)
+		return -1;
+
+	addr = &evpn->u.prefix_evpn;
+
+	switch (addr->route_type) {
+	case BGP_EVPN_MAC_IP_ROUTE:
+		if (IS_IPADDR_V4(&addr->macip_addr.ip))
+			ipaddr2prefix(&addr->macip_addr.ip, IPV4_MAX_BITLEN,
+				      to);
+		else if (IS_IPADDR_V6(&addr->macip_addr.ip))
+			ipaddr2prefix(&addr->macip_addr.ip, IPV6_MAX_BITLEN,
+				      to);
+		else
+			return -1; /* mac only? */
+
+		break;
+	case BGP_EVPN_IP_PREFIX_ROUTE:
+		ipaddr2prefix(&addr->prefix_addr.ip,
+			      addr->prefix_addr.ip_prefix_length, to);
+		break;
+	default:
+		return -1;
+	}
+
+	return 0;
+}
+
 printfrr_ext_autoreg_p("EA", printfrr_ea);
 static ssize_t printfrr_ea(struct fbuf *buf, struct printfrr_eargs *ea,
 			   const void *ptr)
@@ -1432,7 +1530,7 @@ static ssize_t printfrr_ia(struct fbuf *buf, struct printfrr_eargs *ea,
 		ea->fmt++;
 	}
 
-	if (!ipa)
+	if (!ipa || !ipa->ipa_type)
 		return bputs(buf, "(null)");
 
 	if (use_star) {
@@ -1450,7 +1548,7 @@ static ssize_t printfrr_ia(struct fbuf *buf, struct printfrr_eargs *ea,
 				return bputch(buf, '*');
 			break;
 
-		default:
+		case IPADDR_NONE:
 			break;
 		}
 	}
