@@ -8,7 +8,7 @@
  * Chris Torek.
  *
  * Copyright (c) 2011 The FreeBSD Foundation
- * All rights reserved.
+ *
  * Portions of this software were developed by David Chisnall
  * under sponsorship from the FreeBSD Foundation.
  *
@@ -340,7 +340,7 @@ reswitch:	switch (ch) {
 			if (width >= 0)
 				goto rflag;
 			width = -width;
-			/* FALLTHROUGH */
+			fallthrough;
 		case '-':
 			flags |= LADJUST;
 			goto rflag;
@@ -419,9 +419,22 @@ reswitch:	switch (ch) {
 		case 'z':
 			flags |= SIZET;
 			goto rflag;
+		case 'B':
+		case 'b':
+			if (flags & INTMAX_SIZE)
+				ujval = UJARG();
+			else
+				ulval = UARG();
+			base = 2;
+			/* leading 0b/B only if non-zero */
+			if (flags & ALT &&
+			    (flags & INTMAX_SIZE ? ujval != 0 : ulval != 0))
+				ox[1] = ch;
+			goto nosign;
+			break;
 		case 'C':
 			flags |= LONGINT;
-			/*FALLTHROUGH*/
+			fallthrough;
 		case 'c':
 #ifdef WCHAR_SUPPORT
 			if (flags & LONGINT) {
@@ -447,7 +460,7 @@ reswitch:	switch (ch) {
 			break;
 		case 'D':
 			flags |= LONGINT;
-			/*FALLTHROUGH*/
+			fallthrough;
 		case 'd':
 		case 'i':
 			if (flags & INTMAX_SIZE)
@@ -504,14 +517,20 @@ reswitch:	switch (ch) {
 				fmt[4] = ch;
 				fmt[5] = '\0';
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 				snprintf(buf, sizeof(buf), fmt, prec, arg);
+#pragma GCC diagnostic pop
 			} else {
 				double arg = GETARG(double);
 				char fmt[5] = "%.*";
 				fmt[3] = ch;
 				fmt[4] = '\0';
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 				snprintf(buf, sizeof(buf), fmt, prec, arg);
+#pragma GCC diagnostic pop
 			}
 			cp = buf;
 			/* for proper padding */
@@ -532,7 +551,7 @@ reswitch:	switch (ch) {
 			break;
 		case 'O':
 			flags |= LONGINT;
-			/*FALLTHROUGH*/
+			fallthrough;
 		case 'o':
 			if (flags & INTMAX_SIZE)
 				ujval = UJARG();
@@ -576,7 +595,7 @@ reswitch:	switch (ch) {
 			goto nosign;
 		case 'S':
 			flags |= LONGINT;
-			/*FALLTHROUGH*/
+			fallthrough;
 		case 's':
 #ifdef WCHAR_SUPPORT
 			if (flags & LONGINT) {
@@ -602,7 +621,7 @@ reswitch:	switch (ch) {
 			break;
 		case 'U':
 			flags |= LONGINT;
-			/*FALLTHROUGH*/
+			fallthrough;
 		case 'u':
 			if (flags & INTMAX_SIZE)
 				ujval = UJARG();
