@@ -1,23 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Route map northbound CLI implementation.
  *
  * Copyright (C) 2019 Network Device Education Foundation, Inc. ("NetDEF")
  *                    Rafael Zalamena
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA.
  */
 
 #include <zebra.h>
@@ -26,9 +12,7 @@
 #include "lib/northbound_cli.h"
 #include "lib/routemap.h"
 
-#ifndef VTYSH_EXTRACT_PL
 #include "lib/routemap_cli_clippy.c"
-#endif /* VTYSH_EXTRACT_PL */
 
 #define ROUTE_MAP_CMD_STR \
 	"Create route-map or enter route-map command mode\n" \
@@ -106,8 +90,8 @@ DEFPY_YANG(
 int route_map_instance_cmp(const struct lyd_node *dnode1,
 			   const struct lyd_node *dnode2)
 {
-	uint16_t seq1 = yang_dnode_get_uint16(dnode1, "./sequence");
-	uint16_t seq2 = yang_dnode_get_uint16(dnode2, "./sequence");
+	uint16_t seq1 = yang_dnode_get_uint16(dnode1, "sequence");
+	uint16_t seq2 = yang_dnode_get_uint16(dnode2, "sequence");
 
 	return seq1 - seq2;
 }
@@ -116,8 +100,8 @@ void route_map_instance_show(struct vty *vty, const struct lyd_node *dnode,
 			     bool show_defaults)
 {
 	const char *name = yang_dnode_get_string(dnode, "../name");
-	const char *action = yang_dnode_get_string(dnode, "./action");
-	const char *sequence = yang_dnode_get_string(dnode, "./sequence");
+	const char *action = yang_dnode_get_string(dnode, "action");
+	const char *sequence = yang_dnode_get_string(dnode, "sequence");
 
 	vty_out(vty, "route-map %s %s %s\n", name, action, sequence);
 
@@ -204,7 +188,7 @@ DEFPY_YANG(
 DEFPY_YANG(
 	match_ip_address_prefix_list,
 	match_ip_address_prefix_list_cmd,
-	"match ip address prefix-list PREFIXLIST_NAME$name",
+	"match ip address prefix-list PREFIXLIST4_NAME$name",
 	MATCH_STR
 	IP_STR
 	"Match address of route\n"
@@ -225,7 +209,7 @@ DEFPY_YANG(
 
 DEFPY_YANG(
 	no_match_ip_address_prefix_list, no_match_ip_address_prefix_list_cmd,
-	"no match ip address prefix-list [PREFIXLIST_NAME]",
+	"no match ip address prefix-list [PREFIXLIST4_NAME]",
 	NO_STR
 	MATCH_STR
 	IP_STR
@@ -281,7 +265,7 @@ DEFPY_YANG(
 DEFPY_YANG(
 	match_ip_next_hop_prefix_list,
 	match_ip_next_hop_prefix_list_cmd,
-	"match ip next-hop prefix-list PREFIXLIST_NAME$name",
+	"match ip next-hop prefix-list PREFIXLIST4_NAME$name",
 	MATCH_STR
 	IP_STR
 	"Match next-hop address of route\n"
@@ -303,7 +287,7 @@ DEFPY_YANG(
 DEFPY_YANG(
 	no_match_ip_next_hop_prefix_list,
 	no_match_ip_next_hop_prefix_list_cmd,
-	"no match ip next-hop prefix-list [PREFIXLIST_NAME]",
+	"no match ip next-hop prefix-list [PREFIXLIST4_NAME]",
 	NO_STR
 	MATCH_STR
 	IP_STR
@@ -395,7 +379,7 @@ DEFPY_YANG(
 
 DEFPY_YANG(
 	match_ipv6_address_prefix_list, match_ipv6_address_prefix_list_cmd,
-	"match ipv6 address prefix-list PREFIXLIST_NAME$name",
+	"match ipv6 address prefix-list PREFIXLIST6_NAME$name",
 	MATCH_STR
 	IPV6_STR
 	"Match address of route\n"
@@ -417,7 +401,7 @@ DEFPY_YANG(
 DEFPY_YANG(
 	no_match_ipv6_address_prefix_list,
 	no_match_ipv6_address_prefix_list_cmd,
-	"no match ipv6 address prefix-list [PREFIXLIST_NAME]",
+	"no match ipv6 address prefix-list [PREFIXLIST6_NAME]",
 	NO_STR
 	MATCH_STR
 	IPV6_STR
@@ -542,7 +526,7 @@ DEFPY_YANG(
 void route_map_condition_show(struct vty *vty, const struct lyd_node *dnode,
 			      bool show_defaults)
 {
-	const char *condition = yang_dnode_get_string(dnode, "./condition");
+	const char *condition = yang_dnode_get_string(dnode, "condition");
 	const struct lyd_node *ln;
 	const char *acl;
 
@@ -615,11 +599,14 @@ void route_map_condition_show(struct vty *vty, const struct lyd_node *dnode,
 			yang_dnode_get_string(
 				dnode,
 				"./rmap-match-condition/frr-zebra-route-map:ipv4-prefix-length"));
-	} else if (IS_MATCH_SRC_PROTO(condition)) {
+	} else if (IS_MATCH_SRC_PROTO(condition) ||
+		   IS_MATCH_BGP_SRC_PROTO(condition)) {
 		vty_out(vty, " match source-protocol %s\n",
 			yang_dnode_get_string(
 				dnode,
-				"./rmap-match-condition/frr-zebra-route-map:source-protocol"));
+				IS_MATCH_SRC_PROTO(condition)
+					? "./rmap-match-condition/frr-zebra-route-map:source-protocol"
+					: "./rmap-match-condition/frr-bgp-route-map:source-protocol"));
 	} else if (IS_MATCH_SRC_INSTANCE(condition)) {
 		vty_out(vty, " match source-instance %s\n",
 			yang_dnode_get_string(
@@ -742,6 +729,10 @@ void route_map_condition_show(struct vty *vty, const struct lyd_node *dnode,
 			    dnode,
 			    "./rmap-match-condition/frr-bgp-route-map:comm-list/comm-list-name-exact-match"))
 			vty_out(vty, " exact-match");
+		if (yang_dnode_get_bool(
+			    dnode,
+			    "./rmap-match-condition/frr-bgp-route-map:comm-list/comm-list-name-any"))
+			vty_out(vty, " any");
 		vty_out(vty, "\n");
 	} else if (IS_MATCH_LCOMMUNITY(condition)) {
 		vty_out(vty, " match large-community %s",
@@ -752,6 +743,10 @@ void route_map_condition_show(struct vty *vty, const struct lyd_node *dnode,
 			    dnode,
 			    "./rmap-match-condition/frr-bgp-route-map:comm-list/comm-list-name-exact-match"))
 			vty_out(vty, " exact-match");
+		if (yang_dnode_get_bool(
+			    dnode,
+			    "./rmap-match-condition/frr-bgp-route-map:comm-list/comm-list-name-any"))
+			vty_out(vty, " any");
 		vty_out(vty, "\n");
 	} else if (IS_MATCH_EXTCOMMUNITY(condition)) {
 		vty_out(vty, " match extcommunity %s\n",
@@ -906,6 +901,76 @@ DEFPY_YANG(
 	return nb_cli_apply_changes(vty, NULL);
 }
 
+DEFPY_YANG(set_min_metric, set_min_metric_cmd,
+	   "set min-metric <(0-4294967295)$metric>",
+	   SET_STR
+	   "Minimum metric value for destination routing protocol\n"
+	   "Minimum metric value\n")
+{
+	const char *xpath =
+		"./set-action[action='frr-route-map:set-min-metric']";
+	char xpath_value[XPATH_MAXLEN];
+	char value[64];
+
+	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
+
+	snprintf(xpath_value, sizeof(xpath_value),
+		 "%s/rmap-set-action/min-metric", xpath);
+	snprintf(value, sizeof(value), "%s", metric_str);
+
+	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, value);
+
+	return nb_cli_apply_changes(vty, NULL);
+}
+
+DEFPY_YANG(no_set_min_metric, no_set_min_metric_cmd,
+	   "no set min-metric [(0-4294967295)]",
+	   NO_STR SET_STR
+	   "Minimum metric value for destination routing protocol\n"
+	   "Minumum metric value\n")
+{
+	const char *xpath =
+		"./set-action[action='frr-route-map:set-min-metric']";
+
+	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
+	return nb_cli_apply_changes(vty, NULL);
+}
+
+DEFPY_YANG(set_max_metric, set_max_metric_cmd,
+	   "set max-metric <(0-4294967295)$metric>",
+	   SET_STR
+	   "Maximum metric value for destination routing protocol\n"
+	   "Miximum metric value\n")
+{
+	const char *xpath =
+		"./set-action[action='frr-route-map:set-max-metric']";
+	char xpath_value[XPATH_MAXLEN];
+	char value[64];
+
+	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
+
+	snprintf(xpath_value, sizeof(xpath_value),
+		 "%s/rmap-set-action/max-metric", xpath);
+	snprintf(value, sizeof(value), "%s", metric_str);
+
+	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, value);
+
+	return nb_cli_apply_changes(vty, NULL);
+}
+
+DEFPY_YANG(no_set_max_metric, no_set_max_metric_cmd,
+	   "no set max-metric [(0-4294967295)]",
+	   NO_STR SET_STR
+	   "Maximum Metric value for destination routing protocol\n"
+	   "Maximum metric value\n")
+{
+	const char *xpath =
+		"./set-action[action='frr-route-map:set-max-metric']";
+
+	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
+	return nb_cli_apply_changes(vty, NULL);
+}
+
 DEFPY_YANG(
 	set_tag, set_tag_cmd,
 	"set tag (1-4294967295)$tag",
@@ -985,7 +1050,7 @@ DEFUN_YANG (no_set_srte_color,
 void route_map_action_show(struct vty *vty, const struct lyd_node *dnode,
 			   bool show_defaults)
 {
-	const char *action = yang_dnode_get_string(dnode, "./action");
+	const char *action = yang_dnode_get_string(dnode, "action");
 	const struct lyd_node *ln;
 	const char *acl;
 
@@ -1027,9 +1092,17 @@ void route_map_action_show(struct vty *vty, const struct lyd_node *dnode,
 				yang_dnode_get_string(
 					dnode, "./rmap-set-action/value"));
 		}
+	} else if (IS_SET_MIN_METRIC(action)) {
+		vty_out(vty, " set min-metric %s\n",
+			yang_dnode_get_string(dnode,
+					      "./rmap-set-action/min-metric"));
+	} else if (IS_SET_MAX_METRIC(action)) {
+		vty_out(vty, " set max-metric %s\n",
+			yang_dnode_get_string(dnode,
+					      "./rmap-set-action/max-metric"));
 	} else if (IS_SET_TAG(action)) {
 		vty_out(vty, " set tag %s\n",
-			yang_dnode_get_string(dnode, "./rmap-set-action/tag"));
+			yang_dnode_get_string(dnode, "rmap-set-action/tag"));
 	} else if (IS_SET_SR_TE_COLOR(action)) {
 		vty_out(vty, " set sr-te color %s\n",
 			yang_dnode_get_string(dnode,
@@ -1089,6 +1162,11 @@ void route_map_action_show(struct vty *vty, const struct lyd_node *dnode,
 				"./rmap-set-action/frr-bgp-route-map:origin"));
 	} else if (IS_SET_ATOMIC_AGGREGATE(action)) {
 		vty_out(vty, " set atomic-aggregate\n");
+	} else if (IS_SET_AIGP_METRIC(action)) {
+		vty_out(vty, " set aigp-metric %s\n",
+			yang_dnode_get_string(
+				dnode,
+				"./rmap-set-action/frr-bgp-route-map:aigp-metric"));
 	} else if (IS_SET_ORIGINATOR_ID(action)) {
 		vty_out(vty, " set originator-id %s\n",
 			yang_dnode_get_string(
@@ -1116,6 +1194,16 @@ void route_map_action_show(struct vty *vty, const struct lyd_node *dnode,
 		assert(acl);
 
 		vty_out(vty, " set large-comm-list %s delete\n", acl);
+	} else if (IS_SET_EXTCOMM_LIST_DEL(action)) {
+		acl = NULL;
+		ln = yang_dnode_get(dnode, "rmap-set-action/frr-bgp-route-map:comm-list-name");
+
+		if (ln)
+			acl = yang_dnode_get_string(ln, NULL);
+
+		assert(acl);
+
+		vty_out(vty, " set extended-comm-list %s delete\n", acl);
 	} else if (IS_SET_LCOMMUNITY(action)) {
 		if (yang_dnode_exists(
 			    dnode,
@@ -1151,6 +1239,11 @@ void route_map_action_show(struct vty *vty, const struct lyd_node *dnode,
 			yang_dnode_get_string(
 				dnode,
 				"./rmap-set-action/frr-bgp-route-map:extcommunity-rt"));
+	} else if (IS_SET_EXTCOMMUNITY_NT(action)) {
+		vty_out(vty, " set extcommunity nt %s\n",
+			yang_dnode_get_string(
+				dnode,
+				"./rmap-set-action/frr-bgp-route-map:extcommunity-nt"));
 	} else if (IS_SET_EXTCOMMUNITY_SOO(action)) {
 		vty_out(vty, " set extcommunity soo %s\n",
 			yang_dnode_get_string(
@@ -1184,6 +1277,11 @@ void route_map_action_show(struct vty *vty, const struct lyd_node *dnode,
 			strlcat(str, " non-transitive", sizeof(str));
 
 		vty_out(vty, " set extcommunity bandwidth %s\n", str);
+	} else if (IS_SET_EXTCOMMUNITY_COLOR(action)) {
+		vty_out(vty, " set extcommunity color %s\n",
+			yang_dnode_get_string(
+				dnode,
+				"./rmap-set-action/frr-bgp-route-map:extcommunity-color"));
 	} else if (IS_SET_EXTCOMMUNITY_NONE(action)) {
 		if (yang_dnode_get_bool(
 			    dnode,
@@ -1562,6 +1660,12 @@ void route_map_cli_init(void)
 	install_element(RMAP_NODE, &set_metric_cmd);
 	install_element(RMAP_NODE, &no_set_metric_cmd);
 
+	install_element(RMAP_NODE, &set_min_metric_cmd);
+	install_element(RMAP_NODE, &no_set_min_metric_cmd);
+
+	install_element(RMAP_NODE, &set_max_metric_cmd);
+	install_element(RMAP_NODE, &no_set_max_metric_cmd);
+
 	install_element(RMAP_NODE, &set_tag_cmd);
 	install_element(RMAP_NODE, &no_set_tag_cmd);
 
