@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2018  NetDEF, Inc.
  *                     Renato Westphal
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -58,6 +45,7 @@
 		}                                                              \
 	} while (0)
 
+PRINTFRR(2, 0)
 static inline const char *
 yang_dnode_xpath_get_canon(const struct lyd_node *dnode, const char *xpath_fmt,
 			   va_list ap)
@@ -75,6 +63,7 @@ yang_dnode_xpath_get_canon(const struct lyd_node *dnode, const char *xpath_fmt,
 	return lyd_get_value(&__dleaf->node);
 }
 
+PRINTFRR(2, 0)
 static inline const struct lyd_value *
 yang_dnode_xpath_get_value(const struct lyd_node *dnode, const char *xpath_fmt,
 			   va_list ap)
@@ -100,7 +89,7 @@ static const char *yang_get_default_value(const char *xpath)
 	const struct lysc_node *snode;
 	const char *value;
 
-	snode = lys_find_path(ly_native_ctx, NULL, xpath, 0);
+	snode = yang_find_snode(ly_native_ctx, xpath, 0);
 	if (snode == NULL) {
 		flog_err(EC_LIB_YANG_UNKNOWN_DATA_PATH,
 			 "%s: unknown data path: %s", __func__, xpath);
@@ -179,10 +168,9 @@ struct yang_data *yang_data_new_dec64(const char *xpath, double value)
 double yang_dnode_get_dec64(const struct lyd_node *dnode, const char *xpath_fmt,
 			    ...)
 {
-	const double denom[19] = {1e0,   1e-1,  1e-2,  1e-3,  1e-4,
-				  1e-5,  1e-6,  1e-7,  1e-8,  1e-9,
-				  1e-10, 1e-11, 1e-12, 1e-13, 1e-14,
-				  1e-15, 1e-16, 1e-17, 1e-18};
+	const double denom[19] = { 1e0,	 1e1,  1e2,  1e3,  1e4,	 1e5,  1e6,
+				   1e7,	 1e8,  1e9,  1e10, 1e11, 1e12, 1e13,
+				   1e14, 1e15, 1e16, 1e17, 1e18 };
 	const struct lysc_type_dec *dectype;
 	const struct lyd_value *dvalue;
 
@@ -190,7 +178,7 @@ double yang_dnode_get_dec64(const struct lyd_node *dnode, const char *xpath_fmt,
 	dectype = (const struct lysc_type_dec *)dvalue->realtype;
 	assert(dectype->basetype == LY_TYPE_DEC64);
 	assert(dectype->fraction_digits < sizeof(denom) / sizeof(*denom));
-	return (double)dvalue->dec64 * denom[dectype->fraction_digits];
+	return (double)dvalue->dec64 / denom[dectype->fraction_digits];
 }
 
 double yang_get_default_dec64(const char *xpath_fmt, ...)
@@ -217,7 +205,7 @@ int yang_str2enum(const char *xpath, const char *value)
 	const struct lysc_type_enum *type;
 	const struct lysc_type_bitenum_item *enums;
 
-	snode = lys_find_path(ly_native_ctx, NULL, xpath, 0);
+	snode = yang_find_snode(ly_native_ctx, xpath, 0);
 	if (snode == NULL) {
 		flog_err(EC_LIB_YANG_UNKNOWN_DATA_PATH,
 			 "%s: unknown data path: %s", __func__, xpath);
@@ -252,7 +240,7 @@ struct yang_data *yang_data_new_enum(const char *xpath, int value)
 	const struct lysc_type_enum *type;
 	const struct lysc_type_bitenum_item *enums;
 
-	snode = lys_find_path(ly_native_ctx, NULL, xpath, 0);
+	snode = yang_find_snode(ly_native_ctx, xpath, 0);
 	if (snode == NULL) {
 		flog_err(EC_LIB_YANG_UNKNOWN_DATA_PATH,
 			 "%s: unknown data path: %s", __func__, xpath);
@@ -1031,6 +1019,13 @@ void yang_str2mac(const char *value, struct ethaddr *mac)
 	(void)prefix_str2mac(value, mac);
 }
 
+void yang_dnode_get_mac(struct ethaddr *mac, const struct lyd_node *dnode,
+			const char *xpath_fmt, ...)
+{
+	const char *canon = YANG_DNODE_XPATH_GET_CANON(dnode, xpath_fmt);
+	(void)prefix_str2mac(canon, mac);
+}
+
 struct yang_data *yang_data_new_date_and_time(const char *xpath, time_t time)
 {
 	struct tm tm;
@@ -1057,6 +1052,17 @@ struct yang_data *yang_data_new_date_and_time(const char *xpath, time_t time)
 	return yang_data_new(xpath, timebuf);
 }
 
+float yang_dnode_get_bandwidth_ieee_float32(const struct lyd_node *dnode,
+					    const char *xpath_fmt, ...)
+{
+	const char *canon = YANG_DNODE_XPATH_GET_CANON(dnode, xpath_fmt);
+	float value;
+
+	assert(sscanf(canon, "%a", &value) == 1);
+
+	return value;
+}
+
 const char *yang_nexthop_type2str(uint32_t ntype)
 {
 	switch (ntype) {
