@@ -1,10 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* NHRP peer functions
  * Copyright (c) 2014-2015 Timo Teräs
- *
- * This file is free software: you may copy, redistribute and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -15,7 +11,7 @@
 
 #include "zebra.h"
 #include "memory.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "hash.h"
 #include "network.h"
 
@@ -47,17 +43,18 @@ static void nhrp_peer_check_delete(struct nhrp_peer *p)
 	debugf(NHRP_DEBUG_COMMON, "Deleting peer ref:%d remote:%pSU local:%pSU",
 	       p->ref, &p->vc->remote.nbma, &p->vc->local.nbma);
 
-	THREAD_OFF(p->t_fallback);
-	THREAD_OFF(p->t_timer);
-	hash_release(nifp->peer_hash, p);
+	EVENT_OFF(p->t_fallback);
+	EVENT_OFF(p->t_timer);
+	if (nifp->peer_hash)
+		hash_release(nifp->peer_hash, p);
 	nhrp_interface_notify_del(p->ifp, &p->ifp_notifier);
 	nhrp_vc_notify_del(p->vc, &p->vc_notifier);
 	XFREE(MTYPE_NHRP_PEER, p);
 }
 
-static void nhrp_peer_notify_up(struct thread *t)
+static void nhrp_peer_notify_up(struct event *t)
 {
-	struct nhrp_peer *p = THREAD_ARG(t);
+	struct nhrp_peer *p = EVENT_ARG(t);
 	struct nhrp_vc *vc = p->vc;
 	struct interface *ifp = p->ifp;
 	struct nhrp_interface *nifp = ifp->info;
@@ -80,14 +77,14 @@ static void __nhrp_peer_check(struct nhrp_peer *p)
 
 	online = nifp->enabled && (!nifp->ipsec_profile || vc->ipsec);
 	if (p->online != online) {
-		THREAD_OFF(p->t_fallback);
+		EVENT_OFF(p->t_fallback);
 		if (online && notifier_active(&p->notifier_list)) {
 			/* If we requested the IPsec connection, delay
 			 * the up notification a bit to allow things
 			 * settle down. This allows IKE to install
 			 * SPDs and SAs. */
-			thread_add_timer_msec(master, nhrp_peer_notify_up, p,
-					      50, &p->t_fallback);
+			event_add_timer_msec(master, nhrp_peer_notify_up, p, 50,
+					     &p->t_fallback);
 		} else {
 			nhrp_peer_ref(p);
 			p->online = online;
@@ -143,7 +140,7 @@ static void nhrp_peer_ifp_notify(struct notifier_block *n, unsigned long cmd)
 					   nhrp_peer_vc_notify);
 			__nhrp_peer_check(p);
 		}
-		/* fallthru */ /* to post config update */
+		fallthrough; /* to post config update */
 	case NOTIFY_INTERFACE_ADDRESS_CHANGED:
 		notifier_call(&p->notifier_list, NOTIFY_PEER_IFCONFIG_CHANGED);
 		break;
@@ -205,12 +202,7 @@ void nhrp_peer_interface_del(struct interface *ifp)
 	debugf(NHRP_DEBUG_COMMON, "Cleaning up undeleted peer entries (%lu)",
 	       nifp->peer_hash ? nifp->peer_hash->count : 0);
 
-	if (nifp->peer_hash) {
-		hash_clean(nifp->peer_hash, do_peer_hash_free);
-		assert(nifp->peer_hash->count == 0);
-		hash_free(nifp->peer_hash);
-		nifp->peer_hash = NULL;
-	}
+	hash_clean_and_free(&nifp->peer_hash, do_peer_hash_free);
 }
 
 struct nhrp_peer *nhrp_peer_get(struct interface *ifp,
@@ -257,9 +249,9 @@ void nhrp_peer_unref(struct nhrp_peer *p)
 	}
 }
 
-static void nhrp_peer_request_timeout(struct thread *t)
+static void nhrp_peer_request_timeout(struct event *t)
 {
-	struct nhrp_peer *p = THREAD_ARG(t);
+	struct nhrp_peer *p = EVENT_ARG(t);
 	struct nhrp_vc *vc = p->vc;
 	struct interface *ifp = p->ifp;
 	struct nhrp_interface *nifp = ifp->info;
@@ -273,21 +265,21 @@ static void nhrp_peer_request_timeout(struct thread *t)
 		p->fallback_requested = 1;
 		vici_request_vc(nifp->ipsec_fallback_profile, &vc->local.nbma,
 				&vc->remote.nbma, p->prio);
-		thread_add_timer(master, nhrp_peer_request_timeout, p, 30,
-				 &p->t_fallback);
+		event_add_timer(master, nhrp_peer_request_timeout, p, 30,
+				&p->t_fallback);
 	} else {
 		p->requested = p->fallback_requested = 0;
 	}
 }
 
-static void nhrp_peer_defer_vici_request(struct thread *t)
+static void nhrp_peer_defer_vici_request(struct event *t)
 {
-	struct nhrp_peer *p = THREAD_ARG(t);
+	struct nhrp_peer *p = EVENT_ARG(t);
 	struct nhrp_vc *vc = p->vc;
 	struct interface *ifp = p->ifp;
 	struct nhrp_interface *nifp = ifp->info;
 
-	THREAD_OFF(p->t_timer);
+	EVENT_OFF(p->t_timer);
 
 	if (p->online) {
 		debugf(NHRP_DEBUG_COMMON,
@@ -296,10 +288,10 @@ static void nhrp_peer_defer_vici_request(struct thread *t)
 	} else {
 		vici_request_vc(nifp->ipsec_profile, &vc->local.nbma,
 				&vc->remote.nbma, p->prio);
-		thread_add_timer(
-			master, nhrp_peer_request_timeout, p,
-			(nifp->ipsec_fallback_profile && !p->prio) ? 15 : 30,
-			&p->t_fallback);
+		event_add_timer(master, nhrp_peer_request_timeout, p,
+				(nifp->ipsec_fallback_profile && !p->prio) ? 15
+									   : 30,
+				&p->t_fallback);
 	}
 }
 
@@ -329,10 +321,10 @@ int nhrp_peer_check(struct nhrp_peer *p, int establish)
 	if (p->prio) {
 		vici_request_vc(nifp->ipsec_profile, &vc->local.nbma,
 				&vc->remote.nbma, p->prio);
-		thread_add_timer(
-			master, nhrp_peer_request_timeout, p,
-			(nifp->ipsec_fallback_profile && !p->prio) ? 15 : 30,
-			&p->t_fallback);
+		event_add_timer(master, nhrp_peer_request_timeout, p,
+				(nifp->ipsec_fallback_profile && !p->prio) ? 15
+									   : 30,
+				&p->t_fallback);
 	} else {
 		/* Maximum timeout is 1 second */
 		int r_time_ms = frr_weak_random() % 1000;
@@ -340,8 +332,8 @@ int nhrp_peer_check(struct nhrp_peer *p, int establish)
 		debugf(NHRP_DEBUG_COMMON,
 		       "Initiating IPsec connection request to %pSU after %d ms:",
 		       &vc->remote.nbma, r_time_ms);
-		thread_add_timer_msec(master, nhrp_peer_defer_vici_request,
-				      p, r_time_ms, &p->t_timer);
+		event_add_timer_msec(master, nhrp_peer_defer_vici_request, p,
+				     r_time_ms, &p->t_timer);
 	}
 
 	return 0;
@@ -1059,7 +1051,7 @@ static void nhrp_peer_forward(struct nhrp_peer *p,
 				 * append our selves to the transit NHS list
 				 */
 				goto err;
-		/* fallthru */
+			fallthrough;
 		case NHRP_EXTENSION_RESPONDER_ADDRESS:
 			/* Supported compulsory extensions, and any
 			 * non-compulsory that is not explicitly handled,
@@ -1229,7 +1221,7 @@ void nhrp_peer_recv(struct nhrp_peer *p, struct zbuf *zb)
 				/* FIXME: send error-indication */
 			}
 		}
-		/* fallthru */ /* FIXME: double check, is this correct? */
+		fallthrough; /* FIXME: double check, is this correct? */
 	case NHRP_ROUTE_OFF_NBMA:
 		if (packet_types[hdr->type].handler) {
 			packet_types[hdr->type].handler(&pp);
