@@ -1,10 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* NHRP vty handling
  * Copyright (c) 2014-2015 Timo Teräs
- *
- * This file is free software: you may copy, redistribute and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
  */
 
 #include "zebra.h"
@@ -126,6 +122,8 @@ DEFUN_NOSH(show_debugging_nhrp, show_debugging_nhrp_cmd,
 			debug_flags_desc[i].str);
 	}
 
+	cmd_show_lib_debugs(vty);
+
 	return CMD_SUCCESS;
 }
 
