@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Copyright (C) 2003 Yasuhiro Ohara
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -24,10 +9,11 @@
 #include "log.h"
 #include "vty.h"
 #include "command.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "linklist.h"
 #include "lib_errors.h"
 #include "checksum.h"
+#include "network.h"
 
 #include "ospf6_proto.h"
 #include "ospf6_lsa.h"
@@ -282,6 +268,18 @@ static struct ospf6_packet *ospf6_packet_new(size_t size)
 	return new;
 }
 
+static struct ospf6_packet *ospf6_packet_dup(struct ospf6_packet *old)
+{
+	struct ospf6_packet *new;
+
+	new = XCALLOC(MTYPE_OSPF6_PACKET, sizeof(struct ospf6_packet));
+	new->s = stream_dup(old->s);
+	new->dst = old->dst;
+	new->length = old->length;
+
+	return new;
+}
+
 static void ospf6_packet_free(struct ospf6_packet *op)
 {
 	if (op->s)
@@ -421,6 +419,25 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 	hello = (struct ospf6_hello *)((caddr_t)oh
 				       + sizeof(struct ospf6_header));
 
+	if ((oi->state == OSPF6_INTERFACE_POINTTOPOINT
+	     || oi->state == OSPF6_INTERFACE_POINTTOMULTIPOINT)
+	    && oi->p2xp_only_cfg_neigh) {
+		/* NEVER, never, ever, do this on broadcast (or NBMA)!
+		 * DR/BDR election requires everyone to talk to everyone else
+		 * only for PtP/PtMP we can be selective in adjacencies!
+		 */
+		struct ospf6_if_p2xp_neighcfg *p2xp_cfg;
+
+		p2xp_cfg = ospf6_if_p2xp_find(oi, src);
+		if (!p2xp_cfg) {
+			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV_HDR))
+				zlog_debug(
+					"ignoring PtP/PtMP hello from %pI6, neighbor not configured",
+					src);
+			return;
+		}
+	}
+
 	/* HelloInterval check */
 	if (ntohs(hello->hello_interval) != oi->hello_interval) {
 		zlog_warn(
@@ -493,7 +510,7 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 	on->hello_in++;
 
 	/* Always override neighbor's source address */
-	memcpy(&on->linklocal_addr, src, sizeof(struct in6_addr));
+	ospf6_neighbor_lladdr_set(on, src);
 
 	/* Neighbor ifindex check */
 	if (on->ifindex != (ifindex_t)ntohl(hello->interface_id)) {
@@ -549,9 +566,9 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 	oi->hello_in++;
 
 	/* Execute neighbor events */
-	thread_execute(master, hello_received, on, 0);
+	event_execute(master, hello_received, on, 0, NULL);
 	if (twoway)
-		thread_execute(master, twoway_received, on, 0);
+		event_execute(master, twoway_received, on, 0, NULL);
 	else {
 		if (OSPF6_GR_IS_ACTIVE_HELPER(on)) {
 			if (IS_DEBUG_OSPF6_GR)
@@ -567,7 +584,7 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 			 * receives one_way hellow when it acts as HELPER for
 			 * that specific neighbor.
 			 */
-			thread_execute(master, oneway_received, on, 0);
+			event_execute(master, oneway_received, on, 0, NULL);
 		}
 	}
 
@@ -608,9 +625,9 @@ static void ospf6_hello_recv(struct in6_addr *src, struct in6_addr *dst,
 
 	/* Schedule interface events */
 	if (backupseen)
-		thread_add_event(master, backup_seen, oi, 0, NULL);
+		event_add_event(master, backup_seen, oi, 0, NULL);
 	if (neighborchange)
-		thread_add_event(master, neighbor_change, oi, 0, NULL);
+		event_add_event(master, neighbor_change, oi, 0, NULL);
 
 	if (neighbor_ifindex_change && on->state == OSPF6_NEIGHBOR_FULL)
 		OSPF6_ROUTER_LSA_SCHEDULE(oi->area);
@@ -638,15 +655,15 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 		return;
 
 	case OSPF6_NEIGHBOR_INIT:
-		thread_execute(master, twoway_received, on, 0);
+		event_execute(master, twoway_received, on, 0, NULL);
 		if (on->state != OSPF6_NEIGHBOR_EXSTART) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV_HDR))
 				zlog_debug(
 					"Neighbor state is not ExStart, ignore");
 			return;
 		}
-	/* else fall through to ExStart */
-	/* fallthru */
+		/* else fall through to ExStart */
+		fallthrough;
 	case OSPF6_NEIGHBOR_EXSTART:
 		/* if neighbor obeys us as our slave, schedule negotiation_done
 		   and process LSA Headers. Otherwise, ignore this message */
@@ -654,7 +671,7 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 		    && !CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_IBIT)
 		    && ntohl(dbdesc->seqnum) == on->dbdesc_seqnum) {
 			/* execute NegotiationDone */
-			thread_execute(master, negotiation_done, on, 0);
+			event_execute(master, negotiation_done, on, 0, NULL);
 
 			/* Record neighbor options */
 			memcpy(on->options, dbdesc->options,
@@ -664,8 +681,8 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 				  on->ospf6_if->interface->vrf->name, on->name);
 			return;
 		}
-	/* fall through to exchange */
-
+		/* fall through to exchange */
+		fallthrough;
 	case OSPF6_NEIGHBOR_EXCHANGE:
 		if (!memcmp(dbdesc, &on->dbdesc_last,
 			    sizeof(struct ospf6_dbdesc))) {
@@ -681,24 +698,24 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 			zlog_warn(
 				"DbDesc recv: Master/Slave bit mismatch Nbr %s",
 				on->name);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
 		if (CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_IBIT)) {
 			zlog_warn("DbDesc recv: Initialize bit mismatch Nbr %s",
 				  on->name);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
 		if (memcmp(on->options, dbdesc->options, sizeof(on->options))) {
 			zlog_warn("DbDesc recv: Option field mismatch Nbr %s",
 				  on->name);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
@@ -707,7 +724,7 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 				"DbDesc recv: Sequence number mismatch Nbr %s (received %#lx, %#lx expected)",
 				on->name, (unsigned long)ntohl(dbdesc->seqnum),
 				(unsigned long)on->dbdesc_seqnum);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
+			event_add_event(master, seqnumber_mismatch, on, 0,
 					 NULL);
 			return;
 		}
@@ -728,7 +745,7 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 		zlog_warn(
 			"DbDesc recv: Not duplicate dbdesc in state %s Nbr %s",
 			ospf6_neighbor_state_str[on->state], on->name);
-		thread_add_event(master, seqnumber_mismatch, on, 0, NULL);
+		event_add_event(master, seqnumber_mismatch, on, 0, NULL);
 		return;
 
 	default:
@@ -772,8 +789,8 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 				zlog_debug(
 					"SeqNumMismatch (E-bit mismatch), discard");
 			ospf6_lsa_delete(his);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
@@ -801,19 +818,19 @@ static void ospf6_dbdesc_recv_master(struct ospf6_header *oh,
 
 	/* schedule send lsreq */
 	if (on->request_list->count)
-		thread_add_event(master, ospf6_lsreq_send, on, 0,
-				 &on->thread_send_lsreq);
+		event_add_event(master, ospf6_lsreq_send, on, 0,
+				&on->thread_send_lsreq);
 
-	THREAD_OFF(on->thread_send_dbdesc);
+	EVENT_OFF(on->thread_send_dbdesc);
 
 	/* More bit check */
 	if (!CHECK_FLAG(dbdesc->bits, OSPF6_DBDESC_MBIT)
 	    && !CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MBIT))
-		thread_add_event(master, exchange_done, on, 0,
-				 &on->thread_exchange_done);
+		event_add_event(master, exchange_done, on, 0,
+				&on->thread_exchange_done);
 	else {
-		thread_add_event(master, ospf6_dbdesc_send_newone, on, 0,
-				 &on->thread_send_dbdesc);
+		event_add_event(master, ospf6_dbdesc_send_newone, on, 0,
+				&on->thread_send_dbdesc);
 	}
 
 	/* save last received dbdesc */
@@ -842,15 +859,15 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 		return;
 
 	case OSPF6_NEIGHBOR_INIT:
-		thread_execute(master, twoway_received, on, 0);
+		event_execute(master, twoway_received, on, 0, NULL);
 		if (on->state != OSPF6_NEIGHBOR_EXSTART) {
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV_HDR))
 				zlog_debug(
 					"Neighbor state is not ExStart, ignore");
 			return;
 		}
-	/* else fall through to ExStart */
-	/* fallthru */
+		/* else fall through to ExStart */
+		fallthrough;
 	case OSPF6_NEIGHBOR_EXSTART:
 		/* If the neighbor is Master, act as Slave. Schedule
 		   negotiation_done
@@ -869,7 +886,7 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			on->dbdesc_seqnum = ntohl(dbdesc->seqnum);
 
 			/* schedule NegotiationDone */
-			thread_execute(master, negotiation_done, on, 0);
+			event_execute(master, negotiation_done, on, 0, NULL);
 
 			/* Record neighbor options */
 			memcpy(on->options, dbdesc->options,
@@ -889,9 +906,9 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV_HDR))
 				zlog_debug(
 					"Duplicated dbdesc causes retransmit");
-			THREAD_OFF(on->thread_send_dbdesc);
-			thread_add_event(master, ospf6_dbdesc_send, on, 0,
-					 &on->thread_send_dbdesc);
+			EVENT_OFF(on->thread_send_dbdesc);
+			event_add_event(master, ospf6_dbdesc_send, on, 0,
+					&on->thread_send_dbdesc);
 			return;
 		}
 
@@ -899,8 +916,8 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			zlog_warn(
 				"DbDesc slave recv: Master/Slave bit mismatch Nbr %s",
 				on->name);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
@@ -908,8 +925,8 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			zlog_warn(
 				"DbDesc slave recv: Initialize bit mismatch Nbr %s",
 				on->name);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
@@ -917,8 +934,8 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			zlog_warn(
 				"DbDesc slave recv: Option field mismatch Nbr %s",
 				on->name);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
@@ -927,7 +944,7 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 				"DbDesc slave recv: Sequence number mismatch Nbr %s (received %#lx, %#lx expected)",
 				on->name, (unsigned long)ntohl(dbdesc->seqnum),
 				(unsigned long)on->dbdesc_seqnum + 1);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
+			event_add_event(master, seqnumber_mismatch, on, 0,
 					 NULL);
 			return;
 		}
@@ -942,16 +959,16 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV_HDR))
 				zlog_debug(
 					"Duplicated dbdesc causes retransmit");
-			THREAD_OFF(on->thread_send_dbdesc);
-			thread_add_event(master, ospf6_dbdesc_send, on, 0,
-					 &on->thread_send_dbdesc);
+			EVENT_OFF(on->thread_send_dbdesc);
+			event_add_event(master, ospf6_dbdesc_send, on, 0,
+					&on->thread_send_dbdesc);
 			return;
 		}
 
 		zlog_warn(
 			"DbDesc slave recv: Not duplicate dbdesc in state %s Nbr %s",
 			ospf6_neighbor_state_str[on->state], on->name);
-		thread_add_event(master, seqnumber_mismatch, on, 0, NULL);
+		event_add_event(master, seqnumber_mismatch, on, 0, NULL);
 		return;
 
 	default:
@@ -991,8 +1008,8 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 			if (IS_OSPF6_DEBUG_MESSAGE(oh->type, RECV))
 				zlog_debug("E-bit mismatch with LSA Headers");
 			ospf6_lsa_delete(his);
-			thread_add_event(master, seqnumber_mismatch, on, 0,
-					 NULL);
+			event_add_event(master, seqnumber_mismatch, on, 0,
+					NULL);
 			return;
 		}
 
@@ -1013,12 +1030,12 @@ static void ospf6_dbdesc_recv_slave(struct ospf6_header *oh,
 
 	/* schedule send lsreq */
 	if (on->request_list->count)
-		thread_add_event(master, ospf6_lsreq_send, on, 0,
-				 &on->thread_send_lsreq);
+		event_add_event(master, ospf6_lsreq_send, on, 0,
+				&on->thread_send_lsreq);
 
-	THREAD_OFF(on->thread_send_dbdesc);
-	thread_add_event(master, ospf6_dbdesc_send_newone, on, 0,
-			 &on->thread_send_dbdesc);
+	EVENT_OFF(on->thread_send_dbdesc);
+	event_add_event(master, ospf6_dbdesc_send_newone, on, 0,
+			&on->thread_send_dbdesc);
 
 	/* save last received dbdesc */
 	memcpy(&on->dbdesc_last, dbdesc, sizeof(struct ospf6_dbdesc));
@@ -1140,7 +1157,7 @@ static void ospf6_lsreq_recv(struct in6_addr *src, struct in6_addr *dst,
 				"Can't find requested lsa [%s Id:%pI4 Adv:%pI4] send badLSReq",
 				ospf6_lstype_name(e->type), &e->id,
 				&e->adv_router);
-			thread_add_event(master, bad_lsreq, on, 0, NULL);
+			event_add_event(master, bad_lsreq, on, 0, NULL);
 			return;
 		}
 
@@ -1150,9 +1167,9 @@ static void ospf6_lsreq_recv(struct in6_addr *src, struct in6_addr *dst,
 	assert(p == OSPF6_MESSAGE_END(oh));
 
 	/* schedule send lsupdate */
-	THREAD_OFF(on->thread_send_lsupdate);
-	thread_add_event(master, ospf6_lsupdate_send_neighbor, on, 0,
-			 &on->thread_send_lsupdate);
+	EVENT_OFF(on->thread_send_lsupdate);
+	event_add_event(master, ospf6_lsupdate_send_neighbor, on, 0,
+			&on->thread_send_lsupdate);
 }
 
 /* Verify, that the specified memory area contains exactly N valid IPv6
@@ -1921,18 +1938,18 @@ static int ospf6_read_helper(int sockfd, struct ospf6 *ospf6)
 	return OSPF6_READ_CONTINUE;
 }
 
-void ospf6_receive(struct thread *thread)
+void ospf6_receive(struct event *thread)
 {
 	int sockfd;
 	struct ospf6 *ospf6;
 	int count = 0;
 
 	/* add next read thread */
-	ospf6 = THREAD_ARG(thread);
-	sockfd = THREAD_FD(thread);
+	ospf6 = EVENT_ARG(thread);
+	sockfd = EVENT_FD(thread);
 
-	thread_add_read(master, ospf6_receive, ospf6, ospf6->fd,
-			&ospf6->t_ospf6_receive);
+	event_add_read(master, ospf6_receive, ospf6, ospf6->fd,
+		       &ospf6->t_ospf6_receive);
 
 	while (count < ospf6->write_oi_count) {
 		count++;
@@ -2026,6 +2043,9 @@ static void ospf6_auth_trailer_copy_keychain_key(struct ospf6_interface *oi)
 			 * these values
 			 */
 			oi->at_data.hash_algo = key->hash_algo;
+			if (oi->at_data.auth_key)
+				XFREE(MTYPE_OSPF6_AUTH_MANUAL_KEY,
+				      oi->at_data.auth_key);
 			oi->at_data.auth_key = XSTRDUP(
 				MTYPE_OSPF6_AUTH_MANUAL_KEY, key->string);
 			oi->at_data.key_id = key->index;
@@ -2093,9 +2113,9 @@ static uint16_t ospf6_make_hello(struct ospf6_interface *oi, struct stream *s)
 	return length;
 }
 
-static void ospf6_write(struct thread *thread)
+static void ospf6_write(struct event *thread)
 {
-	struct ospf6 *ospf6 = THREAD_ARG(thread);
+	struct ospf6 *ospf6 = EVENT_ARG(thread);
 	struct ospf6_interface *oi;
 	struct ospf6_header *oh;
 	struct ospf6_packet *op;
@@ -2243,17 +2263,30 @@ static void ospf6_write(struct thread *thread)
 
 	/* If packets still remain in queue, call write thread. */
 	if (!list_isempty(ospf6->oi_write_q))
-		thread_add_write(master, ospf6_write, ospf6, ospf6->fd,
-				 &ospf6->t_write);
+		event_add_write(master, ospf6_write, ospf6, ospf6->fd,
+				&ospf6->t_write);
 }
 
-void ospf6_hello_send(struct thread *thread)
+void ospf6_hello_send(struct event *thread)
 {
 	struct ospf6_interface *oi;
-	struct ospf6_packet *op;
-	uint16_t length = OSPF6_HEADER_SIZE;
 
-	oi = (struct ospf6_interface *)THREAD_ARG(thread);
+	oi = (struct ospf6_interface *)EVENT_ARG(thread);
+
+	/* Check if the GR hello-delay is active. */
+	if (oi->gr.hello_delay.t_grace_send)
+		return;
+
+	/* Check if config is still being processed */
+	if (event_is_scheduled(t_ospf6_cfg)) {
+		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_HELLO, SEND))
+			zlog_debug(
+				"Suppressing Hello on interface %s during config load",
+				oi->interface->name);
+		event_add_timer(master, ospf6_hello_send, oi,
+				oi->hello_interval, &oi->thread_send_hello);
+		return;
+	}
 
 	if (oi->state <= OSPF6_INTERFACE_DOWN) {
 		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_HELLO, SEND_HDR))
@@ -2262,6 +2295,20 @@ void ospf6_hello_send(struct thread *thread)
 		return;
 	}
 
+	event_add_timer(master, ospf6_hello_send, oi, oi->hello_interval,
+			 &oi->thread_send_hello);
+
+	ospf6_hello_send_addr(oi, NULL);
+}
+
+/* used to send polls for PtP/PtMP too */
+void ospf6_hello_send_addr(struct ospf6_interface *oi,
+			   const struct in6_addr *addr)
+{
+	struct ospf6_packet *op;
+	uint16_t length = OSPF6_HEADER_SIZE;
+	bool anything = false;
+
 	op = ospf6_packet_new(oi->ifmtu);
 
 	ospf6_make_header(OSPF6_MESSAGE_TYPE_HELLO, oi, op->s);
@@ -2280,20 +2327,40 @@ void ospf6_hello_send(struct thread *thread)
 	/* Set packet length. */
 	op->length = length;
 
-	op->dst = allspfrouters6;
+	if ((oi->state == OSPF6_INTERFACE_POINTTOPOINT
+	     || oi->state == OSPF6_INTERFACE_POINTTOMULTIPOINT)
+	    && !addr && oi->p2xp_no_multicast_hello) {
+		struct listnode *node;
+		struct ospf6_neighbor *on;
+		struct ospf6_packet *opdup;
 
-	ospf6_fill_hdr_checksum(oi, op);
+		for (ALL_LIST_ELEMENTS_RO(oi->neighbor_list, node, on)) {
+			if (on->state < OSPF6_NEIGHBOR_INIT)
+				/* poll-interval for these */
+				continue;
 
-	/* Add packet to the top of the interface output queue, so that they
-	 * can't get delayed by things like long queues of LS Update packets
-	 */
-	ospf6_packet_add_top(oi, op);
+			opdup = ospf6_packet_dup(op);
+			opdup->dst = on->linklocal_addr;
+			ospf6_fill_hdr_checksum(oi, opdup);
+			ospf6_packet_add_top(oi, opdup);
+			anything = true;
+		}
 
-	/* set next thread */
-	thread_add_timer(master, ospf6_hello_send, oi, oi->hello_interval,
-			 &oi->thread_send_hello);
+		ospf6_packet_free(op);
+	} else {
+		op->dst = addr ? *addr : allspfrouters6;
 
-	OSPF6_MESSAGE_WRITE_ON(oi);
+		/* Add packet to the top of the interface output queue, so that
+		 * they can't get delayed by things like long queues of LS
+		 * Update packets
+		 */
+		ospf6_fill_hdr_checksum(oi, op);
+		ospf6_packet_add_top(oi, op);
+		anything = true;
+	}
+
+	if (anything)
+		OSPF6_MESSAGE_WRITE_ON(oi);
 }
 
 static uint16_t ospf6_make_dbdesc(struct ospf6_neighbor *on, struct stream *s)
@@ -2308,7 +2375,7 @@ static uint16_t ospf6_make_dbdesc(struct ospf6_neighbor *on, struct stream *s)
 	/* if this is initial one, initialize sequence number for DbDesc */
 	if (CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_IBIT)
 	    && (on->dbdesc_seqnum == 0)) {
-		on->dbdesc_seqnum = monotime(NULL);
+		on->dbdesc_seqnum = frr_sequence32_next();
 	}
 
 	/* reserved */
@@ -2331,9 +2398,9 @@ static uint16_t ospf6_make_dbdesc(struct ospf6_neighbor *on, struct stream *s)
 			if ((length + sizeof(struct ospf6_lsa_header)
 			     + OSPF6_HEADER_SIZE)
 			    > ospf6_packet_max(on->ospf6_if)) {
-				ospf6_lsa_unlock(lsa);
+				ospf6_lsa_unlock(&lsa);
 				if (lsanext)
-					ospf6_lsa_unlock(lsanext);
+					ospf6_lsa_unlock(&lsanext);
 				break;
 			}
 			stream_put(s, lsa->header,
@@ -2344,13 +2411,13 @@ static uint16_t ospf6_make_dbdesc(struct ospf6_neighbor *on, struct stream *s)
 	return length;
 }
 
-void ospf6_dbdesc_send(struct thread *thread)
+void ospf6_dbdesc_send(struct event *thread)
 {
 	struct ospf6_neighbor *on;
 	uint16_t length = OSPF6_HEADER_SIZE;
 	struct ospf6_packet *op;
 
-	on = (struct ospf6_neighbor *)THREAD_ARG(thread);
+	on = (struct ospf6_neighbor *)EVENT_ARG(thread);
 
 	if (on->state < OSPF6_NEIGHBOR_EXSTART) {
 		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_DBDESC, SEND))
@@ -2362,9 +2429,9 @@ void ospf6_dbdesc_send(struct thread *thread)
 
 	/* set next thread if master */
 	if (CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MSBIT))
-		thread_add_timer(master, ospf6_dbdesc_send, on,
-				 on->ospf6_if->rxmt_interval,
-				 &on->thread_send_dbdesc);
+		event_add_timer(master, ospf6_dbdesc_send, on,
+				on->ospf6_if->rxmt_interval,
+				&on->thread_send_dbdesc);
 
 	op = ospf6_packet_new(on->ospf6_if->ifmtu);
 	ospf6_make_header(OSPF6_MESSAGE_TYPE_DBDESC, on->ospf6_if, op->s);
@@ -2387,13 +2454,13 @@ void ospf6_dbdesc_send(struct thread *thread)
 	OSPF6_MESSAGE_WRITE_ON(on->ospf6_if);
 }
 
-void ospf6_dbdesc_send_newone(struct thread *thread)
+void ospf6_dbdesc_send_newone(struct event *thread)
 {
 	struct ospf6_neighbor *on;
 	struct ospf6_lsa *lsa, *lsanext;
 	unsigned int size = 0;
 
-	on = (struct ospf6_neighbor *)THREAD_ARG(thread);
+	on = (struct ospf6_neighbor *)EVENT_ARG(thread);
 	ospf6_lsdb_remove_all(on->dbdesc_list);
 
 	/* move LSAs from summary_list to dbdesc_list (within neighbor
@@ -2411,9 +2478,9 @@ void ospf6_dbdesc_send_newone(struct thread *thread)
 
 		if (size + sizeof(struct ospf6_lsa_header)
 		    > ospf6_packet_max(on->ospf6_if)) {
-			ospf6_lsa_unlock(lsa);
+			ospf6_lsa_unlock(&lsa);
 			if (lsanext)
-				ospf6_lsa_unlock(lsanext);
+				ospf6_lsa_unlock(&lsanext);
 			break;
 		}
 
@@ -2429,10 +2496,10 @@ void ospf6_dbdesc_send_newone(struct thread *thread)
 	if (!CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MSBIT) && /* Slave */
 	    !CHECK_FLAG(on->dbdesc_last.bits, OSPF6_DBDESC_MBIT)
 	    && !CHECK_FLAG(on->dbdesc_bits, OSPF6_DBDESC_MBIT))
-		thread_add_event(master, exchange_done, on, 0,
-				 &on->thread_exchange_done);
+		event_add_event(master, exchange_done, on, 0,
+				&on->thread_exchange_done);
 
-	thread_execute(master, ospf6_dbdesc_send, on, 0);
+	event_execute(master, ospf6_dbdesc_send, on, 0, NULL);
 }
 
 static uint16_t ospf6_make_lsreq(struct ospf6_neighbor *on, struct stream *s)
@@ -2443,9 +2510,9 @@ static uint16_t ospf6_make_lsreq(struct ospf6_neighbor *on, struct stream *s)
 	for (ALL_LSDB(on->request_list, lsa, lsanext)) {
 		if ((length + OSPF6_HEADER_SIZE)
 		    > ospf6_packet_max(on->ospf6_if)) {
-			ospf6_lsa_unlock(lsa);
+			ospf6_lsa_unlock(&lsa);
 			if (lsanext)
-				ospf6_lsa_unlock(lsanext);
+				ospf6_lsa_unlock(&lsanext);
 			break;
 		}
 		stream_putw(s, 0); /* reserved */
@@ -2458,7 +2525,7 @@ static uint16_t ospf6_make_lsreq(struct ospf6_neighbor *on, struct stream *s)
 
 	if (last_req != NULL) {
 		if (on->last_ls_req != NULL)
-			on->last_ls_req = ospf6_lsa_unlock(on->last_ls_req);
+			ospf6_lsa_unlock(&on->last_ls_req);
 
 		ospf6_lsa_lock(last_req);
 		on->last_ls_req = last_req;
@@ -2509,13 +2576,13 @@ static uint16_t ospf6_make_lsack_neighbor(struct ospf6_neighbor *on,
 	return length;
 }
 
-void ospf6_lsreq_send(struct thread *thread)
+void ospf6_lsreq_send(struct event *thread)
 {
 	struct ospf6_neighbor *on;
 	struct ospf6_packet *op;
 	uint16_t length = OSPF6_HEADER_SIZE;
 
-	on = (struct ospf6_neighbor *)THREAD_ARG(thread);
+	on = (struct ospf6_neighbor *)EVENT_ARG(thread);
 
 	/* LSReq will be sent only in ExStart or Loading */
 	if (on->state != OSPF6_NEIGHBOR_EXCHANGE
@@ -2529,7 +2596,8 @@ void ospf6_lsreq_send(struct thread *thread)
 
 	/* schedule loading_done if request list is empty */
 	if (on->request_list->count == 0) {
-		thread_add_event(master, loading_done, on, 0, NULL);
+		event_add_event(master, loading_done, on, 0,
+				&on->event_loading_done);
 		return;
 	}
 
@@ -2562,9 +2630,9 @@ void ospf6_lsreq_send(struct thread *thread)
 
 	/* set next thread */
 	if (on->request_list->count != 0) {
-		thread_add_timer(master, ospf6_lsreq_send, on,
-				 on->ospf6_if->rxmt_interval,
-				 &on->thread_send_lsreq);
+		event_add_timer(master, ospf6_lsreq_send, on,
+				on->ospf6_if->rxmt_interval,
+				&on->thread_send_lsreq);
 	}
 }
 
@@ -2572,9 +2640,7 @@ static void ospf6_send_lsupdate(struct ospf6_neighbor *on,
 				struct ospf6_interface *oi,
 				struct ospf6_packet *op)
 {
-
 	if (on) {
-
 		if ((on->ospf6_if->state == OSPF6_INTERFACE_POINTTOPOINT)
 		    || (on->ospf6_if->state == OSPF6_INTERFACE_DR)
 		    || (on->ospf6_if->state == OSPF6_INTERFACE_BDR))
@@ -2591,6 +2657,8 @@ static void ospf6_send_lsupdate(struct ospf6_neighbor *on,
 			op->dst = alldrouters6;
 	}
 	if (oi) {
+		struct ospf6 *ospf6;
+
 		ospf6_fill_hdr_checksum(oi, op);
 		ospf6_packet_add(oi, op);
 		/* If ospf instance is being deleted, send the packet
@@ -2598,12 +2666,27 @@ static void ospf6_send_lsupdate(struct ospf6_neighbor *on,
 		 */
 		if ((oi->area == NULL) || (oi->area->ospf6 == NULL))
 			return;
-		if (oi->area->ospf6->inst_shutdown) {
+
+		ospf6 = oi->area->ospf6;
+		if (ospf6->inst_shutdown) {
 			if (oi->on_write_q == 0) {
-				listnode_add(oi->area->ospf6->oi_write_q, oi);
+				listnode_add(ospf6->oi_write_q, oi);
 				oi->on_write_q = 1;
 			}
-			thread_execute(master, ospf6_write, oi->area->ospf6, 0);
+			/*
+			 * When ospf6d immediately calls event_execute
+			 * for items in the oi_write_q.  The event_execute
+			 * will call ospf6_write and cause the oi_write_q
+			 * to be emptied.  *IF* there is already an event
+			 * scheduled for the oi_write_q by something else
+			 * then when it wakes up in the future and attempts
+			 * to cycle through items in the queue it will
+			 * assert.  Let's stop the t_write event and
+			 * if ospf6_write doesn't finish up the work
+			 * it will schedule itself again.
+			 */
+			event_cancel(&ospf6->t_write);
+			event_execute(master, ospf6_write, ospf6, 0, NULL);
 		} else
 			OSPF6_MESSAGE_WRITE_ON(oi);
 	}
@@ -2687,14 +2770,14 @@ static uint16_t ospf6_make_ls_retrans_list(struct ospf6_neighbor *on,
 	return length;
 }
 
-void ospf6_lsupdate_send_neighbor(struct thread *thread)
+void ospf6_lsupdate_send_neighbor(struct event *thread)
 {
 	struct ospf6_neighbor *on;
 	struct ospf6_packet *op;
 	uint16_t length = OSPF6_HEADER_SIZE;
 	int lsa_cnt = 0;
 
-	on = (struct ospf6_neighbor *)THREAD_ARG(thread);
+	on = (struct ospf6_neighbor *)EVENT_ARG(thread);
 
 	if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_LSUPDATE, SEND_HDR))
 		zlog_debug("LSUpdate to neighbor %s", on->name);
@@ -2745,12 +2828,12 @@ void ospf6_lsupdate_send_neighbor(struct thread *thread)
 		ospf6_packet_free(op);
 
 	if (on->lsupdate_list->count != 0) {
-		thread_add_event(master, ospf6_lsupdate_send_neighbor, on, 0,
-				 &on->thread_send_lsupdate);
+		event_add_event(master, ospf6_lsupdate_send_neighbor, on, 0,
+				&on->thread_send_lsupdate);
 	} else if (on->retrans_list->count != 0) {
-		thread_add_timer(master, ospf6_lsupdate_send_neighbor, on,
-				 on->ospf6_if->rxmt_interval,
-				 &on->thread_send_lsupdate);
+		event_add_timer(master, ospf6_lsupdate_send_neighbor, on,
+				on->ospf6_if->rxmt_interval,
+				&on->thread_send_lsupdate);
 	}
 }
 
@@ -2822,14 +2905,14 @@ static uint16_t ospf6_make_lsupdate_interface(struct ospf6_interface *oi,
 	return length;
 }
 
-void ospf6_lsupdate_send_interface(struct thread *thread)
+void ospf6_lsupdate_send_interface(struct event *thread)
 {
 	struct ospf6_interface *oi;
 	struct ospf6_packet *op;
 	uint16_t length = OSPF6_HEADER_SIZE;
 	int lsa_cnt = 0;
 
-	oi = (struct ospf6_interface *)THREAD_ARG(thread);
+	oi = (struct ospf6_interface *)EVENT_ARG(thread);
 
 	if (oi->state <= OSPF6_INTERFACE_WAITING) {
 		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_LSUPDATE,
@@ -2858,18 +2941,18 @@ void ospf6_lsupdate_send_interface(struct thread *thread)
 		ospf6_packet_free(op);
 
 	if (oi->lsupdate_list->count > 0) {
-		thread_add_event(master, ospf6_lsupdate_send_interface, oi, 0,
-				 &oi->thread_send_lsupdate);
+		event_add_event(master, ospf6_lsupdate_send_interface, oi, 0,
+				&oi->thread_send_lsupdate);
 	}
 }
 
-void ospf6_lsack_send_neighbor(struct thread *thread)
+void ospf6_lsack_send_neighbor(struct event *thread)
 {
 	struct ospf6_neighbor *on;
 	struct ospf6_packet *op;
 	uint16_t length = OSPF6_HEADER_SIZE;
 
-	on = (struct ospf6_neighbor *)THREAD_ARG(thread);
+	on = (struct ospf6_neighbor *)EVENT_ARG(thread);
 
 	if (on->state < OSPF6_NEIGHBOR_EXCHANGE) {
 		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_LSACK, SEND_HDR))
@@ -2904,8 +2987,8 @@ void ospf6_lsack_send_neighbor(struct thread *thread)
 	OSPF6_MESSAGE_WRITE_ON(on->ospf6_if);
 
 	if (on->lsack_list->count > 0)
-		thread_add_event(master, ospf6_lsack_send_neighbor, on, 0,
-				 &on->thread_send_lsack);
+		event_add_event(master, ospf6_lsack_send_neighbor, on, 0,
+				&on->thread_send_lsack);
 }
 
 static uint16_t ospf6_make_lsack_interface(struct ospf6_interface *oi,
@@ -2920,13 +3003,13 @@ static uint16_t ospf6_make_lsack_interface(struct ospf6_interface *oi,
 		    > ospf6_packet_max(oi)) {
 			/* if we run out of packet size/space here,
 			   better to try again soon. */
-			THREAD_OFF(oi->thread_send_lsack);
-			thread_add_event(master, ospf6_lsack_send_interface, oi,
-					 0, &oi->thread_send_lsack);
+			EVENT_OFF(oi->thread_send_lsack);
+			event_add_event(master, ospf6_lsack_send_interface, oi,
+					0, &oi->thread_send_lsack);
 
-			ospf6_lsa_unlock(lsa);
+			ospf6_lsa_unlock(&lsa);
 			if (lsanext)
-				ospf6_lsa_unlock(lsanext);
+				ospf6_lsa_unlock(&lsanext);
 			break;
 		}
 		ospf6_lsa_age_update_to_send(lsa, oi->transdelay);
@@ -2939,13 +3022,13 @@ static uint16_t ospf6_make_lsack_interface(struct ospf6_interface *oi,
 	return length;
 }
 
-void ospf6_lsack_send_interface(struct thread *thread)
+void ospf6_lsack_send_interface(struct event *thread)
 {
 	struct ospf6_interface *oi;
 	struct ospf6_packet *op;
 	uint16_t length = OSPF6_HEADER_SIZE;
 
-	oi = (struct ospf6_interface *)THREAD_ARG(thread);
+	oi = (struct ospf6_interface *)EVENT_ARG(thread);
 
 	if (oi->state <= OSPF6_INTERFACE_WAITING) {
 		if (IS_OSPF6_DEBUG_MESSAGE(OSPF6_MESSAGE_TYPE_LSACK, SEND_HDR))
@@ -2986,8 +3069,8 @@ void ospf6_lsack_send_interface(struct thread *thread)
 	OSPF6_MESSAGE_WRITE_ON(oi);
 
 	if (oi->lsack_list->count > 0)
-		thread_add_event(master, ospf6_lsack_send_interface, oi, 0,
-				 &oi->thread_send_lsack);
+		event_add_event(master, ospf6_lsack_send_interface, oi, 0,
+				&oi->thread_send_lsack);
 }
 
 /* Commands */
