@@ -1,22 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * OSPF LSDB support.
  * Copyright (C) 1999, 2000 Alex Zinin, Kunihiro Ishiguro, Toshiaki Takada
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -92,6 +77,21 @@ static void ospf_lsdb_delete_entry(struct ospf_lsdb *lsdb,
 	lsdb->type[lsa->data->type].count--;
 	lsdb->type[lsa->data->type].checksum -= ntohs(lsa->data->checksum);
 	lsdb->total--;
+
+	/* Decrement number of router LSAs received with DC bit set */
+	if (lsa->area && (lsa->area->lsdb == lsdb) && !IS_LSA_SELF(lsa) &&
+	    (lsa->data->type == OSPF_ROUTER_LSA) &&
+	    CHECK_FLAG(lsa->data->options, OSPF_OPTION_DC))
+		lsa->area->fr_info.router_lsas_recv_dc_bit--;
+
+	/*
+	 * If the LSA being deleted is indication LSA, then set the
+	 * pointer to NULL.
+	 */
+	if (lsa->area && lsa->area->fr_info.indication_lsa_self &&
+	    (lsa->area->fr_info.indication_lsa_self == lsa))
+		lsa->area->fr_info.indication_lsa_self = NULL;
+
 	rn->info = NULL;
 	route_unlock_node(rn);
 #ifdef MONITOR_LSDB_CHANGE
@@ -128,6 +128,12 @@ void ospf_lsdb_add(struct ospf_lsdb *lsdb, struct ospf_lsa *lsa)
 	lsdb->type[lsa->data->type].count++;
 	lsdb->total++;
 
+	/* Increment number of router LSAs received with DC bit set */
+	if (lsa->area && (lsa->area->lsdb == lsdb) && !IS_LSA_SELF(lsa) &&
+	    (lsa->data->type == OSPF_ROUTER_LSA) &&
+	    CHECK_FLAG(lsa->data->options, OSPF_OPTION_DC))
+		lsa->area->fr_info.router_lsas_recv_dc_bit++;
+
 #ifdef MONITOR_LSDB_CHANGE
 	if (lsdb->new_lsa_hook != NULL)
 		(*lsdb->new_lsa_hook)(lsa);
