@@ -1,29 +1,15 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * OSPFd main routine.
  *   Copyright (C) 1998, 99 Kunihiro Ishiguro, Toshiaki Takada
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
 #include <lib/version.h>
+#include "bfd.h"
 #include "getopt.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "prefix.h"
 #include "linklist.h"
 #include "if.h"
@@ -41,6 +27,7 @@
 #include "vrf.h"
 #include "libfrr.h"
 #include "routemap.h"
+#include "keychain.h"
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
@@ -53,7 +40,20 @@
 #include "ospfd/ospf_zebra.h"
 #include "ospfd/ospf_vty.h"
 #include "ospfd/ospf_bfd.h"
+#include "ospfd/ospf_gr.h"
 #include "ospfd/ospf_errors.h"
+#include "ospfd/ospf_ldp_sync.h"
+#include "ospfd/ospf_routemap_nb.h"
+
+#define OSPFD_STATE_NAME	 "%s/ospfd.json", frr_libstatedir
+#define OSPFD_INST_STATE_NAME(i) "%s/ospfd-%d.json", frr_runstatedir, i
+
+/* this one includes the path... because the instance number was in the path
+ * before :( ... which totally didn't have a mkdir anywhere.
+ */
+#define OSPFD_COMPAT_STATE_NAME "%s/ospfd-gr.json", frr_libstatedir
+#define OSPFD_COMPAT_INST_STATE_NAME(i)                                        \
+	"%s-%d/ospfd-gr.json", frr_runstatedir, i
 
 /* ospfd privileges */
 zebra_capabilities_t _caps_p[] = {ZCAP_NET_RAW, ZCAP_BIND, ZCAP_NET_ADMIN,
@@ -81,7 +81,7 @@ const struct option longopts[] = {
 /* OSPFd program name */
 
 /* Master of threads. */
-struct thread_master *master;
+struct event_loop *master;
 
 #ifdef SUPPORT_OSPF_API
 extern int ospf_apiserver_enable;
@@ -97,7 +97,9 @@ static void sighup(void)
 static void sigint(void)
 {
 	zlog_notice("Terminating on signal");
+	bfd_protocol_integration_set_shutdown(true);
 	ospf_terminate();
+
 	exit(0);
 }
 
@@ -107,7 +109,7 @@ static void sigusr1(void)
 	zlog_rotate();
 }
 
-struct quagga_signal_t ospf_signals[] = {
+struct frr_signal_t ospf_signals[] = {
 	{
 		.signal = SIGHUP,
 		.handler = &sighup,
@@ -131,23 +133,64 @@ static const struct frr_yang_module_info *const ospfd_yang_modules[] = {
 	&frr_interface_info,
 	&frr_route_map_info,
 	&frr_vrf_info,
+	&frr_ospf_route_map_info,
 };
 
-FRR_DAEMON_INFO(ospfd, OSPF, .vty_port = OSPF_VTY_PORT,
+/* actual paths filled in main() */
+static char state_path[512];
+static char state_compat_path[512];
+static char *state_paths[] = {
+	state_path,
+	state_compat_path,
+	NULL,
+};
 
-		.proghelp = "Implementation of the OSPFv2 routing protocol.",
+/* clang-format off */
+FRR_DAEMON_INFO(ospfd, OSPF,
+	.vty_port = OSPF_VTY_PORT,
+	.proghelp = "Implementation of the OSPFv2 routing protocol.",
 
-		.signals = ospf_signals, .n_signals = array_size(ospf_signals),
+	.signals = ospf_signals,
+	.n_signals = array_size(ospf_signals),
 
-		.privs = &ospfd_privs, .yang_modules = ospfd_yang_modules,
-		.n_yang_modules = array_size(ospfd_yang_modules), )
+	.privs = &ospfd_privs,
+
+	.yang_modules = ospfd_yang_modules,
+	.n_yang_modules = array_size(ospfd_yang_modules),
+
+	.state_paths = state_paths,
+);
+/* clang-format on */
+
+/** Max wait time for config to load before accepting hellos */
+#define OSPF_PRE_CONFIG_MAX_WAIT_SECONDS 600
+
+static void ospf_config_finish(struct event *t)
+{
+	zlog_err("OSPF configuration end timer expired after %d seconds.",
+		 OSPF_PRE_CONFIG_MAX_WAIT_SECONDS);
+}
+
+static void ospf_config_start(void)
+{
+	EVENT_OFF(t_ospf_cfg);
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_debug("ospfd config start callback received.");
+	event_add_timer(master, ospf_config_finish, NULL,
+			OSPF_PRE_CONFIG_MAX_WAIT_SECONDS, &t_ospf_cfg);
+}
+
+static void ospf_config_end(void)
+{
+	if (IS_DEBUG_OSPF_EVENT)
+		zlog_debug("ospfd config end callback received.");
+
+	EVENT_OFF(t_ospf_cfg);
+}
 
 /* OSPFd main routine. */
 int main(int argc, char **argv)
 {
-	unsigned short instance = 0;
-	bool created = false;
-
 #ifdef SUPPORT_OSPF_API
 	/* OSPF apiserver is disabled by default. */
 	ospf_apiserver_enable = 0;
@@ -168,8 +211,8 @@ int main(int argc, char **argv)
 
 		switch (opt) {
 		case 'n':
-			ospfd_di.instance = instance = atoi(optarg);
-			if (instance < 1)
+			ospfd_di.instance = ospf_instance = atoi(optarg);
+			if (ospf_instance < 1)
 				exit(0);
 			break;
 		case 0:
@@ -181,7 +224,6 @@ int main(int argc, char **argv)
 #endif /* SUPPORT_OSPF_API */
 		default:
 			frr_help_exit(1);
-			break;
 		}
 	}
 
@@ -192,6 +234,17 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	if (ospf_instance) {
+		snprintf(state_path, sizeof(state_path),
+			 OSPFD_INST_STATE_NAME(ospf_instance));
+		snprintf(state_compat_path, sizeof(state_compat_path),
+			 OSPFD_COMPAT_INST_STATE_NAME(ospf_instance));
+	} else {
+		snprintf(state_path, sizeof(state_path), OSPFD_STATE_NAME);
+		snprintf(state_compat_path, sizeof(state_compat_path),
+			 OSPFD_COMPAT_STATE_NAME);
+	}
+
 	/* OSPF master init. */
 	ospf_master_init(frr_init());
 
@@ -204,10 +257,14 @@ int main(int argc, char **argv)
 
 	access_list_init();
 	prefix_list_init();
+	keychain_init();
+
+	/* Configuration processing callback initialization. */
+	cmd_init_config_callbacks(ospf_config_start, ospf_config_end);
 
 	/* OSPFd inits. */
 	ospf_if_init();
-	ospf_zebra_init(master, instance);
+	ospf_zebra_init(master, ospf_instance);
 
 	/* OSPF vty inits. */
 	ospf_vty_init();
@@ -215,25 +272,19 @@ int main(int argc, char **argv)
 	ospf_vty_clear_init();
 
 	/* OSPF BFD init */
-	ospf_bfd_init();
+	ospf_bfd_init(master);
+
+	/* OSPF LDP IGP Sync init */
+	ospf_ldp_sync_init();
 
 	ospf_route_map_init();
 	ospf_opaque_init();
+	ospf_gr_init();
+	ospf_gr_helper_init();
 
 	/* OSPF errors init */
 	ospf_error_init();
 
-	/*
-	 * Need to initialize the default ospf structure, so the interface mode
-	 * commands can be duly processed if they are received before 'router
-	 * ospf',  when ospfd is restarted
-	 */
-	if (instance && !ospf_get_instance(instance, &created)) {
-		flog_err(EC_OSPF_INIT_FAIL, "OSPF instance init failed: %s",
-			 strerror(errno));
-		exit(1);
-	}
-
 	frr_config_fork();
 	frr_run(master);
 
