@@ -1,26 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* OSPF version 2 daemon program.
  * Copyright (C) 1999, 2000 Toshiaki Takada
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
 
-#include "thread.h"
+#include "frrevent.h"
 #include "vty.h"
 #include "command.h"
 #include "linklist.h"
@@ -75,7 +60,10 @@ struct ospf_master *om;
 unsigned short ospf_instance;
 
 extern struct zclient *zclient;
+extern struct zclient *zclient_sync;
 
+/* OSPF config processing timer thread */
+struct event *t_ospf_cfg;
 
 static void ospf_remove_vls_through_area(struct ospf *, struct ospf_area *);
 static void ospf_network_free(struct ospf *, struct ospf_network *);
@@ -400,8 +388,8 @@ struct ospf *ospf_new_alloc(unsigned short instance, const char *name)
 	new->maxage_delay = OSPF_LSA_MAXAGE_REMOVE_DELAY_DEFAULT;
 	new->maxage_lsa = route_table_init();
 	new->t_maxage_walker = NULL;
-	thread_add_timer(master, ospf_lsa_maxage_walker, new,
-			 OSPF_LSA_MAXAGE_CHECK_INTERVAL, &new->t_maxage_walker);
+	event_add_timer(master, ospf_lsa_maxage_walker, new,
+			OSPF_LSA_MAXAGE_CHECK_INTERVAL, &new->t_maxage_walker);
 
 	/* Max paths initialization */
 	new->max_multipath = MULTIPATH_NUM;
@@ -413,8 +401,8 @@ struct ospf *ospf_new_alloc(unsigned short instance, const char *name)
 	new->lsa_refresh_interval = OSPF_LSA_REFRESH_INTERVAL_DEFAULT;
 	new->lsa_refresh_timer = OSPF_LS_REFRESH_TIME;
 	new->t_lsa_refresher = NULL;
-	thread_add_timer(master, ospf_lsa_refresh_walker, new,
-			 new->lsa_refresh_interval, &new->t_lsa_refresher);
+	event_add_timer(master, ospf_lsa_refresh_walker, new,
+			new->lsa_refresh_interval, &new->t_lsa_refresher);
 	new->lsa_refresher_started = monotime(NULL);
 
 	new->ibuf = stream_new(OSPF_MAX_PACKET_SIZE + 1);
@@ -434,6 +422,10 @@ struct ospf *ospf_new_alloc(unsigned short instance, const char *name)
 	QOBJ_REG(new, ospf);
 
 	new->fd = -1;
+	new->intf_socket_enabled = true;
+
+	new->recv_sock_bufsize = OSPF_DEFAULT_SOCK_BUFSIZE;
+	new->send_sock_bufsize = OSPF_DEFAULT_SOCK_BUFSIZE;
 
 	return new;
 }
@@ -456,7 +448,7 @@ static struct ospf *ospf_new(unsigned short instance, const char *name)
 		return new;
 	}
 
-	thread_add_read(master, ospf_read, new, new->fd, &new->t_read);
+	event_add_read(master, ospf_read, new, new->fd, &new->t_read);
 
 	new->oi_running = 1;
 	ospf_router_id_update(new);
@@ -467,6 +459,8 @@ static struct ospf *ospf_new(unsigned short instance, const char *name)
 	 */
 	ospf_gr_nvm_read(new);
 
+	new->fr_configured = false;
+
 	return new;
 }
 
@@ -581,39 +575,12 @@ static struct ospf *ospf_lookup_by_name(const char *vrf_name)
 	return NULL;
 }
 
-/* Handle the second half of deferred shutdown. This is called either
- * from the deferred-shutdown timer thread, or directly through
- * ospf_deferred_shutdown_check.
- *
- * Function is to cleanup G-R state, if required then call ospf_finish_final
- * to complete shutdown of this ospf instance. Possibly exit if the
- * whole process is being shutdown and this was the last OSPF instance.
- */
-static void ospf_deferred_shutdown_finish(struct ospf *ospf)
+/* Timer thread for deferred shutdown */
+static void ospf_deferred_shutdown_timer(struct event *t)
 {
-	ospf->stub_router_shutdown_time = OSPF_STUB_ROUTER_UNCONFIGURED;
-	THREAD_OFF(ospf->t_deferred_shutdown);
+	struct ospf *ospf = EVENT_ARG(t);
 
 	ospf_finish_final(ospf);
-
-	/* *ospf is now invalid */
-
-	/* ospfd being shut-down? If so, was this the last ospf instance? */
-	if (CHECK_FLAG(om->options, OSPF_MASTER_SHUTDOWN)
-	    && (listcount(om->ospf) == 0)) {
-		frr_fini();
-		exit(0);
-	}
-
-	return;
-}
-
-/* Timer thread for G-R */
-static void ospf_deferred_shutdown_timer(struct thread *t)
-{
-	struct ospf *ospf = THREAD_ARG(t);
-
-	ospf_deferred_shutdown_finish(ospf);
 }
 
 /* Check whether deferred-shutdown must be scheduled, otherwise call
@@ -640,15 +607,12 @@ static void ospf_deferred_shutdown_check(struct ospf *ospf)
 				ospf_router_lsa_update_area(area);
 		}
 		timeout = ospf->stub_router_shutdown_time;
+		OSPF_TIMER_ON(ospf->t_deferred_shutdown,
+			      ospf_deferred_shutdown_timer, timeout);
 	} else {
 		/* No timer needed */
-		ospf_deferred_shutdown_finish(ospf);
-		return;
+		ospf_finish_final(ospf);
 	}
-
-	OSPF_TIMER_ON(ospf->t_deferred_shutdown, ospf_deferred_shutdown_timer,
-		      timeout);
-	return;
 }
 
 /* Shut down the entire process */
@@ -663,10 +627,6 @@ void ospf_terminate(void)
 
 	SET_FLAG(om->options, OSPF_MASTER_SHUTDOWN);
 
-	/* Skip some steps if OSPF not actually running */
-	if (listcount(om->ospf) == 0)
-		goto done;
-
 	for (ALL_LIST_ELEMENTS(om->ospf, node, nnode, ospf))
 		ospf_finish(ospf);
 
@@ -684,27 +644,31 @@ void ospf_terminate(void)
 	/* Cleanup vrf info */
 	ospf_vrf_terminate();
 
+	keychain_terminate();
+
+	ospf_opaque_term();
+	list_delete(&om->ospf);
+
 	/* Deliberately go back up, hopefully to thread scheduler, as
 	 * One or more ospf_finish()'s may have deferred shutdown to a timer
 	 * thread
 	 */
 	zclient_stop(zclient);
 	zclient_free(zclient);
+	zclient_stop(zclient_sync);
+	zclient_free(zclient_sync);
 
-done:
 	frr_fini();
 }
 
 void ospf_finish(struct ospf *ospf)
 {
-	/* let deferred shutdown decide */
-	ospf_deferred_shutdown_check(ospf);
-
-	/* if ospf_deferred_shutdown returns, then ospf_finish_final is
-	 * deferred to expiry of G-S timer thread. Return back up, hopefully
-	 * to thread scheduler.
-	 */
-	return;
+	if (CHECK_FLAG(om->options, OSPF_MASTER_SHUTDOWN))
+		ospf_finish_final(ospf);
+	else {
+		/* let deferred shutdown decide */
+		ospf_deferred_shutdown_check(ospf);
+	}
 }
 
 /* Final cleanup of ospf instance */
@@ -729,6 +693,7 @@ static void ospf_finish_final(struct ospf *ospf)
 
 	if (!ospf->gr_info.prepare_in_progress)
 		ospf_flush_self_originated_lsas_now(ospf);
+	XFREE(MTYPE_TMP, ospf->gr_info.exit_reason);
 
 	/* Unregister redistribution */
 	for (i = 0; i < ZEBRA_ROUTE_MAX; i++) {
@@ -774,7 +739,7 @@ static void ospf_finish_final(struct ospf *ospf)
 	/* Clear static neighbors */
 	for (rn = route_top(ospf->nbr_nbma); rn; rn = route_next(rn))
 		if ((nbr_nbma = rn->info)) {
-			THREAD_OFF(nbr_nbma->t_poll);
+			EVENT_OFF(nbr_nbma->t_poll);
 
 			if (nbr_nbma->nbr) {
 				nbr_nbma->nbr->nbr_nbma = NULL;
@@ -809,24 +774,6 @@ static void ospf_finish_final(struct ospf *ospf)
 		ospf_area_free(area);
 	}
 
-	/* Cancel all timers. */
-	THREAD_OFF(ospf->t_read);
-	THREAD_OFF(ospf->t_write);
-	THREAD_OFF(ospf->t_spf_calc);
-	THREAD_OFF(ospf->t_ase_calc);
-	THREAD_OFF(ospf->t_maxage);
-	THREAD_OFF(ospf->t_maxage_walker);
-	THREAD_OFF(ospf->t_abr_task);
-	THREAD_OFF(ospf->t_asbr_check);
-	THREAD_OFF(ospf->t_asbr_nssa_redist_update);
-	THREAD_OFF(ospf->t_distribute_update);
-	THREAD_OFF(ospf->t_lsa_refresher);
-	THREAD_OFF(ospf->t_opaque_lsa_self);
-	THREAD_OFF(ospf->t_sr_update);
-	THREAD_OFF(ospf->t_default_routemap_timer);
-	THREAD_OFF(ospf->t_external_aggr);
-	THREAD_OFF(ospf->gr_info.t_grace_period);
-
 	LSDB_LOOP (OPAQUE_AS_LSDB(ospf), rn, lsa)
 		ospf_discard_from_db(ospf, ospf->lsdb, lsa);
 	LSDB_LOOP (EXTERNAL_LSDB(ospf), rn, lsa)
@@ -851,6 +798,10 @@ static void ospf_finish_final(struct ospf *ospf)
 			ospf_route_delete(ospf, ospf->new_table);
 		ospf_route_table_free(ospf->new_table);
 	}
+	if (ospf->oall_rtrs)
+		ospf_rtrs_free(ospf->oall_rtrs);
+	if (ospf->all_rtrs)
+		ospf_rtrs_free(ospf->all_rtrs);
 	if (ospf->old_rtrs)
 		ospf_rtrs_free(ospf->old_rtrs);
 	if (ospf->new_rtrs)
@@ -910,8 +861,27 @@ static void ospf_finish_final(struct ospf *ospf)
 		}
 	}
 
-	route_table_finish(ospf->rt_aggr_tbl);
+	/* Cancel all timers. */
+	EVENT_OFF(ospf->t_read);
+	EVENT_OFF(ospf->t_write);
+	EVENT_OFF(ospf->t_spf_calc);
+	EVENT_OFF(ospf->t_ase_calc);
+	EVENT_OFF(ospf->t_maxage);
+	EVENT_OFF(ospf->t_maxage_walker);
+	EVENT_OFF(ospf->t_deferred_shutdown);
+	EVENT_OFF(ospf->t_abr_task);
+	EVENT_OFF(ospf->t_abr_fr);
+	EVENT_OFF(ospf->t_asbr_check);
+	EVENT_OFF(ospf->t_asbr_redist_update);
+	EVENT_OFF(ospf->t_distribute_update);
+	EVENT_OFF(ospf->t_lsa_refresher);
+	EVENT_OFF(ospf->t_opaque_lsa_self);
+	EVENT_OFF(ospf->t_sr_update);
+	EVENT_OFF(ospf->t_default_routemap_timer);
+	EVENT_OFF(ospf->t_external_aggr);
+	EVENT_OFF(ospf->gr_info.t_grace_period);
 
+	route_table_finish(ospf->rt_aggr_tbl);
 
 	ospf_free_refresh_queue(ospf);
 
@@ -934,6 +904,15 @@ static void ospf_finish_final(struct ospf *ospf)
 	XFREE(MTYPE_OSPF_TOP, ospf);
 }
 
+static void ospf_range_table_node_destroy(route_table_delegate_t *delegate,
+			struct route_table *table, struct route_node *node)
+{
+	XFREE(MTYPE_OSPF_AREA_RANGE, node->info);
+	XFREE(MTYPE_ROUTE_NODE, node);
+}
+
+route_table_delegate_t ospf_range_table_delegate = {.create_node = route_node_create,
+						 .destroy_node = ospf_range_table_node_destroy};
 
 /* allocate new OSPF Area object */
 struct ospf_area *ospf_area_new(struct ospf *ospf, struct in_addr area_id)
@@ -958,10 +937,20 @@ struct ospf_area *ospf_area_new(struct ospf *ospf, struct in_addr area_id)
 	/* Self-originated LSAs initialize. */
 	new->router_lsa_self = NULL;
 
+	/* Initialize FR field */
+	new->fr_info.enabled = false;
+	new->fr_info.configured = false;
+	new->fr_info.state_changed = false;
+	new->fr_info.router_lsas_recv_dc_bit = 0;
+	new->fr_info.indication_lsa_self = NULL;
+	new->fr_info.area_ind_lsa_recvd = false;
+	new->fr_info.area_dc_clear = false;
+
 	ospf_opaque_type10_lsa_init(new);
 
 	new->oiflist = list_new();
-	new->ranges = route_table_init();
+	new->ranges = route_table_init_with_delegate(&ospf_range_table_delegate);
+	new->nssa_ranges = route_table_init_with_delegate(&ospf_range_table_delegate);
 
 	if (area_id.s_addr == OSPF_AREA_BACKBONE)
 		ospf->backbone = new;
@@ -1005,6 +994,7 @@ static void ospf_area_free(struct ospf_area *area)
 	ospf_lsa_unlock(&area->router_lsa_self);
 
 	route_table_finish(area->ranges);
+	route_table_finish(area->nssa_ranges);
 	list_delete(&area->oiflist);
 
 	if (EXPORT_NAME(area))
@@ -1014,8 +1004,8 @@ static void ospf_area_free(struct ospf_area *area)
 		free(IMPORT_NAME(area));
 
 	/* Cancel timer. */
-	THREAD_OFF(area->t_stub_router);
-	THREAD_OFF(area->t_opaque_lsa_self);
+	EVENT_OFF(area->t_stub_router);
+	EVENT_OFF(area->t_opaque_lsa_self);
 
 	if (OSPF_IS_AREA_BACKBONE(area))
 		area->ospf->backbone = NULL;
@@ -1028,13 +1018,14 @@ void ospf_area_check_free(struct ospf *ospf, struct in_addr area_id)
 	struct ospf_area *area;
 
 	area = ospf_area_lookup_by_area_id(ospf, area_id);
-	if (area && listcount(area->oiflist) == 0 && area->ranges->top == NULL
-	    && !ospf_vl_count(ospf, area)
-	    && area->shortcut_configured == OSPF_SHORTCUT_DEFAULT
-	    && area->external_routing == OSPF_AREA_DEFAULT
-	    && area->no_summary == 0 && area->default_cost == 1
-	    && EXPORT_NAME(area) == NULL && IMPORT_NAME(area) == NULL
-	    && area->auth_type == OSPF_AUTH_NULL) {
+	if (area && listcount(area->oiflist) == 0 &&
+	    area->ranges->top == NULL && area->nssa_ranges->top == NULL &&
+	    !ospf_vl_count(ospf, area) &&
+	    area->shortcut_configured == OSPF_SHORTCUT_DEFAULT &&
+	    area->external_routing == OSPF_AREA_DEFAULT &&
+	    area->no_summary == 0 && area->default_cost == 1 &&
+	    EXPORT_NAME(area) == NULL && IMPORT_NAME(area) == NULL &&
+	    area->auth_type == OSPF_AUTH_NULL) {
 		listnode_delete(ospf->areas, area);
 		ospf_area_free(area);
 	}
@@ -1104,6 +1095,7 @@ struct ospf_interface *add_ospf_interface(struct connected *co,
 	   skip network type setting. */
 	oi->type = IF_DEF_PARAMS(co->ifp)->type;
 	oi->ptp_dmvpn = IF_DEF_PARAMS(co->ifp)->ptp_dmvpn;
+	oi->p2mp_delay_reflood = IF_DEF_PARAMS(co->ifp)->p2mp_delay_reflood;
 
 	/* Add pseudo neighbor. */
 	ospf_nbr_self_reset(oi, oi->ospf->router_id);
@@ -1123,6 +1115,17 @@ struct ospf_interface *add_ospf_interface(struct connected *co,
 	    && if_is_operative(co->ifp))
 		ospf_if_up(oi);
 
+	/*
+	 * RFC 3623 - Section 5 ("Unplanned Outages"):
+	 * "The grace-LSAs are encapsulated in Link State Update Packets
+	 * and sent out to all interfaces, even though the restarted
+	 * router has no adjacencies and no knowledge of previous
+	 * adjacencies".
+	 */
+	if (oi->ospf->gr_info.restart_in_progress &&
+	    oi->ospf->gr_info.reason == OSPF_GR_UNKNOWN_RESTART)
+		ospf_gr_unplanned_start_interface(oi);
+
 	return oi;
 }
 
@@ -1223,8 +1226,9 @@ int ospf_network_unset(struct ospf *ospf, struct prefix_ipv4 *p,
 {
 	struct route_node *rn;
 	struct ospf_network *network;
-	struct listnode *node, *nnode;
+	struct listnode *node;
 	struct ospf_interface *oi;
+	struct list *ospf_oiflist = NULL;
 
 	rn = route_node_lookup(ospf->networks, (struct prefix *)p);
 	if (rn == NULL)
@@ -1239,8 +1243,9 @@ int ospf_network_unset(struct ospf *ospf, struct prefix_ipv4 *p,
 	rn->info = NULL;
 	route_unlock_node(rn); /* initial reference */
 
-	/* Find interfaces that are not configured already.  */
-	for (ALL_LIST_ELEMENTS(ospf->oiflist, node, nnode, oi)) {
+	ospf_oiflist = list_dup(ospf->oiflist);
+	/* Find interfaces that are not configured already. */
+	for (ALL_LIST_ELEMENTS_RO(ospf_oiflist, node, oi)) {
 
 		if (oi->type == OSPF_IFTYPE_VIRTUALLINK)
 			continue;
@@ -1248,6 +1253,8 @@ int ospf_network_unset(struct ospf *ospf, struct prefix_ipv4 *p,
 		ospf_network_run_subnet(ospf, oi->connected, NULL, NULL);
 	}
 
+	list_delete(&ospf_oiflist);
+
 	/* Update connected redistribute. */
 	update_redistributed(ospf, 0); /* interfaces possibly removed */
 	ospf_area_check_free(ospf, area_id);
@@ -1255,6 +1262,7 @@ int ospf_network_unset(struct ospf *ospf, struct prefix_ipv4 *p,
 	return 1;
 }
 
+
 /* Ensure there's an OSPF instance, as "ip ospf area" enabled OSPF means
  * there might not be any 'router ospf' config.
  *
@@ -1403,7 +1411,6 @@ static void ospf_network_run_interface(struct ospf *ospf, struct interface *ifp,
 				       struct prefix *p,
 				       struct ospf_area *given_area)
 {
-	struct listnode *cnode;
 	struct connected *co;
 
 	if (memcmp(ifp->name, "VLINK", 5) == 0)
@@ -1415,7 +1422,7 @@ static void ospf_network_run_interface(struct ospf *ospf, struct interface *ifp,
 
 	/* if interface prefix is match specified prefix,
 	   then create socket and join multicast group. */
-	for (ALL_LIST_ELEMENTS_RO(ifp->connected, cnode, co))
+	frr_each (if_connected, ifp->connected, co)
 		ospf_network_run_subnet(ospf, co, p, given_area);
 }
 
@@ -1450,7 +1457,7 @@ void ospf_ls_upd_queue_empty(struct ospf_interface *oi)
 		}
 
 	/* remove update event */
-	THREAD_OFF(oi->t_ls_upd_event);
+	EVENT_OFF(oi->t_ls_upd_event);
 }
 
 void ospf_if_update(struct ospf *ospf, struct interface *ifp)
@@ -1700,7 +1707,7 @@ int ospf_area_nssa_set(struct ospf *ospf, struct in_addr area_id)
 	return 1;
 }
 
-int ospf_area_nssa_unset(struct ospf *ospf, struct in_addr area_id, int argc)
+int ospf_area_nssa_unset(struct ospf *ospf, struct in_addr area_id)
 {
 	struct ospf_area *area;
 
@@ -1708,22 +1715,14 @@ int ospf_area_nssa_unset(struct ospf *ospf, struct in_addr area_id, int argc)
 	if (area == NULL)
 		return 0;
 
-	/* argc < 5 -> 'no area x nssa' */
-	if (argc < 5 && area->external_routing == OSPF_AREA_NSSA) {
-		ospf->anyNSSA--;
-		/* set NSSA area defaults */
-		area->no_summary = 0;
-		area->suppress_fa = 0;
-		area->NSSATranslatorRole = OSPF_NSSA_ROLE_CANDIDATE;
-		area->NSSATranslatorState = OSPF_NSSA_TRANSLATE_DISABLED;
-		area->NSSATranslatorStabilityInterval =
-			OSPF_NSSA_TRANS_STABLE_DEFAULT;
-		ospf_area_type_set(area, OSPF_AREA_DEFAULT);
-	} else {
-		ospf_area_nssa_translator_role_set(ospf, area_id,
-						   OSPF_NSSA_ROLE_CANDIDATE);
-	}
-
+	ospf->anyNSSA--;
+	/* set NSSA area defaults */
+	area->no_summary = 0;
+	area->suppress_fa = 0;
+	area->NSSATranslatorRole = OSPF_NSSA_ROLE_CANDIDATE;
+	area->NSSATranslatorState = OSPF_NSSA_TRANSLATE_DISABLED;
+	area->NSSATranslatorStabilityInterval = OSPF_NSSA_TRANS_STABLE_DEFAULT;
+	ospf_area_type_set(area, OSPF_AREA_DEFAULT);
 	ospf_area_check_free(ospf, area_id);
 
 	return 1;
@@ -1781,6 +1780,51 @@ int ospf_area_nssa_translator_role_set(struct ospf *ospf,
 	return 1;
 }
 
+void ospf_area_nssa_default_originate_set(struct ospf *ospf,
+					  struct in_addr area_id, int metric,
+					  int metric_type)
+{
+	struct ospf_area *area;
+
+	area = ospf_area_lookup_by_area_id(ospf, area_id);
+	if (area == NULL)
+		return;
+
+	if (!area->nssa_default_originate.enabled) {
+		area->nssa_default_originate.enabled = true;
+		if (++ospf->nssa_default_import_check.refcnt == 1) {
+			ospf->nssa_default_import_check.status = false;
+			ospf_zebra_import_default_route(ospf, false);
+		}
+	}
+
+	area->nssa_default_originate.metric_value = metric;
+	area->nssa_default_originate.metric_type = metric_type;
+}
+
+void ospf_area_nssa_default_originate_unset(struct ospf *ospf,
+					    struct in_addr area_id)
+{
+	struct ospf_area *area;
+
+	area = ospf_area_lookup_by_area_id(ospf, area_id);
+	if (area == NULL)
+		return;
+
+	if (area->nssa_default_originate.enabled) {
+		area->nssa_default_originate.enabled = false;
+		if (--ospf->nssa_default_import_check.refcnt == 0) {
+			ospf->nssa_default_import_check.status = false;
+			ospf_zebra_import_default_route(ospf, true);
+		}
+		area->nssa_default_originate.metric_value = -1;
+		area->nssa_default_originate.metric_type = -1;
+
+		if (!IS_OSPF_ABR(ospf))
+			ospf_abr_nssa_type7_defaults(ospf);
+	}
+}
+
 int ospf_area_export_list_set(struct ospf *ospf, struct ospf_area *area,
 			      const char *list_name)
 {
@@ -1858,9 +1902,9 @@ int ospf_timers_refresh_set(struct ospf *ospf, int interval)
 		    - (monotime(NULL) - ospf->lsa_refresher_started);
 
 	if (time_left > interval) {
-		THREAD_OFF(ospf->t_lsa_refresher);
-		thread_add_timer(master, ospf_lsa_refresh_walker, ospf,
-				 interval, &ospf->t_lsa_refresher);
+		EVENT_OFF(ospf->t_lsa_refresher);
+		event_add_timer(master, ospf_lsa_refresh_walker, ospf, interval,
+				&ospf->t_lsa_refresher);
 	}
 	ospf->lsa_refresh_interval = interval;
 
@@ -1875,11 +1919,11 @@ int ospf_timers_refresh_unset(struct ospf *ospf)
 		    - (monotime(NULL) - ospf->lsa_refresher_started);
 
 	if (time_left > OSPF_LSA_REFRESH_INTERVAL_DEFAULT) {
-		THREAD_OFF(ospf->t_lsa_refresher);
+		EVENT_OFF(ospf->t_lsa_refresher);
 		ospf->t_lsa_refresher = NULL;
-		thread_add_timer(master, ospf_lsa_refresh_walker, ospf,
-				 OSPF_LSA_REFRESH_INTERVAL_DEFAULT,
-				 &ospf->t_lsa_refresher);
+		event_add_timer(master, ospf_lsa_refresh_walker, ospf,
+				OSPF_LSA_REFRESH_INTERVAL_DEFAULT,
+				&ospf->t_lsa_refresher);
 	}
 
 	ospf->lsa_refresh_interval = OSPF_LSA_REFRESH_INTERVAL_DEFAULT;
@@ -1927,7 +1971,7 @@ static void ospf_nbr_nbma_delete(struct ospf *ospf,
 
 static void ospf_nbr_nbma_down(struct ospf_nbr_nbma *nbr_nbma)
 {
-	THREAD_OFF(nbr_nbma->t_poll);
+	EVENT_OFF(nbr_nbma->t_poll);
 
 	if (nbr_nbma->nbr) {
 		nbr_nbma->nbr->nbr_nbma = NULL;
@@ -2116,7 +2160,7 @@ int ospf_nbr_nbma_poll_interval_set(struct ospf *ospf, struct in_addr nbr_addr,
 	if (nbr_nbma->v_poll != interval) {
 		nbr_nbma->v_poll = interval;
 		if (nbr_nbma->oi && ospf_if_is_up(nbr_nbma->oi)) {
-			THREAD_OFF(nbr_nbma->t_poll);
+			EVENT_OFF(nbr_nbma->t_poll);
 			OSPF_POLL_TIMER_ON(nbr_nbma->t_poll, ospf_poll_timer,
 					   nbr_nbma->v_poll);
 		}
@@ -2139,7 +2183,33 @@ int ospf_nbr_nbma_poll_interval_unset(struct ospf *ospf, struct in_addr addr)
 	return 1;
 }
 
-void ospf_master_init(struct thread_master *master)
+/*
+ * Update socket bufsize(s), usually after config change
+ */
+void ospf_update_bufsize(struct ospf *ospf, uint32_t recvsize,
+			 uint32_t sendsize)
+{
+	enum ospf_sock_type_e type = OSPF_SOCK_NONE;
+
+	/* Figure out whether there's been a change */
+	if (recvsize != ospf->recv_sock_bufsize) {
+		type = OSPF_SOCK_RECV;
+		ospf->recv_sock_bufsize = recvsize;
+
+		if (sendsize != ospf->send_sock_bufsize) {
+			type = OSPF_SOCK_BOTH;
+			ospf->send_sock_bufsize = sendsize;
+		}
+	} else if (sendsize != ospf->send_sock_bufsize) {
+		type = OSPF_SOCK_SEND;
+		ospf->send_sock_bufsize = sendsize;
+	}
+
+	if (type != OSPF_SOCK_NONE)
+		ospf_sock_bufsize_update(ospf, ospf->fd, type);
+}
+
+void ospf_master_init(struct event_loop *master)
 {
 	memset(&ospf_master, 0, sizeof(ospf_master));
 
@@ -2198,20 +2268,20 @@ static void ospf_set_redist_vrf_bitmaps(struct ospf *ospf, bool set)
 				"%s: setting redist vrf %d bitmap for type %d",
 				__func__, ospf->vrf_id, type);
 		if (set)
-			vrf_bitmap_set(zclient->redist[AFI_IP][type],
+			vrf_bitmap_set(&zclient->redist[AFI_IP][type],
 				       ospf->vrf_id);
 		else
-			vrf_bitmap_unset(zclient->redist[AFI_IP][type],
+			vrf_bitmap_unset(&zclient->redist[AFI_IP][type],
 					 ospf->vrf_id);
 	}
 
 	red_list = ospf->redist[DEFAULT_ROUTE];
 	if (red_list) {
 		if (set)
-			vrf_bitmap_set(zclient->default_information[AFI_IP],
+			vrf_bitmap_set(&zclient->default_information[AFI_IP],
 				       ospf->vrf_id);
 		else
-			vrf_bitmap_unset(zclient->default_information[AFI_IP],
+			vrf_bitmap_unset(&zclient->default_information[AFI_IP],
 					 ospf->vrf_id);
 	}
 }
@@ -2246,8 +2316,8 @@ static int ospf_vrf_enable(struct vrf *vrf)
 			ret = ospf_sock_init(ospf);
 			if (ret < 0 || ospf->fd <= 0)
 				return 0;
-			thread_add_read(master, ospf_read, ospf, ospf->fd,
-					&ospf->t_read);
+			event_add_read(master, ospf_read, ospf, ospf->fd,
+				       &ospf->t_read);
 			ospf->oi_running = 1;
 			ospf_router_id_update(ospf);
 		}
@@ -2285,7 +2355,7 @@ static int ospf_vrf_disable(struct vrf *vrf)
 		if (IS_DEBUG_OSPF_EVENT)
 			zlog_debug("%s: ospf old_vrf_id %d unlinked", __func__,
 				   old_vrf_id);
-		THREAD_OFF(ospf->t_read);
+		EVENT_OFF(ospf->t_read);
 		close(ospf->fd);
 		ospf->fd = -1;
 	}
