@@ -1,23 +1,13 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * PIM for Quagga
  * Copyright (C) 2008  Everton da Silva Marques
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+#include <netinet/icmp6.h>
+#include <sys/ioctl.h>
+
 #include "log.h"
 #include "privs.h"
 #include "if.h"
@@ -59,7 +49,7 @@ int pim_mroute_set(struct pim_instance *pim, int enable)
 	/*
 	 * We need to create the VRF table for the pim mroute_socket
 	 */
-	if (pim->vrf->vrf_id != VRF_DEFAULT) {
+	if (enable && pim->vrf->vrf_id != VRF_DEFAULT) {
 		frr_with_privs (&pimd_privs) {
 
 			data = pim->vrf->data.l.table_id;
@@ -160,7 +150,6 @@ int pim_mroute_msg_nocache(int fd, struct interface *ifp, const kernmsg *msg)
 {
 	struct pim_interface *pim_ifp = ifp->info;
 	struct pim_upstream *up;
-	struct pim_rpf *rpg;
 	pim_sgaddr sg;
 	bool desync = false;
 
@@ -168,31 +157,41 @@ int pim_mroute_msg_nocache(int fd, struct interface *ifp, const kernmsg *msg)
 	sg.src = msg->msg_im_src;
 	sg.grp = msg->msg_im_dst;
 
-	if (!pim_ifp) {
+
+	if (!pim_ifp || !pim_ifp->pim_enable) {
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
-				"%s: PIM not enabled on interface, dropping packet to %pSG",
-				ifp->name, &sg);
+				"%s: %s on interface, dropping packet to %pSG",
+				ifp->name,
+				!pim_ifp ? "Multicast not enabled"
+					 : "PIM not enabled",
+				&sg);
 		return 0;
 	}
 
-	rpg = RP(pim_ifp->pim, msg->msg_im_dst);
-	/*
-	 * If the incoming interface is unknown OR
-	 * the Interface type is SSM we don't need to
-	 * do anything here
-	 */
-	if (!rpg) {
-		if (PIM_DEBUG_MROUTE)
-			zlog_debug("%s: no RPF for packet to %pSG", ifp->name,
-				   &sg);
-		return 0;
-	}
-	if (pim_rpf_addr_is_inaddr_any(rpg)) {
-		if (PIM_DEBUG_MROUTE)
-			zlog_debug("%s: null RPF for packet to %pSG", ifp->name,
-				   &sg);
-		return 0;
+	if (!pim_is_grp_ssm(pim_ifp->pim, sg.grp)) {
+		/* for ASM, check that we have enough information (i.e. path
+		 * to RP) to make a decision on what to do with this packet.
+		 *
+		 * for SSM, this is meaningless, everything is join-driven,
+		 * and for NOCACHE we need to install an empty OIL MFC entry
+		 * so the kernel doesn't keep nagging us.
+		 */
+		struct pim_rpf *rpg;
+
+		rpg = RP(pim_ifp->pim, msg->msg_im_dst);
+		if (!rpg) {
+			if (PIM_DEBUG_MROUTE)
+				zlog_debug("%s: no RPF for packet to %pSG",
+					   ifp->name, &sg);
+			return 0;
+		}
+		if (pim_rpf_addr_is_inaddr_any(rpg)) {
+			if (PIM_DEBUG_MROUTE)
+				zlog_debug("%s: null RPF for packet to %pSG",
+					   ifp->name, &sg);
+			return 0;
+		}
 	}
 
 	/*
@@ -257,7 +256,7 @@ int pim_mroute_msg_nocache(int fd, struct interface *ifp, const kernmsg *msg)
 	up->channel_oil->cc.pktcnt++;
 	// resolve mfcc_parent prior to mroute_add in channel_add_oif
 	if (up->rpf.source_nexthop.interface &&
-	    *oil_parent(up->channel_oil) >= MAXVIFS) {
+	    *oil_incoming_vif(up->channel_oil) >= MAXVIFS) {
 		pim_upstream_mroute_iif_update(up->channel_oil, __func__);
 	}
 	pim_register_join(up);
@@ -663,7 +662,7 @@ static int process_igmp_packet(struct pim_instance *pim, const char *buf,
 
 	connected_src = pim_if_connected_to_source(ifp, ip_hdr->ip_src);
 
-	if (!connected_src) {
+	if (!connected_src && !pim_addr_is_any(ip_hdr->ip_src)) {
 		if (PIM_DEBUG_GM_PACKETS) {
 			zlog_debug(
 				"Recv IGMP packet on interface: %s from a non-connected source: %pI4",
@@ -673,7 +672,8 @@ static int process_igmp_packet(struct pim_instance *pim, const char *buf,
 	}
 
 	pim_ifp = ifp->info;
-	ifaddr = connected_src->u.prefix4;
+	ifaddr = connected_src ? connected_src->u.prefix4
+			       : pim_ifp->primary_address;
 	igmp = pim_igmp_sock_lookup_ifaddr(pim_ifp->gm_socket_list, ifaddr);
 
 	if (PIM_DEBUG_GM_PACKETS) {
@@ -684,11 +684,11 @@ static int process_igmp_packet(struct pim_instance *pim, const char *buf,
 	}
 	if (igmp)
 		pim_igmp_packet(igmp, (char *)buf, buf_size);
-	else if (PIM_DEBUG_GM_PACKETS) {
+	else if (PIM_DEBUG_GM_PACKETS)
 		zlog_debug(
-			"No IGMP socket on interface: %s with connected source: %pFX",
-			ifp->name, connected_src);
-	}
+			"No IGMP socket on interface: %s with connected source: %pI4",
+			ifp->name, &ifaddr);
+
 	return 0;
 }
 #endif
@@ -770,7 +770,7 @@ int pim_mroute_msg(struct pim_instance *pim, const char *buf, size_t buf_size,
 	return 0;
 }
 
-static void mroute_read(struct thread *t)
+static void mroute_read(struct event *t)
 {
 	struct pim_instance *pim;
 	static long long count;
@@ -778,7 +778,7 @@ static void mroute_read(struct thread *t)
 	int cont = 1;
 	int rd;
 	ifindex_t ifindex;
-	pim = THREAD_ARG(t);
+	pim = EVENT_ARG(t);
 
 	while (cont) {
 		rd = pim_socket_recvfromto(pim->mroute_socket, (uint8_t *)buf,
@@ -812,13 +812,13 @@ done:
 
 static void mroute_read_on(struct pim_instance *pim)
 {
-	thread_add_read(router->master, mroute_read, pim, pim->mroute_socket,
-			&pim->thread);
+	event_add_read(router->master, mroute_read, pim, pim->mroute_socket,
+		       &pim->thread);
 }
 
 static void mroute_read_off(struct pim_instance *pim)
 {
-	THREAD_OFF(pim->thread);
+	EVENT_OFF(pim->thread);
 }
 
 int pim_mroute_socket_enable(struct pim_instance *pim)
@@ -1045,10 +1045,10 @@ static inline void pim_mroute_copy(struct channel_oil *out,
 
 	*oil_origin(out) = *oil_origin(in);
 	*oil_mcastgrp(out) = *oil_mcastgrp(in);
-	*oil_parent(out) = *oil_parent(in);
+	*oil_incoming_vif(out) = *oil_incoming_vif(in);
 
 	for (i = 0; i < MAXVIFS; ++i) {
-		if (*oil_parent(out) == i &&
+		if (*oil_incoming_vif(out) == i &&
 		    !pim_mroute_allow_iif_in_oil(in, i)) {
 			oil_if_set(out, i, 0);
 			continue;
@@ -1083,7 +1083,7 @@ static int pim_mroute_add(struct channel_oil *c_oil, const char *name)
 	 * in the case of a (*,G).
 	 */
 	if (pim_addr_is_any(*oil_origin(c_oil))) {
-		oil_if_set(tmp_oil, *oil_parent(c_oil), 1);
+		oil_if_set(tmp_oil, *oil_incoming_vif(c_oil), 1);
 	}
 
 	/*
@@ -1093,18 +1093,17 @@ static int pim_mroute_add(struct channel_oil *c_oil, const char *name)
 	 * the packets to be forwarded.  Then set it
 	 * to the correct IIF afterwords.
 	 */
-	if (!c_oil->installed && !pim_addr_is_any(*oil_origin(c_oil))
-	    && *oil_parent(c_oil) != 0) {
-		*oil_parent(tmp_oil) = 0;
+	if (!c_oil->installed && !pim_addr_is_any(*oil_origin(c_oil)) &&
+	    *oil_incoming_vif(c_oil) != 0) {
+		*oil_incoming_vif(tmp_oil) = 0;
 	}
 	/* For IPv6 MRT_ADD_MFC is defined to MRT6_ADD_MFC */
 	err = setsockopt(pim->mroute_socket, PIM_IPPROTO, MRT_ADD_MFC,
 			 &tmp_oil->oil, sizeof(tmp_oil->oil));
 
-	if (!err && !c_oil->installed
-	    && !pim_addr_is_any(*oil_origin(c_oil))
-	    && *oil_parent(c_oil) != 0) {
-		*oil_parent(tmp_oil) = *oil_parent(c_oil);
+	if (!err && !c_oil->installed && !pim_addr_is_any(*oil_origin(c_oil)) &&
+	    *oil_incoming_vif(c_oil) != 0) {
+		*oil_incoming_vif(tmp_oil) = *oil_incoming_vif(c_oil);
 		err = setsockopt(pim->mroute_socket, PIM_IPPROTO, MRT_ADD_MFC,
 				 &tmp_oil->oil, sizeof(tmp_oil->oil));
 	}
@@ -1161,7 +1160,7 @@ static int pim_upstream_mroute_update(struct channel_oil *c_oil,
 {
 	char buf[1000];
 
-	if (*oil_parent(c_oil) >= MAXVIFS) {
+	if (*oil_incoming_vif(c_oil) >= MAXVIFS) {
 		/* the c_oil cannot be installed as a mroute yet */
 		if (PIM_DEBUG_MROUTE)
 			zlog_debug(
@@ -1208,19 +1207,19 @@ int pim_upstream_mroute_add(struct channel_oil *c_oil, const char *name)
 
 	iif = pim_upstream_get_mroute_iif(c_oil, name);
 
-	if (*oil_parent(c_oil) != iif) {
-		*oil_parent(c_oil) = iif;
+	if (*oil_incoming_vif(c_oil) != iif) {
+		*oil_incoming_vif(c_oil) = iif;
 		if (pim_addr_is_any(*oil_origin(c_oil)) &&
 				c_oil->up)
 			pim_upstream_all_sources_iif_update(c_oil->up);
 	} else {
-		*oil_parent(c_oil) = iif;
+		*oil_incoming_vif(c_oil) = iif;
 	}
 
 	return pim_upstream_mroute_update(c_oil, name);
 }
 
-/* Look for IIF changes and update the dateplane entry only if the IIF
+/* Look for IIF changes and update the dataplane entry only if the IIF
  * has changed.
  */
 int pim_upstream_mroute_iif_update(struct channel_oil *c_oil, const char *name)
@@ -1229,11 +1228,11 @@ int pim_upstream_mroute_iif_update(struct channel_oil *c_oil, const char *name)
 	char buf[1000];
 
 	iif = pim_upstream_get_mroute_iif(c_oil, name);
-	if (*oil_parent(c_oil) == iif) {
+	if (*oil_incoming_vif(c_oil) == iif) {
 		/* no change */
 		return 0;
 	}
-	*oil_parent(c_oil) = iif;
+	*oil_incoming_vif(c_oil) = iif;
 
 	if (pim_addr_is_any(*oil_origin(c_oil)) &&
 			c_oil->up)
@@ -1258,10 +1257,10 @@ void pim_static_mroute_iif_update(struct channel_oil *c_oil,
 				int input_vif_index,
 				const char *name)
 {
-	if (*oil_parent(c_oil) == input_vif_index)
+	if (*oil_incoming_vif(c_oil) == input_vif_index)
 		return;
 
-	*oil_parent(c_oil) = input_vif_index;
+	*oil_incoming_vif(c_oil) = input_vif_index;
 	if (input_vif_index == MAXVIFS)
 		pim_mroute_del(c_oil, name);
 	else
@@ -1279,10 +1278,15 @@ int pim_mroute_del(struct channel_oil *c_oil, const char *name)
 	if (!c_oil->installed) {
 		if (PIM_DEBUG_MROUTE) {
 			char buf[1000];
-			zlog_debug(
-				"%s %s: vifi %d for route is %s not installed, do not need to send del req. ",
-				__FILE__, __func__, *oil_parent(c_oil),
-				pim_channel_oil_dump(c_oil, buf, sizeof(buf)));
+			struct interface *iifp =
+				pim_if_find_by_vif_index(pim, *oil_incoming_vif(
+								      c_oil));
+
+			zlog_debug("%s %s: incoming interface %s for route is %s not installed, do not need to send del req. ",
+				   __FILE__, __func__,
+				   iifp ? iifp->name : "Unknown",
+				   pim_channel_oil_dump(c_oil, buf,
+							sizeof(buf)));
 		}
 		return -2;
 	}
