@@ -1,28 +1,14 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Zebra connect library for SHARP
  * Copyright (C) Cumulus Networks, Inc.
  *               Donald Sharp
- *
- * This file is part of FRR.
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __SHARP_ZEBRA_H__
 #define __SHARP_ZEBRA_H__
 
 extern void sharp_zebra_init(void);
+extern void sharp_zebra_terminate(void);
 
 /* Add and delete extra zapi client sessions, for testing */
 int sharp_zclient_create(uint32_t session_id);
@@ -54,17 +40,22 @@ int sharp_install_lsps_helper(bool install_p, bool update_p,
 void sharp_opaque_send(uint32_t type, uint32_t proto, uint32_t instance,
 		       uint32_t session_id, uint32_t count);
 
-/* Send OPAQUE registration messages, using subtype 'type'. */
+/* Send OPAQUE registration or notification registration messages,
+ * for opaque subtype 'type'.
+ */
 void sharp_opaque_reg_send(bool is_reg, uint32_t proto, uint32_t instance,
 			   uint32_t session_id, uint32_t type);
 
+/* Register/unregister for opaque notifications from zebra about 'type'. */
+void sharp_zebra_opaque_notif_reg(bool is_reg, uint32_t type);
+
 extern void sharp_zebra_send_arp(const struct interface *ifp,
 				 const struct prefix *p);
 
 /* Register Link State Opaque messages */
 extern void sharp_zebra_register_te(void);
 
-extern void sharp_redistribute_vrf(struct vrf *vrf, int source);
+extern void sharp_redistribute_vrf(struct vrf *vrf, int source, bool turn_on);
 
 extern int sharp_zebra_srv6_manager_get_locator_chunk(const char *lname);
 extern int sharp_zebra_srv6_manager_release_locator_chunk(const char *lname);
@@ -75,4 +66,11 @@ extern void sharp_install_seg6local_route_helper(struct prefix *p,
 
 extern int sharp_zebra_send_interface_protodown(struct interface *ifp,
 						bool down);
+extern int sharp_zebra_send_tc_filter_rate(struct interface *ifp,
+					   const struct prefix *source,
+					   const struct prefix *destination,
+					   uint8_t ip_proto, uint16_t src_port,
+					   uint16_t dst_port, uint64_t rate);
+
+extern void sharp_zebra_register_neigh(vrf_id_t vrf_id, afi_t afi, bool reg);
 #endif
