@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# SPDX-License-Identifier: ISC
 
 #
 # test_evpn_mh.py
@@ -7,20 +8,6 @@
 # Cumulus Networks, Inc.
 # Anuradha Karuppiah
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 """
 test_evpn_mh.py: Testing EVPN multihoming
@@ -71,6 +58,8 @@ def build_topo(tgen):
 
     tgen.add_router("spine1")
     tgen.add_router("spine2")
+    tgen.add_router("leaf1")
+    tgen.add_router("leaf2")
     tgen.add_router("torm11")
     tgen.add_router("torm12")
     tgen.add_router("torm21")
@@ -84,88 +73,109 @@ def build_topo(tgen):
     # First switch is for a dummy interface (for local network)
 
     ##################### spine1 ########################
-    # spine1-eth0 is connected to torm11-eth0
+    # spine1-eth0 is connected to leaf1-eth0
     switch = tgen.add_switch("sw1")
     switch.add_link(tgen.gears["spine1"])
-    switch.add_link(tgen.gears["torm11"])
+    switch.add_link(tgen.gears["leaf1"])
 
-    # spine1-eth1 is connected to torm12-eth0
+    # spine1-eth1 is connected to leaf2-eth0
     switch = tgen.add_switch("sw2")
     switch.add_link(tgen.gears["spine1"])
-    switch.add_link(tgen.gears["torm12"])
+    switch.add_link(tgen.gears["leaf2"])
 
-    # spine1-eth2 is connected to torm21-eth0
+    # spine2-eth0 is connected to leaf1-eth1
     switch = tgen.add_switch("sw3")
-    switch.add_link(tgen.gears["spine1"])
-    switch.add_link(tgen.gears["torm21"])
-
-    # spine1-eth3 is connected to torm22-eth0
-    switch = tgen.add_switch("sw4")
-    switch.add_link(tgen.gears["spine1"])
-    switch.add_link(tgen.gears["torm22"])
-
-    ##################### spine2 ########################
-    # spine2-eth0 is connected to torm11-eth1
-    switch = tgen.add_switch("sw5")
     switch.add_link(tgen.gears["spine2"])
+    switch.add_link(tgen.gears["leaf1"])
+
+    # spine2-eth1 is connected to leaf2-eth1
+    switch = tgen.add_switch("sw4")
+    switch.add_link(tgen.gears["spine2"])
+    switch.add_link(tgen.gears["leaf2"])
+
+    ################## leaf1 ##########################
+    # leaf1-eth2 is connected to torm11-eth0
+    switch = tgen.add_switch("sw5")
+    switch.add_link(tgen.gears["leaf1"])
     switch.add_link(tgen.gears["torm11"])
 
-    # spine2-eth1 is connected to torm12-eth1
+    # leaf1-eth3 is connected to torm12-eth0
     switch = tgen.add_switch("sw6")
-    switch.add_link(tgen.gears["spine2"])
+    switch.add_link(tgen.gears["leaf1"])
     switch.add_link(tgen.gears["torm12"])
 
-    # spine2-eth2 is connected to torm21-eth1
+    # leaf1-eth4 is connected to torm21-eth0
     switch = tgen.add_switch("sw7")
-    switch.add_link(tgen.gears["spine2"])
+    switch.add_link(tgen.gears["leaf1"])
     switch.add_link(tgen.gears["torm21"])
 
-    # spine2-eth3 is connected to torm22-eth1
+    # leaf1-eth5 is connected to torm22-eth0
     switch = tgen.add_switch("sw8")
-    switch.add_link(tgen.gears["spine2"])
+    switch.add_link(tgen.gears["leaf1"])
+    switch.add_link(tgen.gears["torm22"])
+
+    ##################### leaf2 ########################
+    # leaf2-eth2 is connected to torm11-eth1
+    switch = tgen.add_switch("sw9")
+    switch.add_link(tgen.gears["leaf2"])
+    switch.add_link(tgen.gears["torm11"])
+
+    # leaf2-eth3 is connected to torm12-eth1
+    switch = tgen.add_switch("sw10")
+    switch.add_link(tgen.gears["leaf2"])
+    switch.add_link(tgen.gears["torm12"])
+
+    # leaf2-eth4 is connected to torm21-eth1
+    switch = tgen.add_switch("sw11")
+    switch.add_link(tgen.gears["leaf2"])
+    switch.add_link(tgen.gears["torm21"])
+
+    # leaf2-eth5 is connected to torm22-eth1
+    switch = tgen.add_switch("sw12")
+    switch.add_link(tgen.gears["leaf2"])
     switch.add_link(tgen.gears["torm22"])
 
     ##################### torm11 ########################
     # torm11-eth2 is connected to hostd11-eth0
-    switch = tgen.add_switch("sw9")
+    switch = tgen.add_switch("sw13")
     switch.add_link(tgen.gears["torm11"])
     switch.add_link(tgen.gears["hostd11"])
 
     # torm11-eth3 is connected to hostd12-eth0
-    switch = tgen.add_switch("sw10")
+    switch = tgen.add_switch("sw14")
     switch.add_link(tgen.gears["torm11"])
     switch.add_link(tgen.gears["hostd12"])
 
     ##################### torm12 ########################
     # torm12-eth2 is connected to hostd11-eth1
-    switch = tgen.add_switch("sw11")
+    switch = tgen.add_switch("sw15")
     switch.add_link(tgen.gears["torm12"])
     switch.add_link(tgen.gears["hostd11"])
 
     # torm12-eth3 is connected to hostd12-eth1
-    switch = tgen.add_switch("sw12")
+    switch = tgen.add_switch("sw16")
     switch.add_link(tgen.gears["torm12"])
     switch.add_link(tgen.gears["hostd12"])
 
     ##################### torm21 ########################
     # torm21-eth2 is connected to hostd21-eth0
-    switch = tgen.add_switch("sw13")
+    switch = tgen.add_switch("sw17")
     switch.add_link(tgen.gears["torm21"])
     switch.add_link(tgen.gears["hostd21"])
 
     # torm21-eth3 is connected to hostd22-eth0
-    switch = tgen.add_switch("sw14")
+    switch = tgen.add_switch("sw18")
     switch.add_link(tgen.gears["torm21"])
     switch.add_link(tgen.gears["hostd22"])
 
     ##################### torm22 ########################
     # torm22-eth2 is connected to hostd21-eth1
-    switch = tgen.add_switch("sw15")
+    switch = tgen.add_switch("sw19")
     switch.add_link(tgen.gears["torm22"])
     switch.add_link(tgen.gears["hostd21"])
 
     # torm22-eth3 is connected to hostd22-eth1
-    switch = tgen.add_switch("sw16")
+    switch = tgen.add_switch("sw20")
     switch.add_link(tgen.gears["torm22"])
     switch.add_link(tgen.gears["hostd22"])
 
@@ -254,7 +264,7 @@ def config_bridge(node):
     node.run("ip link set dev bridge type bridge mcast_snooping 0")
     node.run("ip link set dev bridge type bridge vlan_stats_enabled 1")
     node.run("ip link set dev bridge up")
-    node.run("/sbin/bridge vlan add vid 1000 dev bridge")
+    node.run("/sbin/bridge vlan add vid 1000 dev bridge self")
 
 
 def config_vxlan(node, node_ip):
@@ -604,7 +614,7 @@ def ping_anycast_gw(tgen):
         "--interface=" + intf,
         'Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst="{}")'.format(ipaddr),
     ]
-    for name in ("hostd11", "hostd21"):
+    for name in ("hostd11", "hostd21", "hostd12", "hostd22"):
         host = tgen.net.hosts[name]
         _, stdout, _ = host.cmd_status(ping_cmd, warn=False, stderr=subprocess.STDOUT)
         stdout = stdout.strip()
