@@ -1,24 +1,11 @@
 #!/usr/bin/python
+# SPDX-License-Identifier: ISC
 
 #
 # Copyright (c) 2021 by VMware, Inc. ("VMware")
 # Used Copyright (c) 2018 by Network Device Education Foundation, Inc.
 # ("NetDEF") in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 
 """OSPF Basic Functionality Automation."""
@@ -50,7 +37,6 @@ from lib.common_config import (
     reset_config_on_routers,
     step,
     shutdown_bringup_interface,
-    topo_daemons,
 )
 from lib.topolog import logger
 from lib.topojson import build_topo_from_json, build_config_from_json
@@ -94,6 +80,7 @@ TESTCASES =
 
  """
 
+
 def setup_module(mod):
     """
     Sets up the pytest environment
@@ -112,12 +99,9 @@ def setup_module(mod):
     topo = tgen.json_topo
     # ... and here it calls Mininet initialization functions.
 
-    # get list of daemons needs to be started for this suite.
-    daemons = topo_daemons(tgen, topo)
-
     # Starting topology, create tmp files which are loaded to routers
     #  to start daemons and then start routers
-    start_topology(tgen, daemons)
+    start_topology(tgen)
 
     # Creating configuration from JSON
     build_config_from_json(tgen, topo)
@@ -127,7 +111,7 @@ def setup_module(mod):
         pytest.skip(tgen.errors)
 
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo)
-    assert ospf6_covergence is True, "setup_module :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "setup_module :Failed \n Error:  {}".format(
         ospf6_covergence
     )
 
@@ -157,6 +141,7 @@ def teardown_module(mod):
 # Test cases start here.
 # ##################################
 
+
 def test_ospf6_auth_trailer_tc1_md5(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -197,7 +182,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -229,13 +214,11 @@ def test_ospf6_auth_trailer_tc1_md5(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
-    step(
-        "Disable authentication on R2 "
-    )
+    step("Disable authentication on R2 ")
 
     r2_ospf6_auth = {
         "r2": {
@@ -245,7 +228,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
                         "hash-algo": "md5",
                         "key": "ospf6",
                         "key-id": "10",
-                        "del_action": True
+                        "del_action": True,
                     }
                 }
             }
@@ -261,7 +244,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -289,7 +272,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -304,7 +287,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -318,7 +301,7 @@ def test_ospf6_auth_trailer_tc1_md5(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -365,7 +348,7 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -397,13 +380,11 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
-    step(
-        "Disable authentication on R2 "
-    )
+    step("Disable authentication on R2 ")
 
     r2_ospf6_auth = {
         "r2": {
@@ -413,7 +394,7 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
                         "hash-algo": "hmac-sha-256",
                         "key": "ospf6",
                         "key-id": "10",
-                        "del_action": True
+                        "del_action": True,
                     }
                 }
             }
@@ -429,7 +410,7 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -457,7 +438,7 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -472,7 +453,7 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -486,12 +467,13 @@ def test_ospf6_auth_trailer_tc2_sha256(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -549,7 +531,7 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -579,25 +561,14 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
-    step(
-        "Disable authentication on R2 "
-    )
+    step("Disable authentication on R2 ")
 
     r2_ospf6_auth = {
-        "r2": {
-            "links": {
-                "r1": {
-                    "ospf6": {
-                        "keychain": "auth",
-                        "del_action": True
-                    }
-                }
-            }
-        }
+        "r2": {"links": {"r1": {"ospf6": {"keychain": "auth", "del_action": True}}}}
     }
     result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
@@ -609,7 +580,7 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -635,7 +606,7 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -650,7 +621,7 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -664,12 +635,13 @@ def test_ospf6_auth_trailer_tc3_keychain_md5(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -727,7 +699,7 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -757,25 +729,14 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
-    step(
-        "Disable authentication on R2 "
-    )
+    step("Disable authentication on R2 ")
 
     r2_ospf6_auth = {
-        "r2": {
-            "links": {
-                "r1": {
-                    "ospf6": {
-                        "keychain": "auth",
-                        "del_action": True
-                    }
-                }
-            }
-        }
+        "r2": {"links": {"r1": {"ospf6": {"keychain": "auth", "del_action": True}}}}
     }
     result = config_ospf6_interface(tgen, topo, r2_ospf6_auth)
     assert result is True, "Testcase {} :Failed \n Error: {}".format(tc_name, result)
@@ -787,7 +748,7 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=5
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -813,7 +774,7 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -828,7 +789,7 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
         "show ip ospf6 neighbor cmd."
     )
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut, expected=False)
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -842,12 +803,13 @@ def test_ospf6_auth_trailer_tc4_keychain_sha256(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -888,7 +850,7 @@ def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -925,7 +887,7 @@ def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -957,12 +919,13 @@ def test_ospf6_auth_trailer_tc5_md5_keymissmatch(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -1003,7 +966,7 @@ def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1035,7 +998,7 @@ def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1067,12 +1030,13 @@ def test_ospf6_auth_trailer_tc6_sha256_mismatch(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -1138,7 +1102,7 @@ def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1168,7 +1132,7 @@ def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1198,12 +1162,13 @@ def test_ospf6_auth_trailer_tc7_keychain_md5_missmatch(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
     """
     OSPFv3 Authentication Trailer - Verify ospfv3 authentication trailer
@@ -1269,7 +1234,7 @@ def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1299,7 +1264,7 @@ def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1329,12 +1294,13 @@ def test_ospf6_auth_trailer_tc8_keychain_sha256_missmatch(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc9_keychain_not_configured(request):
     """
     OSPFv3 Neighborship without Authentication Trailer -
@@ -1376,7 +1342,7 @@ def test_ospf6_auth_trailer_tc9_keychain_not_configured(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
@@ -1406,12 +1372,13 @@ def test_ospf6_auth_trailer_tc9_keychain_not_configured(request):
     ospf6_covergence = verify_ospf6_neighbor(
         tgen, topo, dut=dut, expected=False, retry_timeout=3
     )
-    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is not True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 def test_ospf6_auth_trailer_tc10_no_auth_trailer(request):
     """
     OSPFv3 Neighborship without Authentication Trailer -
@@ -1435,12 +1402,13 @@ def test_ospf6_auth_trailer_tc10_no_auth_trailer(request):
 
     dut = "r2"
     ospf6_covergence = verify_ospf6_neighbor(tgen, topo, dut=dut)
-    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:" " {}".format(
+    assert ospf6_covergence is True, "Testcase {} :Failed \n Error:  {}".format(
         tc_name, ospf6_covergence
     )
 
     write_test_footer(tc_name)
 
+
 if __name__ == "__main__":
     args = ["-s"] + sys.argv[1:]
     sys.exit(pytest.main(args))
