@@ -1,24 +1,11 @@
 #!/usr/bin/python
+# SPDX-License-Identifier: ISC
 
 #
 # Copyright (c) 2020 by VMware, Inc. ("VMware")
 # Used Copyright (c) 2018 by Network Device Education Foundation,
 # Inc. ("NetDEF") in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 """
 
     -Verify static route ECMP functionality with 2 next hop
@@ -219,7 +206,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("Configure IBGP IPv4 peering between R2 and R3 router.")
         step("Configure redistribute static in BGP on R2 router")
@@ -274,7 +261,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -284,7 +271,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("Configure the static route with nexthop N1")
 
@@ -341,7 +328,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -351,7 +338,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("Configure the static route with nexthop N2")
         input_dict_4 = {
@@ -384,7 +371,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         nh = NEXT_HOP_IP["nh1"][addr_type]
         result = verify_rib(
@@ -398,7 +385,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -408,7 +395,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -423,7 +410,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -434,20 +421,20 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         dut = "r3"
         result = verify_bgp_rib(tgen, addr_type, dut, input_dict_4)
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Route is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError: Route is  missing in RIB".format(tc_name)
 
         result = verify_rib(
             tgen, addr_type, dut, input_dict_4, protocol=protocol, expected=False
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -466,7 +453,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("Shut nexthop interface N2")
         intf = topo["routers"]["r2"]["links"]["r1-link1"]["interface"]
@@ -490,7 +477,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -502,20 +489,20 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         dut = "r3"
         result = verify_bgp_rib(tgen, addr_type, dut, input_dict_4)
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Route is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError: Route is  missing in RIB".format(tc_name)
 
         result = verify_rib(
             tgen, addr_type, dut, input_dict_4, protocol=protocol, expected=False
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -534,20 +521,20 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         dut = "r3"
         result = verify_bgp_rib(tgen, addr_type, dut, input_dict_4)
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Route is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError: Route is  missing in RIB".format(tc_name)
 
         result = verify_rib(
             tgen, addr_type, dut, input_dict_4, protocol=protocol, expected=False
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -567,13 +554,13 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         dut = "r3"
         result = verify_bgp_rib(tgen, addr_type, dut, input_dict_4)
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -582,7 +569,7 @@ def test_static_route_2nh_p0_tc_1_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Route is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError: Route is  still present in RIB".format(
             tc_name
         )
 
@@ -658,7 +645,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" "missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are missing in RIB".format(tc_name)
 
         rte2_nh2 = {
             "r2": {
@@ -686,7 +673,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
         step("Configure IBGP IPv4 peering between R2 and R3 router.")
         step("Explicit route is added in R3 for R2 nexthop rechability")
@@ -777,7 +764,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" "missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are missing in RIB".format(tc_name)
 
         rte2_nh2 = {
             "r2": {
@@ -796,7 +783,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
         step("Configure the static route with nexthop N1")
         rte1_nh1 = {
@@ -851,7 +838,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -861,7 +848,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("Configure the static route with nexthop N2")
         rte2_nh2 = {
@@ -901,7 +888,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -911,7 +898,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("No shut the nexthop interface N1")
         shutdown_bringup_interface(tgen, dut, intf, True)
@@ -927,7 +914,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("Shut nexthop interface N2")
         intf = topo["routers"]["r2"]["links"]["r1-link1"]["interface"]
@@ -951,7 +938,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" " still present in RIB".format(
+        ), "Testcase {} : Failed \nError:  routes are  still present in RIB".format(
             tc_name
         )
 
@@ -961,7 +948,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" " missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are  missing in RIB".format(tc_name)
 
         step("No shut nexthop interface N2")
         shutdown_bringup_interface(tgen, dut, intf, True)
@@ -989,7 +976,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" "missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are missing in RIB".format(tc_name)
 
         rte2_nh2 = {
             "r2": {
@@ -1017,7 +1004,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
         dut = "r3"
         protocol = "bgp"
@@ -1034,7 +1021,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
         dut = "r2"
         step("Reload the FRR router")
@@ -1066,14 +1053,14 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" "missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are missing in RIB".format(tc_name)
 
         dut = "r3"
         protocol = "bgp"
         result = verify_bgp_rib(tgen, addr_type, dut, rte1_nh1, next_hop=nh)
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" "missing in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are missing in RIB".format(tc_name)
 
         rte2_nh2 = {
             "r2": {
@@ -1101,14 +1088,14 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
         dut = "r3"
         protocol = "bgp"
         result = verify_bgp_rib(tgen, addr_type, dut, rte2_nh2, next_hop=nh)
         assert (
             result is True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
         result = verify_rib(
             tgen,
@@ -1122,7 +1109,7 @@ def test_static_route_2nh_admin_dist_p0_tc_2_ibgp(request):
         )
         assert (
             result is not True
-        ), "Testcase {} : Failed \nError: Routes is" "not active in RIB".format(tc_name)
+        ), "Testcase {} : Failed \nError:  routes are not active in RIB".format(tc_name)
 
     write_test_footer(tc_name)
 
