@@ -1,22 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Zebra dataplane plugin for DPDK based hw offload
  *
  * Copyright (C) 2021 Nvidia
  * Anuradha Karuppiah
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -624,11 +611,11 @@ static void zd_dpdk_port_init(void)
 		if (rte_flow_isolate(port_id, 1, &error)) {
 			if (IS_ZEBRA_DEBUG_DPLANE_DPDK)
 				zlog_debug(
-					"Flow isolate on port %u failed %d\n",
+					"Flow isolate on port %u failed %d",
 					port_id, error.type);
 		} else {
 			if (IS_ZEBRA_DEBUG_DPLANE_DPDK)
-				zlog_debug("Flow isolate on port %u\n",
+				zlog_debug("Flow isolate on port %u",
 					   port_id);
 		}
 		rc = rte_eth_dev_start(port_id);
@@ -658,7 +645,7 @@ static int zd_dpdk_init(void)
 	zd_dpdk_vty_init();
 
 	frr_with_privs (&zserv_privs) {
-		rc = rte_eal_init(ARRAY_SIZE(argv), argv);
+		rc = rte_eal_init(array_size(argv), argv);
 	}
 	if (rc < 0) {
 		zlog_warn("EAL init failed %s", rte_strerror(rte_errno));
@@ -707,7 +694,7 @@ static int zd_dpdk_finish(struct zebra_dplane_provider *prov, bool early)
 }
 
 
-static int zd_dpdk_plugin_init(struct thread_master *tm)
+static int zd_dpdk_plugin_init(struct event_loop *tm)
 {
 	int ret;
 
