@@ -1,26 +1,15 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Kernel communication using netlink interface.
  * Copyright (C) 1999 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+#include <fcntl.h>
 
 #ifdef HAVE_NETLINK
+#include <linux/netlink.h>
+#include <linux/rtnetlink.h>
+#include <linux/filter.h>
 
 #include "linklist.h"
 #include "if.h"
@@ -30,7 +19,7 @@
 #include "table.h"
 #include "memory.h"
 #include "rib.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "privs.h"
 #include "nexthop.h"
 #include "vrf.h"
@@ -50,6 +39,7 @@
 #include "zebra/tc_netlink.h"
 #include "zebra/netconf_netlink.h"
 #include "zebra/zebra_errors.h"
+#include "zebra/ge_netlink.h"
 
 #ifndef SO_RCVBUFFORCE
 #define SO_RCVBUFFORCE  (33)
@@ -91,40 +81,48 @@
  */
 #define NL_DEFAULT_BATCH_SEND_THRESHOLD (15 * NL_PKT_BUF_SIZE)
 
-static const struct message nlmsg_str[] = {{RTM_NEWROUTE, "RTM_NEWROUTE"},
-					   {RTM_DELROUTE, "RTM_DELROUTE"},
-					   {RTM_GETROUTE, "RTM_GETROUTE"},
-					   {RTM_NEWLINK, "RTM_NEWLINK"},
-					   {RTM_SETLINK, "RTM_SETLINK"},
-					   {RTM_DELLINK, "RTM_DELLINK"},
-					   {RTM_GETLINK, "RTM_GETLINK"},
-					   {RTM_NEWADDR, "RTM_NEWADDR"},
-					   {RTM_DELADDR, "RTM_DELADDR"},
-					   {RTM_GETADDR, "RTM_GETADDR"},
-					   {RTM_NEWNEIGH, "RTM_NEWNEIGH"},
-					   {RTM_DELNEIGH, "RTM_DELNEIGH"},
-					   {RTM_GETNEIGH, "RTM_GETNEIGH"},
-					   {RTM_NEWRULE, "RTM_NEWRULE"},
-					   {RTM_DELRULE, "RTM_DELRULE"},
-					   {RTM_GETRULE, "RTM_GETRULE"},
-					   {RTM_NEWNEXTHOP, "RTM_NEWNEXTHOP"},
-					   {RTM_DELNEXTHOP, "RTM_DELNEXTHOP"},
-					   {RTM_GETNEXTHOP, "RTM_GETNEXTHOP"},
-					   {RTM_NEWNETCONF, "RTM_NEWNETCONF"},
-					   {RTM_DELNETCONF, "RTM_DELNETCONF"},
-					   {RTM_NEWTUNNEL, "RTM_NEWTUNNEL"},
-					   {RTM_DELTUNNEL, "RTM_DELTUNNEL"},
-					   {RTM_GETTUNNEL, "RTM_GETTUNNEL"},
-					   {RTM_NEWQDISC, "RTM_NEWQDISC"},
-					   {RTM_DELQDISC, "RTM_DELQDISC"},
-					   {RTM_GETQDISC, "RTM_GETQDISC"},
-					   {RTM_NEWTCLASS, "RTM_NEWTCLASS"},
-					   {RTM_DELTCLASS, "RTM_DELTCLASS"},
-					   {RTM_GETTCLASS, "RTM_GETTCLASS"},
-					   {RTM_NEWTFILTER, "RTM_NEWTFILTER"},
-					   {RTM_DELTFILTER, "RTM_DELTFILTER"},
-					   {RTM_GETTFILTER, "RTM_GETTFILTER"},
-					   {0}};
+static const struct message nlmsg_str[] = {
+	{ RTM_NEWROUTE, "RTM_NEWROUTE" },
+	{ RTM_DELROUTE, "RTM_DELROUTE" },
+	{ RTM_GETROUTE, "RTM_GETROUTE" },
+	{ RTM_NEWLINK, "RTM_NEWLINK" },
+	{ RTM_SETLINK, "RTM_SETLINK" },
+	{ RTM_DELLINK, "RTM_DELLINK" },
+	{ RTM_GETLINK, "RTM_GETLINK" },
+	{ RTM_NEWADDR, "RTM_NEWADDR" },
+	{ RTM_DELADDR, "RTM_DELADDR" },
+	{ RTM_GETADDR, "RTM_GETADDR" },
+	{ RTM_NEWNEIGH, "RTM_NEWNEIGH" },
+	{ RTM_DELNEIGH, "RTM_DELNEIGH" },
+	{ RTM_GETNEIGH, "RTM_GETNEIGH" },
+	{ RTM_NEWRULE, "RTM_NEWRULE" },
+	{ RTM_DELRULE, "RTM_DELRULE" },
+	{ RTM_GETRULE, "RTM_GETRULE" },
+	{ RTM_NEWNEXTHOP, "RTM_NEWNEXTHOP" },
+	{ RTM_DELNEXTHOP, "RTM_DELNEXTHOP" },
+	{ RTM_GETNEXTHOP, "RTM_GETNEXTHOP" },
+	{ RTM_NEWNETCONF, "RTM_NEWNETCONF" },
+	{ RTM_DELNETCONF, "RTM_DELNETCONF" },
+	{ RTM_NEWTUNNEL, "RTM_NEWTUNNEL" },
+	{ RTM_DELTUNNEL, "RTM_DELTUNNEL" },
+	{ RTM_GETTUNNEL, "RTM_GETTUNNEL" },
+	{ RTM_NEWQDISC, "RTM_NEWQDISC" },
+	{ RTM_DELQDISC, "RTM_DELQDISC" },
+	{ RTM_GETQDISC, "RTM_GETQDISC" },
+	{ RTM_NEWTCLASS, "RTM_NEWTCLASS" },
+	{ RTM_DELTCLASS, "RTM_DELTCLASS" },
+	{ RTM_GETTCLASS, "RTM_GETTCLASS" },
+	{ RTM_NEWTFILTER, "RTM_NEWTFILTER" },
+	{ RTM_DELTFILTER, "RTM_DELTFILTER" },
+	{ RTM_GETTFILTER, "RTM_GETTFILTER" },
+	{ RTM_NEWVLAN, "RTM_NEWVLAN" },
+	{ RTM_DELVLAN, "RTM_DELVLAN" },
+	{ RTM_GETVLAN, "RTM_GETVLAN" },
+	{ RTM_NEWCHAIN, "RTM_NEWCHAIN" },
+	{ RTM_DELCHAIN, "RTM_DELCHAIN" },
+	{ RTM_GETCHAIN, "RTM_GETCHAIN" },
+	{ 0 }
+};
 
 static const struct message rtproto_str[] = {
 	{RTPROT_REDIRECT, "redirect"},
@@ -168,7 +166,7 @@ static const struct message rttype_str[] = {{RTN_UNSPEC, "none"},
 					    {RTN_XRESOLVE, "resolver"},
 					    {0}};
 
-extern struct thread_master *master;
+extern struct event_loop *master;
 
 extern struct zebra_privs_t zserv_privs;
 
@@ -202,13 +200,13 @@ struct nl_batch {
 
 	const struct zebra_dplane_info *zns;
 
-	struct dplane_ctx_q ctx_list;
+	struct dplane_ctx_list_head ctx_list;
 
 	/*
 	 * Pointer to the queue of completed contexts outbound back
 	 * towards the dataplane module.
 	 */
-	struct dplane_ctx_q *ctx_out_q;
+	struct dplane_ctx_list_head *ctx_out_q;
 };
 
 int netlink_config_write_helper(struct vty *vty)
@@ -313,7 +311,7 @@ static const char *group2str(uint32_t group)
 /* Make socket for Linux netlink interface. */
 static int netlink_socket(struct nlsock *nl, unsigned long groups,
 			  uint32_t ext_groups[], uint8_t ext_group_size,
-			  ns_id_t ns_id)
+			  ns_id_t ns_id, int nl_family)
 {
 	int ret;
 	struct sockaddr_nl snl;
@@ -321,7 +319,7 @@ static int netlink_socket(struct nlsock *nl, unsigned long groups,
 	int namelen;
 
 	frr_with_privs(&zserv_privs) {
-		sock = ns_socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE, ns_id);
+		sock = ns_socket(AF_NETLINK, SOCK_RAW, nl_family, ns_id);
 		if (sock < 0) {
 			zlog_err("Can't open %s socket: %s", nl->name,
 				 safe_strerror(errno));
@@ -410,7 +408,7 @@ static int netlink_information_fetch(struct nlmsghdr *h, ns_id_t ns_id,
 	case RTM_NEWLINK:
 		return netlink_link_change(h, ns_id, startup);
 	case RTM_DELLINK:
-		return netlink_link_change(h, ns_id, startup);
+		return 0;
 	case RTM_NEWNEIGH:
 	case RTM_DELNEIGH:
 	case RTM_GETNEIGH:
@@ -423,6 +421,25 @@ static int netlink_information_fetch(struct nlmsghdr *h, ns_id_t ns_id,
 		return netlink_nexthop_change(h, ns_id, startup);
 	case RTM_DELNEXTHOP:
 		return netlink_nexthop_change(h, ns_id, startup);
+	case RTM_NEWQDISC:
+	case RTM_DELQDISC:
+		return netlink_qdisc_change(h, ns_id, startup);
+	case RTM_NEWTCLASS:
+	case RTM_DELTCLASS:
+		return netlink_tclass_change(h, ns_id, startup);
+	case RTM_NEWTFILTER:
+	case RTM_DELTFILTER:
+		return netlink_tfilter_change(h, ns_id, startup);
+	case RTM_NEWVLAN:
+		return netlink_vlan_change(h, ns_id, startup);
+	case RTM_DELVLAN:
+		return netlink_vlan_change(h, ns_id, startup);
+
+	/* Messages we may receive, but ignore */
+	case RTM_NEWCHAIN:
+	case RTM_DELCHAIN:
+	case RTM_GETCHAIN:
+		return 0;
 
 	/* Messages handled in the dplane thread */
 	case RTM_NEWADDR:
@@ -473,6 +490,7 @@ static int dplane_netlink_information_fetch(struct nlmsghdr *h, ns_id_t ns_id,
 
 	case RTM_NEWLINK:
 	case RTM_DELLINK:
+		return netlink_link_change(h, ns_id, startup);
 
 	default:
 		break;
@@ -481,9 +499,9 @@ static int dplane_netlink_information_fetch(struct nlmsghdr *h, ns_id_t ns_id,
 	return 0;
 }
 
-static void kernel_read(struct thread *thread)
+static void kernel_read(struct event *thread)
 {
-	struct zebra_ns *zns = (struct zebra_ns *)THREAD_ARG(thread);
+	struct zebra_ns *zns = (struct zebra_ns *)EVENT_ARG(thread);
 	struct zebra_dplane_info dp_info;
 
 	/* Capture key info from ns struct */
@@ -492,8 +510,8 @@ static void kernel_read(struct thread *thread)
 	netlink_parse_info(netlink_information_fetch, &zns->netlink, &dp_info,
 			   5, false);
 
-	thread_add_read(zrouter.master, kernel_read, zns, zns->netlink.sock,
-			&zns->t_netlink);
+	event_add_read(zrouter.master, kernel_read, zns, zns->netlink.sock,
+		       &zns->t_netlink);
 }
 
 /*
@@ -696,6 +714,12 @@ bool nl_attr_put32(struct nlmsghdr *n, unsigned int maxlen, int type,
 	return nl_attr_put(n, maxlen, type, &data, sizeof(uint32_t));
 }
 
+bool nl_attr_put64(struct nlmsghdr *n, unsigned int maxlen, int type,
+		   uint64_t data)
+{
+	return nl_attr_put(n, maxlen, type, &data, sizeof(uint64_t));
+}
+
 struct rtattr *nl_attr_nest(struct nlmsghdr *n, unsigned int maxlen, int type)
 {
 	struct rtattr *nest = NLMSG_TAIL(n);
@@ -733,58 +757,6 @@ void nl_attr_rtnh_end(struct nlmsghdr *n, struct rtnexthop *rtnh)
 	rtnh->rtnh_len = (uint8_t *)NLMSG_TAIL(n) - (uint8_t *)rtnh;
 }
 
-bool nl_rta_put(struct rtattr *rta, unsigned int maxlen, int type,
-		const void *data, int alen)
-{
-	struct rtattr *subrta;
-	int len = RTA_LENGTH(alen);
-
-	if (RTA_ALIGN(rta->rta_len) + RTA_ALIGN(len) > maxlen) {
-		zlog_err("ERROR max allowed bound %d exceeded for rtattr",
-			 maxlen);
-		return false;
-	}
-	subrta = (struct rtattr *)(((char *)rta) + RTA_ALIGN(rta->rta_len));
-	subrta->rta_type = type;
-	subrta->rta_len = len;
-	if (alen)
-		memcpy(RTA_DATA(subrta), data, alen);
-	rta->rta_len = NLMSG_ALIGN(rta->rta_len) + RTA_ALIGN(len);
-
-	return true;
-}
-
-bool nl_rta_put16(struct rtattr *rta, unsigned int maxlen, int type,
-		  uint16_t data)
-{
-	return nl_rta_put(rta, maxlen, type, &data, sizeof(uint16_t));
-}
-
-bool nl_rta_put64(struct rtattr *rta, unsigned int maxlen, int type,
-		  uint64_t data)
-{
-	return nl_rta_put(rta, maxlen, type, &data, sizeof(uint64_t));
-}
-
-struct rtattr *nl_rta_nest(struct rtattr *rta, unsigned int maxlen, int type)
-{
-	struct rtattr *nest = RTA_TAIL(rta);
-
-	if (nl_rta_put(rta, maxlen, type, NULL, 0))
-		return NULL;
-
-	nest->rta_type |= NLA_F_NESTED;
-
-	return nest;
-}
-
-int nl_rta_nest_end(struct rtattr *rta, struct rtattr *nest)
-{
-	nest->rta_len = (uint8_t *)RTA_TAIL(rta) - (uint8_t *)nest;
-
-	return rta->rta_len;
-}
-
 const char *nl_msg_type_to_str(uint16_t msg_type)
 {
 	return lookup_msg(nlmsg_str, msg_type, "");
@@ -1071,7 +1043,8 @@ static int netlink_parse_error(const struct nlsock *nl, struct nlmsghdr *h,
 				   nl_msg_type_to_str(msg_type), msg_type,
 				   err->msg.nlmsg_seq, err->msg.nlmsg_pid);
 	} else {
-		if ((msg_type != RTM_GETNEXTHOP) || !startup)
+		if ((msg_type != RTM_GETNEXTHOP && msg_type != RTM_GETVLAN) ||
+		    !startup)
 			flog_err(EC_ZEBRA_UNEXPECTED_MESSAGE,
 				 "%s error: %s, type=%s(%u), seq=%u, pid=%u",
 				 nl->name, safe_strerror(-errnum),
@@ -1161,7 +1134,6 @@ int netlink_parse_info(int (*filter)(struct nlmsghdr *, ns_id_t, int),
 					h->nlmsg_type, h->nlmsg_len,
 					h->nlmsg_seq, h->nlmsg_pid);
 
-
 			/*
 			 * Ignore messages that maybe sent from
 			 * other actors besides the kernel
@@ -1257,6 +1229,33 @@ int netlink_talk(int (*filter)(struct nlmsghdr *, ns_id_t, int startup),
 	return netlink_talk_info(filter, n, &dp_info, startup);
 }
 
+/*
+ * Synchronous version of netlink_talk_info. Converts args to suit the
+ * common version, which is suitable for both sync and async use.
+ */
+int ge_netlink_talk(int (*filter)(struct nlmsghdr *, ns_id_t, int startup),
+		    struct nlmsghdr *n, struct zebra_ns *zns, bool startup)
+{
+	struct zebra_dplane_info dp_info;
+
+	if (zns->ge_netlink_cmd.sock < 0)
+		return -1;
+
+	/* Increment sequence number before capturing snapshot of ns socket
+	 * info.
+	 */
+	zns->ge_netlink_cmd.seq = zebra_router_get_next_sequence();
+
+	/* Capture info in intermediate info struct */
+	dp_info.ns_id = zns->ns_id;
+
+	dp_info.is_cmd = true;
+	dp_info.sock = zns->ge_netlink_cmd.sock;
+	dp_info.seq = zns->ge_netlink_cmd.seq;
+
+	return netlink_talk_info(filter, n, &dp_info, startup);
+}
+
 /* Issue request message to kernel via netlink socket. GET messages
  * are issued through this interface.
  */
@@ -1281,18 +1280,15 @@ int netlink_request(struct nlsock *nl, void *req)
 	return 0;
 }
 
-static int nl_batch_read_resp(struct nl_batch *bth)
+static int nl_batch_read_resp(struct nl_batch *bth, struct nlsock *nl)
 {
 	struct nlmsghdr *h;
 	struct sockaddr_nl snl;
 	struct msghdr msg = {};
 	int status, seq;
-	struct nlsock *nl;
 	struct zebra_dplane_ctx *ctx;
 	bool ignore_msg;
 
-	nl = kernel_netlink_nlsock_lookup(bth->zns->sock);
-
 	msg.msg_name = (void *)&snl;
 	msg.msg_namelen = sizeof(snl);
 
@@ -1437,10 +1433,11 @@ static void nl_batch_reset(struct nl_batch *bth)
 	bth->msgcnt = 0;
 	bth->zns = NULL;
 
-	TAILQ_INIT(&(bth->ctx_list));
+	dplane_ctx_q_init(&(bth->ctx_list));
 }
 
-static void nl_batch_init(struct nl_batch *bth, struct dplane_ctx_q *ctx_out_q)
+static void nl_batch_init(struct nl_batch *bth,
+			  struct dplane_ctx_list_head *ctx_out_q)
 {
 	/*
 	 * If the size of the buffer has changed, free and then allocate a new
@@ -1484,7 +1481,7 @@ static void nl_batch_send(struct nl_batch *bth)
 			err = true;
 
 		if (!err) {
-			if (nl_batch_read_resp(bth) == -1)
+			if (nl_batch_read_resp(bth, nl) == -1)
 				err = true;
 		}
 	}
@@ -1622,6 +1619,7 @@ static enum netlink_msg_status nl_put_msg(struct nl_batch *bth,
 	case DPLANE_OP_IPSET_DELETE:
 	case DPLANE_OP_IPSET_ENTRY_ADD:
 	case DPLANE_OP_IPSET_ENTRY_DELETE:
+	case DPLANE_OP_STARTUP_STAGE:
 		return FRR_NETLINK_ERROR;
 
 	case DPLANE_OP_GRE_SET:
@@ -1640,23 +1638,33 @@ static enum netlink_msg_status nl_put_msg(struct nl_batch *bth,
 	case DPLANE_OP_INTF_DELETE:
 		return netlink_put_intf_update_msg(bth, ctx);
 
-	case DPLANE_OP_TC_INSTALL:
-	case DPLANE_OP_TC_UPDATE:
-	case DPLANE_OP_TC_DELETE:
-		return netlink_put_tc_update_msg(bth, ctx);
+	case DPLANE_OP_TC_QDISC_INSTALL:
+	case DPLANE_OP_TC_QDISC_UNINSTALL:
+		return netlink_put_tc_qdisc_update_msg(bth, ctx);
+	case DPLANE_OP_TC_CLASS_ADD:
+	case DPLANE_OP_TC_CLASS_DELETE:
+	case DPLANE_OP_TC_CLASS_UPDATE:
+		return netlink_put_tc_class_update_msg(bth, ctx);
+	case DPLANE_OP_TC_FILTER_ADD:
+	case DPLANE_OP_TC_FILTER_DELETE:
+	case DPLANE_OP_TC_FILTER_UPDATE:
+		return netlink_put_tc_filter_update_msg(bth, ctx);
+
+	case DPLANE_OP_SRV6_ENCAP_SRCADDR_SET:
+		return netlink_put_sr_tunsrc_set_msg(bth, ctx);
 	}
 
 	return FRR_NETLINK_ERROR;
 }
 
-void kernel_update_multi(struct dplane_ctx_q *ctx_list)
+void kernel_update_multi(struct dplane_ctx_list_head *ctx_list)
 {
 	struct nl_batch batch;
 	struct zebra_dplane_ctx *ctx;
-	struct dplane_ctx_q handled_list;
+	struct dplane_ctx_list_head handled_list;
 	enum netlink_msg_status res;
 
-	TAILQ_INIT(&handled_list);
+	dplane_ctx_q_init(&handled_list);
 	nl_batch_init(&batch, &handled_list);
 
 	while (true) {
@@ -1687,7 +1695,7 @@ void kernel_update_multi(struct dplane_ctx_q *ctx_list)
 
 	nl_batch_send(&batch);
 
-	TAILQ_INIT(ctx_list);
+	dplane_ctx_q_init(ctx_list);
 	dplane_ctx_list_append(ctx_list, &handled_list);
 }
 
@@ -1744,7 +1752,7 @@ void kernel_init(struct zebra_ns *zns)
 {
 	uint32_t groups, dplane_groups, ext_groups;
 #if defined SOL_NETLINK
-	int one, ret;
+	int one, ret, grp;
 #endif
 
 	/*
@@ -1755,17 +1763,17 @@ void kernel_init(struct zebra_ns *zns)
 	 * keeping track of all the different values would
 	 * lead to confusion, so we need to convert the
 	 * RTNLGRP_XXX to a bit position for ourself
+	 *
+	 *
+	 * NOTE: If the bit is >= 32, you must use setsockopt(). Those
+	 * groups are added further below after SOL_NETLINK is verified to
+	 * exist.
 	 */
-	groups = RTMGRP_LINK                   |
-		RTMGRP_IPV4_ROUTE              |
-		RTMGRP_IPV4_IFADDR             |
-		RTMGRP_IPV6_ROUTE              |
-		RTMGRP_IPV6_IFADDR             |
-		RTMGRP_IPV4_MROUTE             |
-		RTMGRP_NEIGH                   |
-		((uint32_t) 1 << (RTNLGRP_IPV4_RULE - 1)) |
-		((uint32_t) 1 << (RTNLGRP_IPV6_RULE - 1)) |
-		((uint32_t) 1 << (RTNLGRP_NEXTHOP - 1));
+	groups = RTMGRP_IPV4_ROUTE | RTMGRP_IPV6_ROUTE | RTMGRP_IPV4_MROUTE |
+		 RTMGRP_NEIGH | ((uint32_t)1 << (RTNLGRP_IPV4_RULE - 1)) |
+		 ((uint32_t)1 << (RTNLGRP_IPV6_RULE - 1)) |
+		 ((uint32_t)1 << (RTNLGRP_NEXTHOP - 1)) |
+		 ((uint32_t)1 << (RTNLGRP_TC - 1));
 
 	dplane_groups = (RTMGRP_LINK            |
 			 RTMGRP_IPV4_IFADDR     |
@@ -1780,8 +1788,8 @@ void kernel_init(struct zebra_ns *zns)
 	snprintf(zns->netlink.name, sizeof(zns->netlink.name),
 		 "netlink-listen (NS %u)", zns->ns_id);
 	zns->netlink.sock = -1;
-	if (netlink_socket(&zns->netlink, groups, &ext_groups, 1, zns->ns_id) <
-	    0) {
+	if (netlink_socket(&zns->netlink, groups, &ext_groups, 1, zns->ns_id,
+			   NETLINK_ROUTE) < 0) {
 		zlog_err("Failure to create %s socket",
 			 zns->netlink.name);
 		exit(-1);
@@ -1792,7 +1800,8 @@ void kernel_init(struct zebra_ns *zns)
 	snprintf(zns->netlink_cmd.name, sizeof(zns->netlink_cmd.name),
 		 "netlink-cmd (NS %u)", zns->ns_id);
 	zns->netlink_cmd.sock = -1;
-	if (netlink_socket(&zns->netlink_cmd, 0, 0, 0, zns->ns_id) < 0) {
+	if (netlink_socket(&zns->netlink_cmd, 0, 0, 0, zns->ns_id,
+			   NETLINK_ROUTE) < 0) {
 		zlog_err("Failure to create %s socket",
 			 zns->netlink_cmd.name);
 		exit(-1);
@@ -1805,7 +1814,8 @@ void kernel_init(struct zebra_ns *zns)
 		 sizeof(zns->netlink_dplane_out.name), "netlink-dp (NS %u)",
 		 zns->ns_id);
 	zns->netlink_dplane_out.sock = -1;
-	if (netlink_socket(&zns->netlink_dplane_out, 0, 0, 0, zns->ns_id) < 0) {
+	if (netlink_socket(&zns->netlink_dplane_out, 0, 0, 0, zns->ns_id,
+			   NETLINK_ROUTE) < 0) {
 		zlog_err("Failure to create %s socket",
 			 zns->netlink_dplane_out.name);
 		exit(-1);
@@ -1819,7 +1829,7 @@ void kernel_init(struct zebra_ns *zns)
 		 zns->ns_id);
 	zns->netlink_dplane_in.sock = -1;
 	if (netlink_socket(&zns->netlink_dplane_in, dplane_groups, 0, 0,
-			   zns->ns_id) < 0) {
+			   zns->ns_id, NETLINK_ROUTE) < 0) {
 		zlog_err("Failure to create %s socket",
 			 zns->netlink_dplane_in.name);
 		exit(-1);
@@ -1827,12 +1837,37 @@ void kernel_init(struct zebra_ns *zns)
 
 	kernel_netlink_nlsock_insert(&zns->netlink_dplane_in);
 
+	/* Generic Netlink socket. */
+	snprintf(zns->ge_netlink_cmd.name, sizeof(zns->ge_netlink_cmd.name),
+		 "generic-netlink-cmd (NS %u)", zns->ns_id);
+	zns->ge_netlink_cmd.sock = -1;
+	if (netlink_socket(&zns->ge_netlink_cmd, 0, 0, 0, zns->ns_id,
+			   NETLINK_GENERIC) < 0) {
+		zlog_warn("Failure to create %s socket",
+			  zns->ge_netlink_cmd.name);
+	}
+
+	if (zns->ge_netlink_cmd.sock >= 0)
+		kernel_netlink_nlsock_insert(&zns->ge_netlink_cmd);
+
 	/*
 	 * SOL_NETLINK is not available on all platforms yet
 	 * apparently.  It's in bits/socket.h which I am not
 	 * sure that we want to pull into our build system.
 	 */
 #if defined SOL_NETLINK
+
+	/*
+	 * setsockopt multicast group subscriptions that don't fit in nl_groups
+	 */
+	grp = RTNLGRP_BRVLAN;
+	ret = setsockopt(zns->netlink.sock, SOL_NETLINK, NETLINK_ADD_MEMBERSHIP,
+			 &grp, sizeof(grp));
+
+	if (ret < 0)
+		zlog_notice(
+			"Registration for RTNLGRP_BRVLAN Membership failed : %d %s",
+			errno, safe_strerror(errno));
 	/*
 	 * Let's tell the kernel that we want to receive extended
 	 * ACKS over our command socket(s)
@@ -1853,6 +1888,15 @@ void kernel_init(struct zebra_ns *zns)
 		zlog_notice("Registration for extended dp ACK failed : %d %s",
 			    errno, safe_strerror(errno));
 
+	if (zns->ge_netlink_cmd.sock >= 0) {
+		one = 1;
+		ret = setsockopt(zns->ge_netlink_cmd.sock, SOL_NETLINK,
+				 NETLINK_EXT_ACK, &one, sizeof(one));
+		if (ret < 0)
+			zlog_err("Registration for extended generic netlink cmd ACK failed : %d %s",
+				 errno, safe_strerror(errno));
+	}
+
 	/*
 	 * Trim off the payload of the original netlink message in the
 	 * acknowledgment. This option is available since Linux 4.2, so if
@@ -1885,12 +1929,22 @@ void kernel_init(struct zebra_ns *zns)
 			 zns->netlink_dplane_in.name, safe_strerror(errno),
 			 errno);
 
+	if (zns->ge_netlink_cmd.sock >= 0) {
+		if (fcntl(zns->ge_netlink_cmd.sock, F_SETFL, O_NONBLOCK) < 0)
+			zlog_err("Can't set %s socket error: %s(%d)",
+				 zns->ge_netlink_cmd.name, safe_strerror(errno),
+				 errno);
+	}
+
 	/* Set receive buffer size if it's set from command line */
 	if (rcvbufsize) {
 		netlink_recvbuf(&zns->netlink, rcvbufsize);
 		netlink_recvbuf(&zns->netlink_cmd, rcvbufsize);
 		netlink_recvbuf(&zns->netlink_dplane_out, rcvbufsize);
 		netlink_recvbuf(&zns->netlink_dplane_in, rcvbufsize);
+
+		if (zns->ge_netlink_cmd.sock >= 0)
+			netlink_recvbuf(&zns->ge_netlink_cmd, rcvbufsize);
 	}
 
 	/* Set filter for inbound sockets, to exclude events we've generated
@@ -1905,10 +1959,12 @@ void kernel_init(struct zebra_ns *zns)
 
 	zns->t_netlink = NULL;
 
-	thread_add_read(zrouter.master, kernel_read, zns,
-			zns->netlink.sock, &zns->t_netlink);
+	event_add_read(zrouter.master, kernel_read, zns, zns->netlink.sock,
+		       &zns->t_netlink);
 
 	rt_netlink_init();
+
+	ge_netlink_init(zns);
 }
 
 /* Helper to clean up an nlsock */
@@ -1925,7 +1981,7 @@ static void kernel_nlsock_fini(struct nlsock *nls)
 
 void kernel_terminate(struct zebra_ns *zns, bool complete)
 {
-	THREAD_OFF(zns->t_netlink);
+	EVENT_OFF(zns->t_netlink);
 
 	kernel_nlsock_fini(&zns->netlink);
 
@@ -1933,11 +1989,16 @@ void kernel_terminate(struct zebra_ns *zns, bool complete)
 
 	kernel_nlsock_fini(&zns->netlink_dplane_in);
 
+	kernel_nlsock_fini(&zns->ge_netlink_cmd);
+
 	/* During zebra shutdown, we need to leave the dataplane socket
 	 * around until all work is done.
 	 */
-	if (complete)
+	if (complete) {
 		kernel_nlsock_fini(&zns->netlink_dplane_out);
+
+		XFREE(MTYPE_NL_BUF, nl_batch_tx_buf);
+	}
 }
 
 /*
