@@ -1,23 +1,11 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Zebra NS collector and notifier for Network NameSpaces
  * Copyright (C) 2017 6WIND
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+#include <fcntl.h>
 
 #ifdef HAVE_NETLINK
 #ifdef HAVE_NETNS
@@ -30,7 +18,7 @@
 #include <sys/inotify.h>
 #include <sys/stat.h>
 
-#include "thread.h"
+#include "frrevent.h"
 #include "ns.h"
 #include "command.h"
 #include "memory.h"
@@ -54,18 +42,18 @@
 #define ZEBRA_NS_POLLING_MAX_RETRIES  200
 
 DEFINE_MTYPE_STATIC(ZEBRA, NETNS_MISC, "ZebraNetNSInfo");
-static struct thread *zebra_netns_notify_current;
+static struct event *zebra_netns_notify_current;
 
 struct zebra_netns_info {
 	const char *netnspath;
 	unsigned int retries;
 };
 
-static void zebra_ns_ready_read(struct thread *t);
+static void zebra_ns_ready_read(struct event *t);
 static void zebra_ns_notify_create_context_from_entry_name(const char *name);
 static int zebra_ns_continue_read(struct zebra_netns_info *zns_info,
 				  int stop_retry);
-static void zebra_ns_notify_read(struct thread *t);
+static void zebra_ns_notify_read(struct event *t);
 
 static struct vrf *vrf_handler_create(struct vty *vty, const char *vrfname)
 {
@@ -146,9 +134,9 @@ static int zebra_ns_continue_read(struct zebra_netns_info *zns_info,
 		XFREE(MTYPE_NETNS_MISC, zns_info);
 		return 0;
 	}
-	thread_add_timer_msec(zrouter.master, zebra_ns_ready_read,
-			      (void *)zns_info, ZEBRA_NS_POLLING_INTERVAL_MSEC,
-			      NULL);
+	event_add_timer_msec(zrouter.master, zebra_ns_ready_read,
+			     (void *)zns_info, ZEBRA_NS_POLLING_INTERVAL_MSEC,
+			     NULL);
 	return 0;
 }
 
@@ -242,9 +230,9 @@ static bool zebra_ns_notify_is_default_netns(const char *name)
 	return false;
 }
 
-static void zebra_ns_ready_read(struct thread *t)
+static void zebra_ns_ready_read(struct event *t)
 {
-	struct zebra_netns_info *zns_info = THREAD_ARG(t);
+	struct zebra_netns_info *zns_info = EVENT_ARG(t);
 	const char *netnspath;
 	int err, stop_retry = 0;
 
@@ -293,15 +281,16 @@ static void zebra_ns_ready_read(struct thread *t)
 	zebra_ns_continue_read(zns_info, 1);
 }
 
-static void zebra_ns_notify_read(struct thread *t)
+static void zebra_ns_notify_read(struct event *t)
 {
-	int fd_monitor = THREAD_FD(t);
+	int fd_monitor = EVENT_FD(t);
 	struct inotify_event *event;
 	char buf[BUFSIZ];
 	ssize_t len;
+	char event_name[NAME_MAX + 1];
 
-	thread_add_read(zrouter.master, zebra_ns_notify_read, NULL, fd_monitor,
-			&zebra_netns_notify_current);
+	event_add_read(zrouter.master, zebra_ns_notify_read, NULL, fd_monitor,
+		       &zebra_netns_notify_current);
 	len = read(fd_monitor, buf, sizeof(buf));
 	if (len < 0) {
 		flog_err_sys(EC_ZEBRA_NS_NOTIFY_READ,
@@ -331,11 +320,41 @@ static void zebra_ns_notify_read(struct thread *t)
 			break;
 		}
 
+		/*
+		 * Coverity Scan extra steps to satisfy `STRING_NULL` warning:
+		 * - Make sure event name is present by checking `len != 0`
+		 * - Event name length must be at most `NAME_MAX + 1`
+		 *   (null byte inclusive)
+		 * - Copy event name to a stack buffer to make sure it
+		 *   includes the null byte. `event->name` includes at least
+		 *   one null byte and `event->len` accounts the null bytes,
+		 *   so the operation after `memcpy` will look like a
+		 *   truncation to satisfy Coverity Scan null byte ending.
+		 *
+		 *   Example:
+		 *   if `event->name` is `abc\0` and `event->len` is 4,
+		 *   `memcpy` will copy the 4 bytes and then we set the
+		 *   null byte again at the position 4.
+		 *
+		 * For more information please read inotify(7) man page.
+		 */
+		if (event->len == 0)
+			continue;
+
+		if (event->len > sizeof(event_name)) {
+			flog_err(EC_ZEBRA_NS_NOTIFY_READ,
+				 "NS notify error: unexpected big event name");
+			break;
+		}
+
+		memcpy(event_name, event->name, event->len);
+		event_name[event->len - 1] = 0;
+
 		if (event->mask & IN_DELETE) {
-			zebra_ns_delete(event->name);
+			zebra_ns_delete(event_name);
 			continue;
 		}
-		netnspath = ns_netns_pathname(NULL, event->name);
+		netnspath = ns_netns_pathname(NULL, event_name);
 		if (!netnspath)
 			continue;
 		netnspath = XSTRDUP(MTYPE_NETNS_MISC, netnspath);
@@ -343,8 +362,8 @@ static void zebra_ns_notify_read(struct thread *t)
 				    sizeof(struct zebra_netns_info));
 		netnsinfo->retries = ZEBRA_NS_POLLING_MAX_RETRIES;
 		netnsinfo->netnspath = netnspath;
-		thread_add_timer_msec(zrouter.master, zebra_ns_ready_read,
-				      (void *)netnsinfo, 0, NULL);
+		event_add_timer_msec(zrouter.master, zebra_ns_ready_read,
+				     (void *)netnsinfo, 0, NULL);
 	}
 }
 
@@ -409,8 +428,8 @@ void zebra_ns_notify_init(void)
 			     "NS notify watch: failed to add watch (%s)",
 			     safe_strerror(errno));
 	}
-	thread_add_read(zrouter.master, zebra_ns_notify_read, NULL, fd_monitor,
-			&zebra_netns_notify_current);
+	event_add_read(zrouter.master, zebra_ns_notify_read, NULL, fd_monitor,
+		       &zebra_netns_notify_current);
 }
 
 void zebra_ns_notify_close(void)
@@ -424,7 +443,7 @@ void zebra_ns_notify_close(void)
 		fd = zebra_netns_notify_current->u.fd;
 
 	if (zebra_netns_notify_current->master != NULL)
-		THREAD_OFF(zebra_netns_notify_current);
+		EVENT_OFF(zebra_netns_notify_current);
 
 	/* auto-removal of notify items */
 	if (fd > 0)
