@@ -1,23 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Zebra NS header
  * Copyright (C) 2016 Cumulus Networks, Inc.
  *                    Donald Sharp
- *
- * This file is part of Quagga.
- *
- * Quagga is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * Quagga is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #if !defined(__ZEBRA_NS_H__)
 #define __ZEBRA_NS_H__
@@ -33,6 +18,8 @@ extern "C" {
 #endif
 
 #ifdef HAVE_NETLINK
+#include <linux/netlink.h>
+
 /* Socket interface to kernel */
 struct nlsock {
 	int sock;
@@ -61,7 +48,9 @@ struct zebra_ns {
 	 */
 	struct nlsock netlink_dplane_out;
 	struct nlsock netlink_dplane_in;
-	struct thread *t_netlink;
+	struct event *t_netlink;
+
+	struct nlsock ge_netlink_cmd; /* command channel for generic netlink */
 #endif
 
 	struct route_table *if_table;
@@ -81,7 +70,8 @@ int zebra_ns_early_shutdown(struct ns *ns,
 int zebra_ns_final_shutdown(struct ns *ns,
 			    void *param_in __attribute__((unused)),
 			    void **param_out __attribute__((unused)));
-int zebra_ns_config_write(struct vty *vty, struct ns *ns);
+
+void zebra_ns_startup_continue(struct zebra_dplane_ctx *ctx);
 
 #ifdef __cplusplus
 }
